(*
Copyright 2009 Christian Sternagel, René Thiemann, Sarah Winkler, Harald Zankl

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
*)
theory DpTransImpl imports DpFramework UsableRules
begin

types ('f,'v)dp = "('f shp,'v)rule" 
types ('f,'v)dpL = "('f,'v)dp list"


fun computeID :: "('f,'v)trsL \<Rightarrow> ('f shp,'v)trsL"
where "computeID R = map (\<lambda> lr. (plain (fst lr), plain (snd lr))) R"

lemma computeIDSound[simp] : "set (computeID R) = ID (set R)"
proof (induct R, simp add: ID_def)
  case (Cons lr R)
  have "ID (set (lr # R)) = {(s,t). \<exists> l r. s = plain l \<and> t = plain r \<and> (l, r) \<in> set (lr # R)}" by (simp add: ID_def)
  also have "\<dots> = {(s,t). \<exists> l r. s = plain l \<and> t = plain r \<and> (l, r) \<in> set R} \<union> {(s,t). \<exists> l r. s = plain l \<and> t = plain r \<and> (l, r) = lr}" by auto
  also have "\<dots> = set (computeID R) \<union> {(s,t). \<exists> l r. s = plain l \<and> t = plain r \<and> (l, r) = lr}" using Cons by (auto simp: ID_def)
  finally show ?case by (cases lr, auto)
qed

declare computeID.simps[simp del]


definition checkWfTRS :: "('f :: sshow,'v :: sshow)trsL \<Rightarrow> sstring list monad" 
where "checkWfTRS R \<equiv>  (
  checkAllM (\<lambda> lr. checkB (\<not> is_var (fst lr)) 
       (sConcat [''the left side of rule '', showRule lr,  '' is a variable''])) R 
  \<guillemotright>
  checkAllM (\<lambda> lr. (checkSubset (vars_list (snd lr)) (vars_list (fst lr))
         \<leftarrow> (\<lambda> x. sConcat [''the variable '', sshow x,  '' occurs only in the right-hand side of rule '', showRule lr]))) R
                      ) \<leftarrow> (\<lambda> e. [''the TRS is not well-formed'',e])"

lemma checkWfTRSSound: assumes "isOK(checkWfTRS R)" 
  shows "wf_trs (set R)"
proof -
  from assms 
  show ?thesis unfolding checkWfTRS_def wf_trs_def
  proof (simp, clarify)
    fix l r
    assume mem: "(l,r) \<in> set R"
      and "\<forall> lr \<in> set R. \<not> is_var (fst lr)"
      and c2: "\<forall> lr \<in> set R. vars (snd lr) \<subseteq> vars (fst lr)"
    hence c1: "\<not> is_var (fst (l,r))" by auto
    from mem c2 have "vars (snd (l,r)) \<subseteq> vars (fst (l,r))" by force
    with c1 show "(\<exists>f ts. l = Fun f ts) \<and> vars r \<subseteq> vars l" 
      by (auto, cases l, auto)
  qed
qed

lemma checkWfTRSComplete: assumes " \<not> isOK(checkWfTRS R)"
  shows "\<not> wf_trs (set R)"
proof 
  assume wftrs: "wf_trs (set R)"
  hence "\<forall> lr \<in> (set R). \<not> is_var (fst lr)" unfolding wf_trs_def by force
  hence one: "\<And> s. isOK (checkAllM (\<lambda> lr. checkB (\<not> is_var (fst lr)) s) R)" by auto

  from wftrs have "\<forall> lr \<in> set R. set (vars_list (snd lr)) \<subseteq> set (vars_list (fst lr))" unfolding wf_trs_def by force
  hence two: "isOK (checkAllM (\<lambda> lr. (checkSubset (vars_list (snd lr)) (vars_list (fst lr)))) R)" by auto 

  from one two have "isOK(checkWfTRS R)" unfolding checkWfTRS_def by auto
  with assms show False ..
qed

lemma checkWfTRSSoundComplete[simp]: "isOK(checkWfTRS R) = wf_trs (set R)"
  using checkWfTRSSound checkWfTRSComplete by auto

fun checkSupterm :: "('f,'v) term \<Rightarrow> ('f,'v) term \<Rightarrow> bool"
  where "checkSupterm (Var x) _ = False"
     |  "checkSupterm (Fun f ts) s = (s mem ts \<or> (any (\<lambda> t. checkSupterm t s) ts))" 

lemma supteqp_var_imp_eq : fixes t :: "('f,'v) term" and x :: 'v
  assumes st: "Var x \<unrhd> t" shows "t = Var x"
proof -
  from st supteqp_imp_ctxt obtain C where  "C\<langle>t\<rangle> = Var x" by force
  thus ?thesis by (cases C, auto)
qed
    
lemma checkSuptermSound[simp] : "checkSupterm t s = (t \<rhd> s)"
proof (induct t arbitrary: s, simp, rule impI)
  fix x s
  assume "Var x \<unrhd> s"
  from supteqp_var_imp_eq[OF this] show "Var x = s" by simp
next
  case (Fun f ts)
  have "checkSupterm (Fun f ts) s = (s mem ts \<or> (any (\<lambda> t. checkSupterm t s) ts))" by simp
  also have "\<dots> = (s mem ts \<or> (\<exists> t. t mem ts \<and> (\<lambda> t. checkSupterm t s) t))" using any_mem_conv  by auto
  also have "\<dots> = (s \<in> set ts \<or> (\<exists> t \<in> set ts. checkSupterm t s))" by (auto simp:mem_iff)
  also have "\<dots> = (s \<in> set ts \<or> (\<exists> t \<in> set ts. t \<rhd> s))" using Fun by auto
  also have "\<dots> = (Fun f ts \<rhd> s)" 
  proof (rule iffI)
    assume "s \<in> set ts \<or> (\<exists> t \<in> set ts. t \<rhd> s)" (is "?l \<or> ?r")
    thus "Fun f ts \<rhd> s"
    proof
      assume ?l
      thus ?thesis ..
    next
      assume ?r
      from this obtain t where "t \<in> set ts" and "t \<rhd> s" by auto      
      thus ?thesis .. 
    qed
  next
    assume "Fun f ts \<rhd> s"
    thus "s \<in> set ts \<or> (\<exists> t \<in> set ts. t \<rhd> s)"       
      by (rule suptp.cases, auto)
  qed
  finally show ?case .
qed

declare checkSupterm.simps[simp del]

fun computeDPsOfTerm :: "'f list \<Rightarrow> ('f,'v) term \<Rightarrow> ('f shp,'v) term \<Rightarrow> ('f,'v)term \<Rightarrow> ('f shp,'v)rule list"
where "computeDPsOfTerm D l lshp (Var _) = []"
    | "computeDPsOfTerm D l lshp (Fun f ts) = ((if ((f mem D) \<and> \<not> (checkSupterm l (Fun f ts))) then [(lshp,sharp(Fun f ts))] else []) @ 
  (foldr (\<lambda> t dps. list_union (computeDPsOfTerm D l lshp t) dps) ts []))"

fun computeDPsOfRule :: "'f list \<Rightarrow> ('f,'v)rule \<Rightarrow> ('f shp,'v)rule list"
where "computeDPsOfRule D (Fun f ss,r) = computeDPsOfTerm D (Fun f ss) (sharp (Fun f ss)) r"
    | "computeDPsOfRule D (Var x,r) = []"


lemma computeDPsOfRuleSound : 
   "set (computeDPsOfRule D (Fun g ss, r)) = {(s,t). s = sharp (Fun g ss) \<and> (\<exists> h us. t = sharp (Fun h us) \<and> r \<unrhd> (Fun h us) \<and> h mem D \<and> \<not> (Fun g ss) \<rhd> Fun h us)}"
proof (simp del:sharp.simps)
  let ?l = "Fun g ss"
  let ?sl = "sharp ?l"
  let ?cond = "\<lambda> r. {?sl} \<times> {sharp (Fun h us) | h us. r \<unrhd> Fun h us \<and> h mem D \<and> (?l \<unrhd> Fun h us \<longrightarrow> g = h \<and> ss = us)}"
  let ?condS = "\<lambda> r. {?sl} \<times> {sharp (Fun h us) | h us. r \<rhd> Fun h us \<and> h mem D \<and> (?l \<unrhd> Fun h us \<longrightarrow> g = h \<and> ss = us)}"
  let ?condEQ = "\<lambda> r. {?sl} \<times> {sharp (Fun h us) | h us. r = Fun h us \<and> h mem D \<and> (?l \<unrhd> Fun h us \<longrightarrow> g = h \<and> ss = us)}"
  show "set (computeDPsOfTerm D ?l ?sl r) = ?cond r"
  proof (induct r)
    case (Var x)
    have varSubt: "\<And> h us. \<not> Var x \<unrhd> Fun h us"
    proof 
      fix h us
      assume "Var x \<unrhd> Fun h us"
      hence "Fun h us = Var x" by (rule supteqp_var_imp_eq) 
      thus False by blast
    qed
    show ?case 
    proof auto
      fix h us
      assume "Var x \<unrhd> Fun h us" with varSubt show False .. 
    next
      assume "Var x \<unrhd> Fun g ss" with varSubt show False ..
    qed
  next
    case (Fun f ts)
    have "set (computeDPsOfTerm D ?l ?sl (Fun f ts)) = set ((if ((f mem D) \<and> \<not> (checkSupterm ?l (Fun f ts))) then [(?sl,sharp(Fun f ts))] else []) @ 
      (foldr (\<lambda> t dps. list_union (computeDPsOfTerm D ?l ?sl t) dps) ts []))" (is "_ = set (_ @ (?fold ts))") by auto
    also have "\<dots> = set ((if ((f mem D) \<and> \<not> (?l \<rhd> (Fun f ts))) then [(?sl,sharp(Fun f ts))] else []) @ ?fold ts)" (is "_ = set (?elem @ _)")
      using checkSuptermSound[where ?t = ?l and ?s = "Fun f ts"] by auto
    also have "\<dots> = ?condS (Fun f ts) \<union> ?condEQ (Fun f ts)" 
    proof -
      have elem: "set ?elem = ?condEQ (Fun f ts)" by (cases "f mem D", cases "Fun g ss \<rhd> Fun f ts", auto)
      from Fun have "set (?fold ts) = ?condS (Fun f ts)" 
      proof -
	have "set (?fold ts) = (\<Union> t \<in> set ts. set (computeDPsOfTerm D ?l ?sl t))" by (induct ts, auto simp: list_union_sound)
	also have "\<dots> = (\<Union> t \<in> set ts. ?cond t)" using Fun by simp
	also have "\<dots> = ?condS (Fun f ts)" 
	proof -
	  show ?thesis 
	  proof (simp del:sharp.simps, rule set_ext)
	    fix slhus
	    show "(slhus \<in> (\<Union>t\<in>set ts.
                 {?sl} \<times>
                 {sharp (Fun h us) |h us.
                  t \<unrhd> Fun h us \<and>
                  h mem D \<and> (?l \<unrhd> Fun h us \<longrightarrow> g = h \<and> ss = us)}))
              = (slhus \<in> {?sl} \<times>  {sharp (Fun h us) |h us.
              Fun f ts \<unrhd> Fun h us \<and>
              (f = h \<longrightarrow> ts \<noteq> us) \<and>
              h mem D \<and> (?l \<unrhd> Fun h us \<longrightarrow> g = h \<and> ss = us)})"
	      (is "(slhus \<in> ?left ts) = (slhus \<in> ?right ts)")
	    proof (cases slhus)
	      case (Pair sl hus)
	      show ?thesis
	      proof (cases hus)
		case (Fun h us)
		have "((sl, Fun h us) \<in> ?left ts) = ((sl, Fun h us) \<in> ?right ts)" 		 
		proof
		  assume "(sl, Fun h us) \<in> ?left ts"
		  from this obtain t where t: "t \<in> set ts" and 
		    slhus: "(sl, Fun h us) \<in> {?sl} \<times>
        {sharp (Fun h us) |h us. t \<unrhd> Fun h us \<and> h mem D \<and> (?l \<unrhd> Fun h us \<longrightarrow> g = h \<and> ss = us)}" by force
		  from slhus have sl: "sl = ?sl" by simp
		  from slhus obtain hh uus where hus: "Fun h us = sharp (Fun hh uus)" and thhuus: "t \<unrhd> Fun hh uus" and h: "hh mem D" and l: "?l \<unrhd> Fun hh uus \<longrightarrow> g = hh \<and> ss = uus" by force
		  from t thhuus have "Fun f ts \<rhd> Fun hh uus"
		  proof auto
		    assume set: "t \<in> set uus" and sub: "t \<unrhd> Fun hh uus"
		    from set have "Fun hh uus \<rhd> t" by auto
		    hence one: "size (Fun hh uus) > size t" by (rule suptp_size)
		    from sub have "size t \<ge> size (Fun hh uus)" proof
		      assume "t = Fun hh uus" thus ?thesis by simp
		    next
		      assume "t \<rhd> Fun hh uus" hence "size t > size (Fun hh uus)" by (rule suptp_size)
		      thus ?thesis by simp
		    qed
		    with one show False by simp
		  qed
		  with hus h l 
		  have "Fun h us \<in> {sharp (Fun h us) |h us. Fun f ts \<unrhd> Fun h us \<and> (f = h \<longrightarrow> ts \<noteq> us) \<and> h mem D \<and> (?l \<unrhd> Fun h us \<longrightarrow> g = h \<and> ss = us)}"
		    by auto
		  with sl show "(sl, Fun h us) \<in> ?right ts" by auto 		    
		next
		  assume slhus: "(sl, Fun h us) \<in> ?right ts"
		  from this have sl: "sl = ?sl" by auto
		  from slhus obtain hh uus where hus: "Fun h us = sharp(Fun hh uus)" and fts: "Fun f ts \<unrhd> Fun hh uus" 
		    and fhts: "(f = hh \<longrightarrow> ts \<noteq> uus)" and hh: "hh mem D" and l: "?l \<unrhd> Fun hh uus \<longrightarrow> g = hh \<and> ss = uus"  by force
		  from fts and fhts have "Fun f ts \<rhd> Fun hh uus" by simp
		  thus "(sl, Fun h us) \<in> ?left ts"
		  proof (rule suptp.cases)
		    fix t tts ff
		    assume "Fun f ts = Fun ff tts" and hhuust: "Fun hh uus = t" and "t \<in> set tts"
		    hence tts: "t \<in> set ts" by simp
		    with hhuust hus l sl hh
		    have "Fun h us = sharp(Fun hh uus) \<and> t \<unrhd> Fun hh uus \<and> hh mem D \<and> (?l \<unrhd> Fun hh uus \<longrightarrow> g = hh \<and> ss = uus)" (is "?setCond hh uus")
		      by auto
		    hence "\<exists> hh uus. ?setCond hh uus" by force
		    hence "Fun h us \<in> {sharp (Fun h us) |h us. t \<unrhd> Fun h us \<and> h mem D \<and> (Fun g ss \<unrhd> Fun h us \<longrightarrow> g = h \<and> ss = us)}" by simp
		    with sl tts show ?thesis by force
		  next
		    fix t tts hus ff
		    assume "Fun f ts = Fun ff tts" and hhuust: "Fun hh uus = hus" and "t \<in> set tts" and thu: "t \<rhd> hus"		    
		    hence tts: "t \<in> set ts" by simp
		    with hhuust thu have thhu: "t \<rhd> Fun hh uus" by simp
		    have "Fun h us \<in> {sharp (Fun h us) |h us. t \<unrhd> Fun h us \<and> h mem D \<and> (?l \<unrhd> Fun h us \<longrightarrow> g = h \<and> ss = us)}"
		      using thhu hus l hh by auto
		    with tts sl show ?thesis by auto
		  qed
		qed
		with Fun Pair show ?thesis by simp
	      next
		case (Var x)
		with Pair show ?thesis by auto 
	      qed
	    qed
	  qed
	qed
	finally show ?thesis . 
      qed             	
      with elem show ?thesis by auto 
    qed
    also have "\<dots> = ?cond (Fun f ts)" using supteqp_iff_suptp_or_eq[where ?s = "Fun f ts"] by (simp del:sharp.simps, blast)
    finally show ?case .
  qed
qed



fun computeDPs :: "('f,'v)trsL \<Rightarrow> ('f shp,'v)trsL"
where "computeDPs r = (let D = defs_list r in foldr (\<lambda> lr dps. list_union(computeDPsOfRule D lr) dps) r [])"

lemma computeDPsSound[simp] : fixes R :: "('f,'v)trsL"
  assumes wf: "wf_trs (set R)"
  shows "set (computeDPs R) = Trs.DP (set R)" 
proof -
  let ?D = "defs_list R :: 'f list"
  from defs_list_sound[where ?R = R] have D: "\<forall> f. (f mem ?D = defined (set R) f)" by auto
  have main: "\<forall> D. (set (foldr (\<lambda> lr dps. list_union (computeDPsOfRule D lr) dps) R []) = 
    {(s,t). \<exists>l r h us. s = sharp l \<and> t = sharp(Fun h us) \<and> (l,r) \<in> set R \<and> r \<unrhd> (Fun h us) \<and> h mem D \<and> \<not> l \<rhd> Fun h us})" 
  proof
    fix D
    from wf show "set (foldr (\<lambda> lr dps. list_union (computeDPsOfRule D lr) dps) R []) = 
      {(s,t). \<exists>l r h us. s = sharp l \<and> t = sharp(Fun h us) \<and> (l,r) \<in> set R \<and> r \<unrhd> (Fun h us) \<and> h mem D \<and> \<not> l \<rhd> Fun h us}" (is "set (?fold R) = ?DP R")
    proof (induct R, simp)
      case (Cons lr R)
      show ?case
      proof (cases lr)
	case (Pair l r)
	show ?thesis
	proof (cases l)
	  case (Var x)
	  with `wf_trs (set (lr # R))` Pair have False by (unfold wf_trs_def, auto)  thus ?thesis ..
	next
	  case (Fun f ss)
	  have "set (?fold (lr # R)) = set (list_union (computeDPsOfRule D lr) (?fold R))" by simp
	  also have "\<dots> = (set (computeDPsOfRule D lr) \<union> set (?fold R))" using list_union_sound[where ?xs = "computeDPsOfRule D lr" and ?ys = "?fold R"] 
	    by blast
	  also have "\<dots> = (set (computeDPsOfRule D lr) \<union> ?DP R)" using Cons by (auto simp: wf_trs_def)
	  also have "\<dots> = (set (computeDPsOfRule D (Fun f ss,r)) \<union> ?DP R)" using Fun Pair by simp
	  also have "\<dots> = ({(s,t). s = sharp (Fun f ss) \<and> (\<exists> h us. t = sharp (Fun h us) \<and> r \<unrhd> (Fun h us) \<and> h mem D \<and> \<not> (Fun f ss) \<rhd> Fun h us)} \<union> ?DP R)" (is "_ = (?elems \<union> _)")
	  proof -
	    have "set (computeDPsOfRule D (Fun f ss, r)) = ?elems" 
	      using computeDPsOfRuleSound[where ?D = D and ?r = r and ?g = f and ?ss = ss] by blast
	    thus ?thesis by auto
	  qed
	  also have "\<dots> = (?DP [(Fun f ss,r)] \<union> ?DP R)" (is "_ = ?dps \<union> _") 
	  proof -
	    have "?elems = ?dps" by (simp only:set.simps, blast)
	    thus ?thesis by auto
	  qed
	  also have "\<dots> = ?DP [lr] \<union> ?DP R" using Pair Fun by simp
	  also have "\<dots> = ?DP (lr # R)" by (simp, blast)
	  finally show ?thesis .
	qed
      qed
    qed
  qed

  hence "set (computeDPs R) =
      {(s, t).
       \<exists>l r h us.
          s = sharp l \<and>
          t = sharp (Fun h us) \<and>
          (l, r) \<in> set R \<and> r \<unrhd> Fun h us \<and> h mem ?D \<and> \<not> l \<rhd> Fun h us}" by (simp add: Let_def)
  also have "\<dots> = Trs.DP (set R)" using D by (unfold Trs.DP_def, blast) 
  finally show ?thesis .    
qed  

declare computeDPs.simps[simp del]

      
definition checkDPs :: "('f :: sshow, 'v :: sshow)trsL \<Rightarrow> ('f,'v)dpL \<Rightarrow> sstring list monad"
  where "checkDPs r P  \<equiv> (
(checkWfTRS r   
\<guillemotright>    (checkSubset (computeDPs r) P 
            \<leftarrow> (\<lambda> dp. [sConcat [''the DP '', showRule dp ,  '' does not appear in the DP problem'']] )))
  \<leftarrow> (\<lambda> s.  ''the DP-transformation is not applied correctly.'' #  s))
" 


lemma checkDPsSound[simp] : assumes check: "isOK(checkDPs r P)"
  shows "finite_dpp (set P, ID (set r)) \<Longrightarrow> SN (rstep (set r))"
proof -
  assume fin: "finite_dpp (set P, ID (set r))"
  show "SN (rstep (set r))"
  proof -
    from check have wf: "wf_trs (set r)" 
      and dp: "Trs.DP (set r) \<subseteq> set P" 
      unfolding checkDPs_def by auto
    let ?DP = "Trs.DP (set r)"
    from fin have "finite_dpp (set P, ID (set r))"  by simp
    hence fin2: "finite_dpp (set P \<inter> ?DP, ID (set r))" by (unfold finite_dpp_def, auto)
    from dp have "?DP = set P \<inter> ?DP" by auto
    with fin2 have "finite_dpp (initial_dpp (set r))" by simp
    with wf show "?thesis" by (rule finite_ini_imp_SN)
  qed
qed

definition checkNotWellFormed :: "('f :: sshow, 'v :: sshow)trsL \<Rightarrow> string list monad"
  where "checkNotWellFormed R \<equiv> checkB (\<not> isOK (checkWfTRS R)) [''The TRS is well formed'']"

lemma SN_imp_wf_trs: 
  fixes R::"('f,'v)trs"
  assumes "SN(rstep R)"
  shows "wf_trs R"
proof (rule ccontr)
  assume "\<not> wf_trs R"
  from this obtain l r::"('f,'v)term" where lr: "(l,r) \<in> R" and nwf: "\<not> (\<exists> f ts. l = Fun f ts) \<or> \<not> vars l \<supseteq> vars r" (is "?lv \<or> ?fr")
    unfolding wf_trs_def by blast
  from rstep_contains_r lr have step: "(l,r) \<in> rstep R" by auto
  from nwf show False
  proof
    assume ?lv hence "is_var l" by (cases l, auto)
    with var_rewrite_imp_not_SN assms step show False by blast
  next
    assume ?fr 
    from lr have "(l,r) \<in> rstep_r_p_s R (l,r) \<epsilon> \<iota>" unfolding rstep_r_p_s_def by auto
    with free_right_rewrite_imp_not_SN `?fr` assms show False by blast
  qed
qed
  

lemma checkNotWellFormedSound[simp] : assumes check: "isOK(checkNotWellFormed R)"
  shows "\<not> SN (rstep (set R))"
proof -
  from assms have "\<not> wf_trs (set R)" unfolding checkNotWellFormed_def by auto
  with SN_imp_wf_trs show ?thesis by auto
qed
  

end
