{-# OPTIONS_GHC -fglasgow-exts #-}

module DpTransImpl where {

import Util;
import List;
import Term;
import Trs;
import HOL;
import ErrorMonad;
import Show;
import Eithera;

checkSupterm :: forall a b. (Eq a, Eq b) => Term a b -> Term a b -> Bool;
checkSupterm (Var x) uu = False;
checkSupterm (Fun f ts) s = member s ts || anya (\ t -> checkSupterm t s) ts;

computeDPsOfTerm ::
  forall a b.
    (Eq a,
      Eq b) => [a] ->
                 Term a b ->
                   Term (Shp a) b ->
                     Term a b -> [(Term (Shp a) b, Term (Shp a) b)];
computeDPsOfTerm d l lshp (Var uu) = [];
computeDPsOfTerm d l lshp (Fun f ts) =
  append
    (if member f d && not (checkSupterm l (Fun f ts))
      then [(lshp, sharpa (Fun f ts))] else [])
    (foldra (\ t -> list_union (computeDPsOfTerm d l lshp t)) ts []);

computeDPsOfRule ::
  forall a b.
    (Eq a,
      Eq b) => [a] ->
                 (Term a b, Term a b) -> [(Term (Shp a) b, Term (Shp a) b)];
computeDPsOfRule d (Fun f ss, r) =
  computeDPsOfTerm d (Fun f ss) (sharpa (Fun f ss)) r;
computeDPsOfRule d (Var x, r) = [];

computeDPs ::
  forall a b.
    (Eq a,
      Eq b) => [(Term a b, Term a b)] -> [(Term (Shp a) b, Term (Shp a) b)];
computeDPs r =
  let {
    d = defs_list r;
  } in foldra (\ lr -> list_union (computeDPsOfRule d lr)) r [];

checkWfTRS ::
  forall f v.
    (Sshow f, Eq v, Sshow v) => [(Term f v, Term f v)] -> Eithera [[Char]] ();
checkWfTRS r =
  catcha
    (bind (catcha
            (checkAllMI
              (\ lr ->
                checkB (not (is_var (fst lr)))
                  (concata
                    [['t', 'h', 'e', ' ', 'l', 'e', 'f', 't', ' ', 's', 'i',
                       'd', 'e', ' ', 'o', 'f', ' ', 'r', 'u', 'l', 'e', ' '],
                      showRule lr,
                      [' ', 'i', 's', ' ', 'a', ' ', 'v', 'a', 'r', 'i', 'a',
                        'b', 'l', 'e']]))
              r)
            (\ x -> errora (snd x)))
      (\ x ->
        catcha
          (checkAllMI
            (\ lr ->
              catcha
                (checkSubset (vars_term_list (snd lr))
                  (vars_term_list (fst lr)))
                (\ xa ->
                  errora
                    (concata
                      [['t', 'h', 'e', ' ', 'v', 'a', 'r', 'i', 'a', 'b', 'l',
                         'e', ' '],
                        sshow xa,
                        [' ', 'o', 'c', 'c', 'u', 'r', 's', ' ', 'o', 'n', 'l',
                          'y', ' ', 'i', 'n', ' ', 't', 'h', 'e', ' ', 'r', 'i',
                          'g', 'h', 't', '-', 'h', 'a', 'n', 'd', ' ', 's', 'i',
                          'd', 'e', ' ', 'o', 'f', ' ', 'r', 'u', 'l', 'e',
                          ' '],
                        showRule lr])))
            r)
          (\ xa -> errora (snd xa))))
    (\ x ->
      errora
        [['t', 'h', 'e', ' ', 'T', 'R', 'S', ' ', 'i', 's', ' ', 'n', 'o', 't',
           ' ', 'w', 'e', 'l', 'l', '-', 'f', 'o', 'r', 'm', 'e', 'd'],
          x]);

checkDPs ::
  forall f v.
    (Eq f, Sshow f, Eq v,
      Sshow v) => [(Term f v, Term f v)] ->
                    [(Term (Shp f) v, Term (Shp f) v)] -> Eithera [[Char]] ();
checkDPs r p =
  catcha
    (bind (checkWfTRS r)
      (\ x ->
        catcha (checkSubset (computeDPs r) p)
          (\ xa ->
            errora
              [concata
                 [['t', 'h', 'e', ' ', 'D', 'P', ' '], showRule xa,
                   [' ', 'd', 'o', 'e', 's', ' ', 'n', 'o', 't', ' ', 'a', 'p',
                     'p', 'e', 'a', 'r', ' ', 'i', 'n', ' ', 't', 'h', 'e', ' ',
                     'D', 'P', ' ', 'p', 'r', 'o', 'b', 'l', 'e', 'm']]])))
    (\ x ->
      errora
        (['t', 'h', 'e', ' ', 'D', 'P', '-', 't', 'r', 'a', 'n', 's', 'f', 'o',
           'r', 'm', 'a', 't', 'i', 'o', 'n', ' ', 'i', 's', ' ', 'n', 'o', 't',
           ' ', 'a', 'p', 'p', 'l', 'i', 'e', 'd', ' ', 'c', 'o', 'r', 'r', 'e',
           'c', 't', 'l', 'y', '.']
          : x));

computeID ::
  forall a b. [(Term a b, Term a b)] -> [(Term (Shp a) b, Term (Shp a) b)];
computeID r = mapa (\ lr -> (plaina (fst lr), plaina (snd lr))) r;

checkNotWellFormed ::
  forall f v.
    (Sshow f, Eq v, Sshow v) => [(Term f v, Term f v)] -> Eithera [[Char]] ();
checkNotWellFormed r =
  checkB (not (isValue (checkWfTRS r)))
    [['T', 'h', 'e', ' ', 'T', 'R', 'S', ' ', 'i', 's', ' ', 'w', 'e', 'l', 'l',
       ' ', 'f', 'o', 'r', 'm', 'e', 'd']];

}
