{-# OPTIONS_GHC -fglasgow-exts #-}

module RedPairImpl where {

import qualified List;
import qualified Show;
import qualified PolyOrder;
import qualified ErrorMonad;
import qualified Term;
import qualified Eithera;
import qualified Nat;
import qualified Orderings;
import qualified Integer;
import qualified Matrix;

data Arctic = MinInfty | Num_arc Integer;

data Redpair a = Nat_carrier [((a, Nat.Nat), (Nat.Nat, [Nat.Nat]))]
  | Int_carrier [((a, Nat.Nat), (Integer, [Integer]))]
  | Arctic_carrier [((a, Nat.Nat), (RedPairImpl.Arctic, [RedPairImpl.Arctic]))]
  | Int_mat_carrier Nat.Nat [((a, Nat.Nat), ([[Integer]], [[[Integer]]]))]
  | Arc_mat_carrier Nat.Nat
      [((a, Nat.Nat), ([[RedPairImpl.Arctic]], [[[RedPairImpl.Arctic]]]))];

arc_mat_name :: [Char];
arc_mat_name =
  ['m', 'a', 't', 'r', 'i', 'c', 'e', 's', ' ', 'o', 'v', 'e', 'r', ' ', 't',
    'h', 'e', ' ', 'a', 'r', 'c', 't', 'i', 'c', ' ', 's', 'e', 'm', 'i', '-',
    'r', 'i', 'n', 'g'];

int_mat_name :: [Char];
int_mat_name =
  ['m', 'a', 't', 'r', 'i', 'c', 'e', 's', ' ', 'o', 'v', 'e', 'r', ' ', 't',
    'h', 'e', ' ', 'n', 'a', 't', 'u', 'r', 'a', 'l', 's', ' ', 'w', 'i', 't',
    'h', ' ', 'n', 'e', 'g', 'a', 't', 'i', 'v', 'e', ' ', 'c', 'o', 'n', 's',
    't', 'a', 'n', 't', 's'];

pos_arctic :: RedPairImpl.Arctic -> Bool;
pos_arctic RedPairImpl.MinInfty = False;
pos_arctic (RedPairImpl.Num_arc n) = 0 <= n;

arctic_name :: [Char];
arctic_name =
  ['a', 'r', 'c', 't', 'i', 'c', ' ', 's', 'e', 'm', 'i', 'r', 'i', 'n', 'g'];

gt_arctic :: RedPairImpl.Arctic -> RedPairImpl.Arctic -> Bool;
gt_arctic uu RedPairImpl.MinInfty = True;
gt_arctic RedPairImpl.MinInfty (RedPairImpl.Num_arc uv) = False;
gt_arctic (RedPairImpl.Num_arc x) (RedPairImpl.Num_arc y) = y < x;

geq_arctic :: RedPairImpl.Arctic -> RedPairImpl.Arctic -> Bool;
geq_arctic xa RedPairImpl.MinInfty = True;
geq_arctic RedPairImpl.MinInfty (RedPairImpl.Num_arc uu) = False;
geq_arctic (RedPairImpl.Num_arc x) (RedPairImpl.Num_arc y) = y <= x;

mult_arctic :: RedPairImpl.Arctic -> RedPairImpl.Arctic -> RedPairImpl.Arctic;
mult_arctic RedPairImpl.MinInfty ya = RedPairImpl.MinInfty;
mult_arctic (RedPairImpl.Num_arc v) RedPairImpl.MinInfty = RedPairImpl.MinInfty;
mult_arctic (RedPairImpl.Num_arc x) (RedPairImpl.Num_arc y) =
  RedPairImpl.Num_arc (x + y);

plus_arctic :: RedPairImpl.Arctic -> RedPairImpl.Arctic -> RedPairImpl.Arctic;
plus_arctic RedPairImpl.MinInfty ya = ya;
plus_arctic (RedPairImpl.Num_arc v) RedPairImpl.MinInfty =
  RedPairImpl.Num_arc v;
plus_arctic (RedPairImpl.Num_arc x) (RedPairImpl.Num_arc y) =
  RedPairImpl.Num_arc (Orderings.maxa x y);

eq_arctic :: RedPairImpl.Arctic -> RedPairImpl.Arctic -> Bool;
eq_arctic (RedPairImpl.Num_arc a) RedPairImpl.MinInfty = False;
eq_arctic RedPairImpl.MinInfty (RedPairImpl.Num_arc a) = False;
eq_arctic (RedPairImpl.Num_arc int) (RedPairImpl.Num_arc int') = int == int';
eq_arctic RedPairImpl.MinInfty RedPairImpl.MinInfty = True;

instance Eq RedPairImpl.Arctic where {
  a == b = RedPairImpl.eq_arctic a b;
};

int_name :: [Char];
int_name =
  ['n', 'a', 't', 'u', 'r', 'a', 'l', 's', ' ', 'w', 'i', 't', 'h', ' ', 'n',
    'e', 'g', 'a', 't', 'i', 'v', 'e', ' ', 'c', 'o', 'n', 's', 't', 'a', 'n',
    't', 's'];

nat_name :: [Char];
nat_name = ['n', 'a', 't', 'u', 'r', 'a', 'l', 's'];

sshow_arctic :: RedPairImpl.Arctic -> [Char];
sshow_arctic (RedPairImpl.Num_arc x) = Show.sshow_int x;
sshow_arctic RedPairImpl.MinInfty = ['-', 'i', 'n', 'f'];

instance Show.Sshow RedPairImpl.Arctic where {
  sshow = RedPairImpl.sshow_arctic;
};

create_rule_removal_proc ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => RedPairImpl.Redpair a ->
                         [(Term.Term a b, Term.Term a b)] ->
                           [(Term.Term a b, Term.Term a b)] ->
                             Eithera.Eithera [[Char]] ();
create_rule_removal_proc (RedPairImpl.Int_carrier ia) =
  PolyOrder.create_rule_removal_proc_comp
    ((\ x -> True),
      (0, (1, ((\ a b -> a + b),
                ((\ a b -> a * b),
                  (Orderings.maxa 0,
                    ((\ x y -> y <= x),
                      ((\ x y -> y < x),
                        (True, (RedPairImpl.int_name, (\ x -> True)))))))))))
    (\ a -> 0 < a) ia;
create_rule_removal_proc (RedPairImpl.Nat_carrier i) =
  PolyOrder.create_rule_removal_proc_comp
    ((\ x -> True),
      (Nat.Zero_nat,
        (Nat.one_nat,
          (Nat.plus_nat,
            (Nat.times_nat,
              (id, ((\ x y -> Nat.less_eq_nat y x),
                     ((\ x y -> Nat.less_nat y x),
                       (True, (RedPairImpl.nat_name, (\ x -> True)))))))))))
    (\ x -> not (x == Nat.Zero_nat)) i;
create_rule_removal_proc (RedPairImpl.Arctic_carrier ida) =
  (\ uu uua ->
    ErrorMonad.errora
      [['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
         'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
         's']]);
create_rule_removal_proc (RedPairImpl.Arc_mat_carrier n ic) =
  (\ uu uua ->
    ErrorMonad.errora
      [['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
         'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
         's']]);
create_rule_removal_proc (RedPairImpl.Int_mat_carrier n ib) =
  (\ uu uua ->
    ErrorMonad.errora
      [['m', 'o', 'n', 'o', 't', 'o', 'n', 'i', 'c', ' ', 'm', 'a', 't', 'r',
         'i', 'x', '-', 'o', 'r', 'd', 'e', 'r', 's', ' ', 'n', 'o', 't', ' ',
         'y', 'e', 't', ' ', 's', 'u', 'p', 'p', 'o', 'r', 't', 'e', 'd']]);

create_mono_redpair_ur_proc ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => RedPairImpl.Redpair a ->
                         [(Term.Term a b, Term.Term a b)] ->
                           [(Term.Term a b, Term.Term a b)] ->
                             [(Term.Term a b, Term.Term a b)] ->
                               Maybe (a -> [(Term.Term a b, Term.Term a b)]) ->
                                 [(Term.Term a b, Term.Term a b)] ->
                                   [(Term.Term a b, Term.Term a b)] ->
                                     Eithera.Eithera [[Char]] ();
create_mono_redpair_ur_proc (RedPairImpl.Int_carrier ia) =
  PolyOrder.create_mono_redpair_ur_proc_comp
    ((\ x -> True),
      (0, (1, ((\ a b -> a + b),
                ((\ a b -> a * b),
                  (Orderings.maxa 0,
                    ((\ x y -> y <= x),
                      ((\ x y -> y < x),
                        (True, (RedPairImpl.int_name, (\ x -> True)))))))))))
    (\ a -> 0 < a) ia;
create_mono_redpair_ur_proc (RedPairImpl.Nat_carrier i) =
  PolyOrder.create_mono_redpair_ur_proc_comp
    ((\ x -> True),
      (Nat.Zero_nat,
        (Nat.one_nat,
          (Nat.plus_nat,
            (Nat.times_nat,
              (id, ((\ x y -> Nat.less_eq_nat y x),
                     ((\ x y -> Nat.less_nat y x),
                       (True, (RedPairImpl.nat_name, (\ x -> True)))))))))))
    (\ x -> not (x == Nat.Zero_nat)) i;
create_mono_redpair_ur_proc (RedPairImpl.Arctic_carrier ida) =
  (\ uu uua uub uuc uud uue ->
    ErrorMonad.errora
      [['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
         'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
         's']]);
create_mono_redpair_ur_proc (RedPairImpl.Arc_mat_carrier n ic) =
  (\ uu uua uub uuc uud uue ->
    ErrorMonad.errora
      [['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
         'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
         's']]);
create_mono_redpair_ur_proc (RedPairImpl.Int_mat_carrier n ib) =
  (\ uu uua uub uuc uud uue ->
    ErrorMonad.errora
      [['m', 'o', 'n', 'o', 't', 'o', 'n', 'i', 'c', ' ', 'm', 'a', 't', 'r',
         'i', 'x', '-', 'o', 'r', 'd', 'e', 'r', 's', ' ', 'n', 'o', 't', ' ',
         'y', 'e', 't', ' ', 's', 'u', 'p', 'p', 'o', 'r', 't', 'e', 'd']]);

create_mono_redpair_proc ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => RedPairImpl.Redpair a ->
                         [(Term.Term a b, Term.Term a b)] ->
                           [(Term.Term a b, Term.Term a b)] ->
                             [(Term.Term a b, Term.Term a b)] ->
                               [(Term.Term a b, Term.Term a b)] ->
                                 Eithera.Eithera [[Char]] ();
create_mono_redpair_proc (RedPairImpl.Int_carrier ia) =
  PolyOrder.create_mono_redpair_proc_comp
    ((\ x -> True),
      (0, (1, ((\ a b -> a + b),
                ((\ a b -> a * b),
                  (Orderings.maxa 0,
                    ((\ x y -> y <= x),
                      ((\ x y -> y < x),
                        (True, (RedPairImpl.int_name, (\ x -> True)))))))))))
    (\ a -> 0 < a) ia;
create_mono_redpair_proc (RedPairImpl.Nat_carrier i) =
  PolyOrder.create_mono_redpair_proc_comp
    ((\ x -> True),
      (Nat.Zero_nat,
        (Nat.one_nat,
          (Nat.plus_nat,
            (Nat.times_nat,
              (id, ((\ x y -> Nat.less_eq_nat y x),
                     ((\ x y -> Nat.less_nat y x),
                       (True, (RedPairImpl.nat_name, (\ x -> True)))))))))))
    (\ x -> not (x == Nat.Zero_nat)) i;
create_mono_redpair_proc (RedPairImpl.Arctic_carrier ida) =
  (\ uu uua uub uuc ->
    ErrorMonad.errora
      [['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
         'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
         's']]);
create_mono_redpair_proc (RedPairImpl.Arc_mat_carrier n ic) =
  (\ uu uua uub uuc ->
    ErrorMonad.errora
      [['a', 'r', 'c', 't', 'i', 'c', ' ', 'n', 'o', 't', ' ', 'm', 'o', 'n',
         'o', 't', 'o', 'n', 'i', 'c', ' ', 'f', 'o', 'r', ' ', 'T', 'R', 'S',
         's']]);
create_mono_redpair_proc (RedPairImpl.Int_mat_carrier n ib) =
  (\ uu uua uub uuc ->
    ErrorMonad.errora
      [['m', 'o', 'n', 'o', 't', 'o', 'n', 'i', 'c', ' ', 'm', 'a', 't', 'r',
         'i', 'x', '-', 'o', 'r', 'd', 'e', 'r', 's', ' ', 'n', 'o', 't', ' ',
         'y', 'e', 't', ' ', 's', 'u', 'p', 'p', 'o', 'r', 't', 'e', 'd']]);

create_redpair_ur_af_proc ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => RedPairImpl.Redpair a ->
                         [(Term.Term a b, Term.Term a b)] ->
                           [(Term.Term a b, Term.Term a b)] ->
                             [(Term.Term a b, Term.Term a b)] ->
                               Maybe (a -> [(Term.Term a b, Term.Term a b)]) ->
                                 [(Term.Term a b, Term.Term a b)] ->
                                   Eithera.Eithera [[Char]] ();
create_redpair_ur_af_proc (RedPairImpl.Int_carrier ia) =
  PolyOrder.create_redpair_ur_af_proc_comp
    ((\ x -> True),
      (0, (1, ((\ a b -> a + b),
                ((\ a b -> a * b),
                  (Orderings.maxa 0,
                    ((\ x y -> y <= x),
                      ((\ x y -> y < x),
                        (True, (RedPairImpl.int_name, (\ x -> True)))))))))))
    ia;
create_redpair_ur_af_proc (RedPairImpl.Nat_carrier i) =
  PolyOrder.create_redpair_ur_af_proc_comp
    ((\ x -> True),
      (Nat.Zero_nat,
        (Nat.one_nat,
          (Nat.plus_nat,
            (Nat.times_nat,
              (id, ((\ x y -> Nat.less_eq_nat y x),
                     ((\ x y -> Nat.less_nat y x),
                       (True, (RedPairImpl.nat_name, (\ x -> True)))))))))))
    i;
create_redpair_ur_af_proc (RedPairImpl.Arctic_carrier ida) =
  PolyOrder.create_redpair_ur_af_proc_comp
    ((\ x -> True),
      (RedPairImpl.MinInfty,
        (RedPairImpl.Num_arc 0,
          (RedPairImpl.plus_arctic,
            (RedPairImpl.mult_arctic,
              (id, (RedPairImpl.geq_arctic,
                     (RedPairImpl.gt_arctic,
                       (False,
                         (RedPairImpl.arctic_name,
                           RedPairImpl.pos_arctic))))))))))
    ida;
create_redpair_ur_af_proc (RedPairImpl.Int_mat_carrier n ib) =
  PolyOrder.create_redpair_ur_af_proc_comp
    (Matrix.wf_mat_rcI (Nat.Suc n) (Nat.Suc n),
      (Matrix.mat0I 0 (Nat.Suc n) (Nat.Suc n),
        (Matrix.mat1I 0 1 (Nat.Suc n),
          (Matrix.mat_plusI (\ a b -> a + b),
            (Matrix.mat_mulI 0 (\ a b -> a + b) (\ a b -> a * b) (Nat.Suc n),
              (Matrix.mat_max0I (Orderings.maxa 0),
                (Matrix.mat_geI (\ x y -> y <= x),
                  (Matrix.mat_gtI (\ x y -> y <= x) (\ x y -> y < x),
                    (True,
                      (RedPairImpl.int_mat_name,
                        Matrix.mat_arc_posI (\ x -> True)))))))))))
    ib;
create_redpair_ur_af_proc (RedPairImpl.Arc_mat_carrier n ic) =
  PolyOrder.create_redpair_ur_af_proc_comp
    (Matrix.wf_mat_rcI (Nat.Suc n) (Nat.Suc n),
      (Matrix.mat0I RedPairImpl.MinInfty (Nat.Suc n) (Nat.Suc n),
        (Matrix.mat1I RedPairImpl.MinInfty (RedPairImpl.Num_arc 0) (Nat.Suc n),
          (Matrix.mat_plusI RedPairImpl.plus_arctic,
            (Matrix.mat_mulI RedPairImpl.MinInfty RedPairImpl.plus_arctic
               RedPairImpl.mult_arctic (Nat.Suc n),
              (Matrix.mat_max0I id,
                (Matrix.mat_geI RedPairImpl.geq_arctic,
                  (Matrix.mat_geI RedPairImpl.gt_arctic,
                    (False,
                      (RedPairImpl.arc_mat_name,
                        Matrix.mat_arc_posI RedPairImpl.pos_arctic))))))))))
    ic;

create_redpair_proc ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => RedPairImpl.Redpair a ->
                         [(Term.Term a b, Term.Term a b)] ->
                           [(Term.Term a b, Term.Term a b)] ->
                             [(Term.Term a b, Term.Term a b)] ->
                               Eithera.Eithera [[Char]] ();
create_redpair_proc (RedPairImpl.Int_carrier ia) =
  PolyOrder.create_redpair_proc_comp
    ((\ x -> True),
      (0, (1, ((\ a b -> a + b),
                ((\ a b -> a * b),
                  (Orderings.maxa 0,
                    ((\ x y -> y <= x),
                      ((\ x y -> y < x),
                        (True, (RedPairImpl.int_name, (\ x -> True)))))))))))
    ia;
create_redpair_proc (RedPairImpl.Nat_carrier i) =
  PolyOrder.create_redpair_proc_comp
    ((\ x -> True),
      (Nat.Zero_nat,
        (Nat.one_nat,
          (Nat.plus_nat,
            (Nat.times_nat,
              (id, ((\ x y -> Nat.less_eq_nat y x),
                     ((\ x y -> Nat.less_nat y x),
                       (True, (RedPairImpl.nat_name, (\ x -> True)))))))))))
    i;
create_redpair_proc (RedPairImpl.Arctic_carrier ida) =
  PolyOrder.create_redpair_proc_comp
    ((\ x -> True),
      (RedPairImpl.MinInfty,
        (RedPairImpl.Num_arc 0,
          (RedPairImpl.plus_arctic,
            (RedPairImpl.mult_arctic,
              (id, (RedPairImpl.geq_arctic,
                     (RedPairImpl.gt_arctic,
                       (False,
                         (RedPairImpl.arctic_name,
                           RedPairImpl.pos_arctic))))))))))
    ida;
create_redpair_proc (RedPairImpl.Int_mat_carrier n ib) =
  PolyOrder.create_redpair_proc_comp
    (Matrix.wf_mat_rcI (Nat.Suc n) (Nat.Suc n),
      (Matrix.mat0I 0 (Nat.Suc n) (Nat.Suc n),
        (Matrix.mat1I 0 1 (Nat.Suc n),
          (Matrix.mat_plusI (\ a b -> a + b),
            (Matrix.mat_mulI 0 (\ a b -> a + b) (\ a b -> a * b) (Nat.Suc n),
              (Matrix.mat_max0I (Orderings.maxa 0),
                (Matrix.mat_geI (\ x y -> y <= x),
                  (Matrix.mat_gtI (\ x y -> y <= x) (\ x y -> y < x),
                    (True,
                      (RedPairImpl.int_mat_name,
                        Matrix.mat_arc_posI (\ x -> True)))))))))))
    ib;
create_redpair_proc (RedPairImpl.Arc_mat_carrier n ic) =
  PolyOrder.create_redpair_proc_comp
    (Matrix.wf_mat_rcI (Nat.Suc n) (Nat.Suc n),
      (Matrix.mat0I RedPairImpl.MinInfty (Nat.Suc n) (Nat.Suc n),
        (Matrix.mat1I RedPairImpl.MinInfty (RedPairImpl.Num_arc 0) (Nat.Suc n),
          (Matrix.mat_plusI RedPairImpl.plus_arctic,
            (Matrix.mat_mulI RedPairImpl.MinInfty RedPairImpl.plus_arctic
               RedPairImpl.mult_arctic (Nat.Suc n),
              (Matrix.mat_max0I id,
                (Matrix.mat_geI RedPairImpl.geq_arctic,
                  (Matrix.mat_geI RedPairImpl.gt_arctic,
                    (False,
                      (RedPairImpl.arc_mat_name,
                        Matrix.mat_arc_posI RedPairImpl.pos_arctic))))))))))
    ic;

}
