{-# OPTIONS_GHC -fglasgow-exts #-}

module SemanticLabeling where {

import qualified List;
import qualified HOL;
import qualified Show;
import qualified Util;
import qualified Term;
import qualified Eithera;
import qualified ErrorMonad;
import qualified Option;
import qualified Nat;
import qualified Orderings;
import qualified Product_Type;
import qualified Trs;

data Lab a b = Lab (SemanticLabeling.Lab a b) b
  | FunLab (SemanticLabeling.Lab a b) [(SemanticLabeling.Lab a b)] | UnLab a;

data ArithFun = Arg Nat.Nat | Const Nat.Nat | Sum [SemanticLabeling.ArithFun]
  | Max [SemanticLabeling.ArithFun] | Min [SemanticLabeling.ArithFun]
  | Prod [SemanticLabeling.ArithFun];

data Sl_inter a b =
  SL_Inter Nat.Nat
    [((SemanticLabeling.Lab a b, Nat.Nat), SemanticLabeling.ArithFun)];

enum_vectors :: forall a b. [a] -> [b] -> [[(b, a)]];
enum_vectors c [] = [[]];
enum_vectors c (x : xs) =
  let {
    a = SemanticLabeling.enum_vectors c xs;
  } in Util.flat_map (\ vec -> List.mapa (\ ca -> (x, ca) : vec) c) a;

assign_from_vector :: forall a b. (Eq a) => [(a, b)] -> a -> b;
assign_from_vector vec x = Option.the (Util.lookupa x vec);

eq_lab ::
  forall a b.
    (Eq a,
      Eq b) => SemanticLabeling.Lab a b -> SemanticLabeling.Lab a b -> Bool;
eq_lab (SemanticLabeling.FunLab fa ls) (SemanticLabeling.FunLab g ks) =
  SemanticLabeling.eq_lab fa g && List.list_all2 SemanticLabeling.eq_lab ls ks;
eq_lab (SemanticLabeling.UnLab c) (SemanticLabeling.FunLab a ba) = False;
eq_lab (SemanticLabeling.UnLab c) (SemanticLabeling.Lab a b) = False;
eq_lab (SemanticLabeling.FunLab ca d) (SemanticLabeling.Lab a b) = False;
eq_lab (SemanticLabeling.FunLab a ba) (SemanticLabeling.UnLab c) = False;
eq_lab (SemanticLabeling.Lab a b) (SemanticLabeling.UnLab c) = False;
eq_lab (SemanticLabeling.Lab a b) (SemanticLabeling.FunLab ca d) = False;
eq_lab (SemanticLabeling.UnLab f) (SemanticLabeling.UnLab f') = f == f';
eq_lab (SemanticLabeling.Lab lab l) (SemanticLabeling.Lab lab' l') =
  SemanticLabeling.eq_lab lab lab' && l == l';

instance (Eq a, Eq b) => Eq (SemanticLabeling.Lab a b) where {
  a == b = SemanticLabeling.eq_lab a b;
};

sshow_lab ::
  forall c d.
    (Show.Sshow c, Show.Sshow d) => SemanticLabeling.Lab c d -> [Char];
sshow_lab (SemanticLabeling.UnLab f) = Show.sshow f;
sshow_lab (SemanticLabeling.Lab fa l) =
  List.append
    (List.append (List.append (SemanticLabeling.sshow_lab fa) ['['])
      (Show.sshow l))
    [']'];
sshow_lab (SemanticLabeling.FunLab fa la) =
  List.append
    (List.append (List.append (SemanticLabeling.sshow_lab fa) ['['])
      (Util.flat_map SemanticLabeling.sshow_lab la))
    [']'];

instance (Show.Sshow a,
           Show.Sshow b) => Show.Sshow (SemanticLabeling.Lab a b) where {
  sshow = SemanticLabeling.sshow_lab;
};

label ::
  forall a b.
    SemanticLabeling.Lab a b ->
      Eithera.Eithera b [(SemanticLabeling.Lab a b)] ->
        SemanticLabeling.Lab a b;
label f (Eithera.Lefta l) = SemanticLabeling.Lab f l;
label f (Eithera.Righta la) = SemanticLabeling.FunLab f la;

eval_lab ::
  forall a b c d.
    (SemanticLabeling.Lab a b -> [c] -> c) ->
      (SemanticLabeling.Lab a b ->
        [c] -> Eithera.Eithera b [(SemanticLabeling.Lab a b)])
        -> (d -> c) ->
             Term.Term (SemanticLabeling.Lab a b) d ->
               (c, Term.Term (SemanticLabeling.Lab a b) d);
eval_lab i l alpha (Term.Var x) = (alpha x, Term.Var x);
eval_lab i l alpha (Term.Fun f ts) =
  let {
    clts = List.mapa (SemanticLabeling.eval_lab i l alpha) ts;
    cs = List.mapa fst clts;
    c = i f cs;
    lts = List.mapa snd clts;
    la = l f cs;
  } in (c, Term.Fun (SemanticLabeling.label f la) lts);

check_sl_rule_ass ::
  forall a b c d.
    (Eq a, Show.Sshow a, Eq b, Show.Sshow b, Eq c, Show.Sshow c, Eq d,
      Show.Sshow d) => Bool ->
                         (SemanticLabeling.Lab a b -> [c] -> c) ->
                           (SemanticLabeling.Lab a b ->
                             [c] ->
                               Eithera.Eithera b [(SemanticLabeling.Lab a b)])
                             -> [(Term.Term (SemanticLabeling.Lab a b) d,
                                   Term.Term (SemanticLabeling.Lab a b) d)]
                                  -> (d -> c) ->
                                       (Term.Term (SemanticLabeling.Lab a b) d,
 Term.Term (SemanticLabeling.Lab a b) d)
 -> Eithera.Eithera [Char] ();
check_sl_rule_ass mc i la lR alpha (l, r) =
  let {
    cl_ll = SemanticLabeling.eval_lab i la alpha l;
    cr_lr = SemanticLabeling.eval_lab i la alpha r;
  } in ErrorMonad.bind
         (ErrorMonad.checkB (HOL.implies mc (fst cl_ll == fst cr_lr))
           (List.append
             (List.append
               (List.append
                 (List.append
                   (List.append ['r', 'u', 'l', 'e', ' '] (Trs.showRule (l, r)))
                   [' ', 'v', 'i', 'o', 'l', 'a', 't', 'e', 's', ' ', 'm', 'o',
                     'd', 'e', 'l', ' ', 'c', 'o', 'n', 'd', 'i', 't', 'i', 'o',
                     'n', ',', ' ', '[', 'l', 'h', 's', ']', ' ', '=', ' '])
                 (Show.sshow (fst cl_ll :: c)))
               [',', ' ', '[', 'r', 'h', 's', ']', ' ', '=', ' '])
             (Show.sshow (fst cr_lr :: c))))
         (\ x ->
           ErrorMonad.checkB (List.member (snd cl_ll, snd cr_lr) lR)
             (List.append
               (List.append
                 ['l', 'a', 'b', 'e', 'l', 'e', 'd', ' ', 'r', 'u', 'l', 'e',
                   ' ']
                 (Trs.showRule (snd cl_ll, snd cr_lr)))
               [' ', 'm', 'i', 's', 's', 'i', 'n', 'g']));

check_sl_rule ::
  forall a b c d.
    (Eq a, Show.Sshow a, Eq b, Show.Sshow b, Eq c, Show.Sshow c, Eq d,
      Show.Sshow d) => (SemanticLabeling.Lab a b -> [c] -> c) ->
                         (SemanticLabeling.Lab a b ->
                           [c] ->
                             Eithera.Eithera b [(SemanticLabeling.Lab a b)])
                           -> [c] ->
                                Bool ->
                                  [(Term.Term (SemanticLabeling.Lab a b) d,
                                     Term.Term (SemanticLabeling.Lab a b) d)]
                                    -> (Term.Term (SemanticLabeling.Lab a b) d,
 Term.Term (SemanticLabeling.Lab a b) d)
 -> Eithera.Eithera [Char] ();
check_sl_rule i la c mc lR (l, r) =
  ErrorMonad.catcha
    (ErrorMonad.checkAllMI
      (\ alpha -> SemanticLabeling.check_sl_rule_ass mc i la lR alpha (l, r))
      (List.mapa SemanticLabeling.assign_from_vector
        (SemanticLabeling.enum_vectors c
          (Util.list_union (Term.vars_term_list l) (Term.vars_term_list r)))))
    (\ x -> ErrorMonad.errora (snd x));

check_sl_model_lab_trs ::
  forall f l c v.
    (Eq f, Show.Sshow f, Eq l, Show.Sshow l, Eq c, Show.Sshow c, Eq v,
      Show.Sshow v) => (SemanticLabeling.Lab f l -> [c] -> c) ->
                         (SemanticLabeling.Lab f l ->
                           [c] ->
                             Eithera.Eithera l [(SemanticLabeling.Lab f l)])
                           -> [c] ->
                                [(Term.Term (SemanticLabeling.Lab f l) v,
                                   Term.Term (SemanticLabeling.Lab f l) v)]
                                  -> [(Term.Term (SemanticLabeling.Lab f l) v,
Term.Term (SemanticLabeling.Lab f l) v)]
                                       -> Eithera.Eithera [Char] ();
check_sl_model_lab_trs i l c lR r =
  ErrorMonad.catcha
    (ErrorMonad.checkAllMI (SemanticLabeling.check_sl_rule i l c True lR) r)
    (\ x -> ErrorMonad.errora (snd x));

check_sl_lab_trs ::
  forall f l c v.
    (Eq f, Show.Sshow f, Eq l, Show.Sshow l, Eq c, Show.Sshow c, Eq v,
      Show.Sshow v) => (SemanticLabeling.Lab f l -> [c] -> c) ->
                         (SemanticLabeling.Lab f l ->
                           [c] ->
                             Eithera.Eithera l [(SemanticLabeling.Lab f l)])
                           -> [c] ->
                                [(Term.Term (SemanticLabeling.Lab f l) v,
                                   Term.Term (SemanticLabeling.Lab f l) v)]
                                  -> [(Term.Term (SemanticLabeling.Lab f l) v,
Term.Term (SemanticLabeling.Lab f l) v)]
                                       -> Eithera.Eithera [Char] ();
check_sl_lab_trs i l c lP p =
  ErrorMonad.catcha
    (ErrorMonad.checkAllMI (SemanticLabeling.check_sl_rule i l c False lP) p)
    (\ x -> ErrorMonad.errora (snd x));

get_lab ::
  forall a b.
    SemanticLabeling.Lab a b -> Eithera.Eithera b [(SemanticLabeling.Lab a b)];
get_lab (SemanticLabeling.Lab f l) = Eithera.Lefta l;
get_lab (SemanticLabeling.FunLab f la) = Eithera.Righta la;

unlab :: forall a b. SemanticLabeling.Lab a b -> SemanticLabeling.Lab a b;
unlab (SemanticLabeling.Lab f l) = f;
unlab (SemanticLabeling.FunLab f la) = f;
unlab (SemanticLabeling.UnLab v) = SemanticLabeling.UnLab v;

map_term :: forall a b c. (a -> b) -> Term.Term a c -> Term.Term b c;
map_term fg (Term.Var x) = Term.Var x;
map_term fg (Term.Fun f ts) =
  Term.Fun (fg f) (List.mapa (SemanticLabeling.map_term fg) ts);

check_sl_only_lab_and_decr ::
  forall b a c.
    (Eq b, Show.Sshow b, Eq a, Show.Sshow a, Eq c,
      Show.Sshow c) => ((Eithera.Eithera b [(SemanticLabeling.Lab a b)],
                          Eithera.Eithera b [(SemanticLabeling.Lab a b)])
                         -> Bool)
                         -> [(Term.Term (SemanticLabeling.Lab a b) c,
                               Term.Term (SemanticLabeling.Lab a b) c)]
                              -> [(Term.Term (SemanticLabeling.Lab a b) c,
                                    Term.Term (SemanticLabeling.Lab a b) c)]
                                   -> Eithera.Eithera [Char] ();
check_sl_only_lab_and_decr gr r =
  (\ xs ->
    ErrorMonad.catcha
      (ErrorMonad.checkAllMI
        (\ (a @ (ll, lr)) ->
          let {
            l = SemanticLabeling.map_term SemanticLabeling.unlab ll;
            ra = SemanticLabeling.map_term SemanticLabeling.unlab lr;
          } in ErrorMonad.checkB
                 (if l == ra
                   then not (Term.is_var ll) &&
                          (not (Term.is_var lr) &&
                            gr (SemanticLabeling.get_lab
                                  (Option.the (Term.root ll)),
                                 SemanticLabeling.get_lab
                                   (Option.the (Term.root lr))))
                          || ll == lr && List.member (l, ra) r
                   else List.member (l, ra) r)
                 (List.append
                   (List.append
                     ['l', 'a', 'b', 'e', 'l', 'e', 'd', ' ', 'r', 'u', 'l',
                       'e', ' ']
                     (Trs.showRule (l, ra)))
                   [' ', 'i', 's', ' ', 'n', 'e', 'i', 't', 'h', 'e', 'r', ' ',
                     'd', 'e', 'c', 'r', 'e', 'a', 's', 'i', 'n', 'g', ' ', 'n',
                     'o', 'r', ' ', 'a', ' ', 'l', 'a', 'b', 'e', 'l', 'e', 'd',
                     ' ', 'v', 'e', 'r', 's', 'i', 'o', 'n', ' ', 'o', 'f', ' ',
                     'o', 'r', 'i', 'g', 'i', 'n', 'a', 'l', ' ', 'r', 'u', 'l',
                     'e']))
        xs)
      (\ x -> ErrorMonad.errora (snd x)));

check_sem_lab_tt ::
  forall f l v c.
    (Eq f, Show.Sshow f, Eq l, Show.Sshow l, Eq v,
      Show.Sshow v) => ([(Term.Term (SemanticLabeling.Lab f l) v,
                           Term.Term (SemanticLabeling.Lab f l) v)]
                         -> [(Term.Term (SemanticLabeling.Lab f l) v,
                               Term.Term (SemanticLabeling.Lab f l) v)]
                              -> Eithera.Eithera [Char] ())
                         -> ([(Term.Term (SemanticLabeling.Lab f l) v,
                                Term.Term (SemanticLabeling.Lab f l) v)]
                              -> [(Term.Term (SemanticLabeling.Lab f l) v,
                                    Term.Term (SemanticLabeling.Lab f l) v)]
                                   -> Eithera.Eithera [Char] ())
                              -> ((Eithera.Eithera l
                                     [(SemanticLabeling.Lab f l)],
                                    Eithera.Eithera l
                                      [(SemanticLabeling.Lab f l)])
                                   -> Bool)
                                   -> (SemanticLabeling.Lab f l -> [c] -> c) ->
(SemanticLabeling.Lab f l ->
  [c] -> Eithera.Eithera l [(SemanticLabeling.Lab f l)])
  -> [(Term.Term (SemanticLabeling.Lab f l) v,
        Term.Term (SemanticLabeling.Lab f l) v)]
       -> [(Term.Term (SemanticLabeling.Lab f l) v,
             Term.Term (SemanticLabeling.Lab f l) v)]
            -> Eithera.Eithera [[Char]] ();
check_sem_lab_tt check_lab check_model_lab gr i l r lR =
  ErrorMonad.catcha
    (ErrorMonad.bind (check_model_lab lR r)
      (\ x -> SemanticLabeling.check_sl_only_lab_and_decr gr r lR))
    (\ x ->
      ErrorMonad.errora
        [['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't', 'h', ' ', 'l',
           'a', 'b', 'e', 'l', 'e', 'd', ' ', 'T', 'R', 'S', ':'],
          x]);

check_sem_lab_proc ::
  forall f l v c.
    (Eq f, Show.Sshow f, Eq l, Show.Sshow l, Eq v,
      Show.Sshow v) => ([(Term.Term (SemanticLabeling.Lab f l) v,
                           Term.Term (SemanticLabeling.Lab f l) v)]
                         -> [(Term.Term (SemanticLabeling.Lab f l) v,
                               Term.Term (SemanticLabeling.Lab f l) v)]
                              -> Eithera.Eithera [Char] ())
                         -> ([(Term.Term (SemanticLabeling.Lab f l) v,
                                Term.Term (SemanticLabeling.Lab f l) v)]
                              -> [(Term.Term (SemanticLabeling.Lab f l) v,
                                    Term.Term (SemanticLabeling.Lab f l) v)]
                                   -> Eithera.Eithera [Char] ())
                              -> ((Eithera.Eithera l
                                     [(SemanticLabeling.Lab f l)],
                                    Eithera.Eithera l
                                      [(SemanticLabeling.Lab f l)])
                                   -> Bool)
                                   -> (SemanticLabeling.Lab f l -> [c] -> c) ->
(SemanticLabeling.Lab f l ->
  [c] -> Eithera.Eithera l [(SemanticLabeling.Lab f l)])
  -> [(Term.Term (SemanticLabeling.Lab f l) v,
        Term.Term (SemanticLabeling.Lab f l) v)]
       -> [(Term.Term (SemanticLabeling.Lab f l) v,
             Term.Term (SemanticLabeling.Lab f l) v)]
            -> [(Term.Term (SemanticLabeling.Lab f l) v,
                  Term.Term (SemanticLabeling.Lab f l) v)]
                 -> [(Term.Term (SemanticLabeling.Lab f l) v,
                       Term.Term (SemanticLabeling.Lab f l) v)]
                      -> Eithera.Eithera [[Char]] ();
check_sem_lab_proc check_lab check_model_lab gr i l p r lP lR =
  ErrorMonad.bind
    (ErrorMonad.bind
      (ErrorMonad.catcha (check_lab lP p)
        (\ x ->
          ErrorMonad.errora
            [['n', 'o', 't', ' ', 'a', 'l', 'l', ' ', 'l', 'a', 'b', 'e', 'l',
               'e', 'd', ' ', 'D', 'P', 's', ' ', 'p', 'r', 'e', 's', 'e', 'n',
               't', ':'],
              x]))
      (\ x ->
        ErrorMonad.catcha (check_model_lab lR r)
          (\ xa ->
            ErrorMonad.errora
              [['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't', 'h', ' ',
                 'l', 'a', 'b', 'e', 'l', 'e', 'd', ' ', 'T', 'R', 'S', ':'],
                xa])))
    (\ x ->
      ErrorMonad.catcha (SemanticLabeling.check_sl_only_lab_and_decr gr r lR)
        (\ xa -> ErrorMonad.errora [xa]));

init_lab :: forall a b. a -> SemanticLabeling.Lab a b;
init_lab f = SemanticLabeling.UnLab f;

map_rule ::
  forall a b c.
    (a -> b) ->
      (Term.Term a c, Term.Term a c) -> (Term.Term b c, Term.Term b c);
map_rule fg lr =
  (SemanticLabeling.map_term fg (fst lr),
    SemanticLabeling.map_term fg (snd lr));

sshow_arithFun :: SemanticLabeling.ArithFun -> [Char];
sshow_arithFun (SemanticLabeling.Arg i) =
  List.append ['x', '_'] (Show.sshow_nat (Nat.Suc i));
sshow_arithFun (SemanticLabeling.Const n) = Show.sshow_nat n;
sshow_arithFun (SemanticLabeling.Sum (f : fs)) =
  List.append (SemanticLabeling.sshow_arithFun f)
    (Util.flat_map
      (\ g -> List.append [' ', '+', ' '] (SemanticLabeling.sshow_arithFun g))
      fs);
sshow_arithFun (SemanticLabeling.Prod (f : fs)) =
  List.append
    (List.append (List.append ['('] (SemanticLabeling.sshow_arithFun f))
      (Util.flat_map
        (\ g -> List.append [' ', '*', ' '] (SemanticLabeling.sshow_arithFun g))
        fs))
    [')'];
sshow_arithFun (SemanticLabeling.Min (f : fs)) =
  List.append
    (List.append
      (List.append ['m', 'i', 'n', '('] (SemanticLabeling.sshow_arithFun f))
      (Util.flat_map
        (\ g -> List.append [','] (SemanticLabeling.sshow_arithFun g)) fs))
    [')'];
sshow_arithFun (SemanticLabeling.Max (f : fs)) =
  List.append
    (List.append
      (List.append ['m', 'a', 'x', '('] (SemanticLabeling.sshow_arithFun f))
      (Util.flat_map
        (\ g -> List.append [','] (SemanticLabeling.sshow_arithFun g)) fs))
    [')'];

sshow_sl_inter ::
  forall c d.
    (Show.Sshow c, Show.Sshow d) => SemanticLabeling.Sl_inter c d -> [Char];
sshow_sl_inter (SemanticLabeling.SL_Inter n fnas) =
  Show.showLista
    (List.append
       (List.append
         ['c', 'a', 'r', 'r', 'i', 'e', 'r', ' ', '{', '0', ',', '.', '.', '.',
           ',']
         (Show.sshow_nat n))
       ['}']
      : List.mapa
          (\ (a @ (aa, b)) ->
            let {
              (f, na) = aa;
            } in (\ ab ->
                   List.append
                     (List.append
                       (List.append
                         (List.append
                           (List.append ['['] (SemanticLabeling.sshow_lab f))
                           ['/'])
                         (Show.sshow_nat na))
                       [']', ' ', '=', ' '])
                     (SemanticLabeling.sshow_arithFun ab))
              b)
          fnas);

take_default :: forall a. a -> [a] -> Nat.Nat -> a;
take_default def [] uu = def;
take_default uv (x : xs) Nat.Zero_nat = x;
take_default def (uw : xs) (Nat.Suc i) = SemanticLabeling.take_default def xs i;

eval_arithFun_unbound :: [Nat.Nat] -> SemanticLabeling.ArithFun -> Nat.Nat;
eval_arithFun_unbound nats (SemanticLabeling.Arg i) =
  SemanticLabeling.take_default Nat.Zero_nat nats i;
eval_arithFun_unbound nats (SemanticLabeling.Const n) = n;
eval_arithFun_unbound nats (SemanticLabeling.Sum []) = Nat.Zero_nat;
eval_arithFun_unbound nats (SemanticLabeling.Sum (f : fs)) =
  Nat.plus_nat (SemanticLabeling.eval_arithFun_unbound nats f)
    (SemanticLabeling.eval_arithFun_unbound nats (SemanticLabeling.Sum fs));
eval_arithFun_unbound nats (SemanticLabeling.Prod []) = Nat.one_nat;
eval_arithFun_unbound nats (SemanticLabeling.Prod (f : fs)) =
  Nat.times_nat (SemanticLabeling.eval_arithFun_unbound nats f)
    (SemanticLabeling.eval_arithFun_unbound nats (SemanticLabeling.Prod fs));
eval_arithFun_unbound nats (SemanticLabeling.Max [f]) =
  SemanticLabeling.eval_arithFun_unbound nats f;
eval_arithFun_unbound nats (SemanticLabeling.Max (f : v : va)) =
  Orderings.maxa (SemanticLabeling.eval_arithFun_unbound nats f)
    (SemanticLabeling.eval_arithFun_unbound nats
      (SemanticLabeling.Max (v : va)));
eval_arithFun_unbound nats (SemanticLabeling.Min [f]) =
  SemanticLabeling.eval_arithFun_unbound nats f;
eval_arithFun_unbound nats (SemanticLabeling.Min (f : v : va)) =
  Orderings.mina (SemanticLabeling.eval_arithFun_unbound nats f)
    (SemanticLabeling.eval_arithFun_unbound nats
      (SemanticLabeling.Min (v : va)));

eval_arithFun :: Nat.Nat -> SemanticLabeling.ArithFun -> [Nat.Nat] -> Nat.Nat;
eval_arithFun c f nats =
  Nat.mod_nat (SemanticLabeling.eval_arithFun_unbound nats f) c;

sl_inter_to_inter ::
  forall a b.
    (Eq a,
      Eq b) => SemanticLabeling.Sl_inter a b ->
                 SemanticLabeling.Lab a b -> [Nat.Nat] -> Nat.Nat;
sl_inter_to_inter (SemanticLabeling.SL_Inter c ls) fl cs =
  (case Util.lookupa (fl, List.size_list cs) ls of {
    Nothing -> Nat.Zero_nat;
    Just f -> SemanticLabeling.eval_arithFun (Nat.Suc c) f cs;
  });

get_largest_element :: forall a b. SemanticLabeling.Sl_inter a b -> Nat.Nat;
get_largest_element (SemanticLabeling.SL_Inter n uu) = n;

create_sem_lab_tt ::
  forall f l v.
    (Eq f, Show.Sshow f, Eq l, Show.Sshow l, Eq v,
      Show.Sshow v) => SemanticLabeling.Sl_inter f l ->
                         (SemanticLabeling.Lab f l ->
                           [Nat.Nat] ->
                             Eithera.Eithera l [(SemanticLabeling.Lab f l)])
                           -> [(Term.Term (SemanticLabeling.Lab f l) v,
                                 Term.Term (SemanticLabeling.Lab f l) v)]
                                -> [(Term.Term (SemanticLabeling.Lab f l) v,
                                      Term.Term (SemanticLabeling.Lab f l) v)]
                                     -> Eithera.Eithera [[Char]] ();
create_sem_lab_tt sli l r lr =
  let {
    c = SemanticLabeling.get_largest_element sli;
    ca = List.upt Nat.Zero_nat (Nat.Suc c);
    i = SemanticLabeling.sl_inter_to_inter sli;
  } in ErrorMonad.catcha
         (SemanticLabeling.check_sem_lab_tt
           (SemanticLabeling.check_sl_lab_trs i l ca)
           (SemanticLabeling.check_sl_model_lab_trs i l ca) (\ x -> False) i l r
           lr)
         (\ x -> ErrorMonad.errora (SemanticLabeling.sshow_sl_inter sli : x));

map_trsI ::
  forall a b c.
    (a -> b) ->
      [(Term.Term a c, Term.Term a c)] -> [(Term.Term b c, Term.Term b c)];
map_trsI fg r = List.mapa (SemanticLabeling.map_rule fg) r;

unlab_of_trsI ::
  forall f l v.
    (Eq f, Eq l,
      Eq v) => [(Term.Term (SemanticLabeling.Lab f l) v,
                  Term.Term (SemanticLabeling.Lab f l) v)]
                 -> [(Term.Term (SemanticLabeling.Lab f l) v,
                       Term.Term (SemanticLabeling.Lab f l) v)];
unlab_of_trsI r =
  List.remdups
    (SemanticLabeling.map_trsI SemanticLabeling.unlab
      (List.filtera
        (\ (a @ (l, ra)) ->
          l == ra ||
            not (SemanticLabeling.map_term SemanticLabeling.unlab l ==
                  SemanticLabeling.map_term SemanticLabeling.unlab ra))
        r));

unlab_trs_tt ::
  forall f l v.
    (Eq f, Show.Sshow f, Eq l, Show.Sshow l, Eq v,
      Show.Sshow v) => [(Term.Term (SemanticLabeling.Lab f l) v,
                          Term.Term (SemanticLabeling.Lab f l) v)]
                         -> [(Term.Term (SemanticLabeling.Lab f l) v,
                               Term.Term (SemanticLabeling.Lab f l) v)]
                              -> Eithera.Eithera [[Char]] ();
unlab_trs_tt r rR =
  let {
    u = SemanticLabeling.unlab_of_trsI r;
  } in ErrorMonad.catcha
         (ErrorMonad.bind (ErrorMonad.checkSubset rR u)
           (\ x -> ErrorMonad.checkSubset u rR))
         (\ x ->
           ErrorMonad.errora
             (List.append (Trs.showRules rR)
               (List.append
                 [[' ', ' ', 'd', 'o', 'e', 's', ' ', 'n', 'o', 't', ' ', 'c',
                    'o', 'r', 'r', 'e', 's', 'p', 'o', 'n', 'd', ' ', 't', 'o',
                    ' ', 'u', 'n', 'l', 'a', 'b', 'e', 'l', 'e', 'd', ' ', 'T',
                    'R', 'S', ' ', ' ']]
                 (Trs.showRules u))));

create_sem_lab_proc ::
  forall f l v.
    (Eq f, Show.Sshow f, Eq l, Show.Sshow l, Eq v,
      Show.Sshow v) => SemanticLabeling.Sl_inter f l ->
                         (SemanticLabeling.Lab f l ->
                           [Nat.Nat] ->
                             Eithera.Eithera l [(SemanticLabeling.Lab f l)])
                           -> [(Term.Term (SemanticLabeling.Lab f l) v,
                                 Term.Term (SemanticLabeling.Lab f l) v)]
                                -> [(Term.Term (SemanticLabeling.Lab f l) v,
                                      Term.Term (SemanticLabeling.Lab f l) v)]
                                     -> [(Term.Term (SemanticLabeling.Lab f l)
    v,
   Term.Term (SemanticLabeling.Lab f l) v)]
  -> [(Term.Term (SemanticLabeling.Lab f l) v,
        Term.Term (SemanticLabeling.Lab f l) v)]
       -> Eithera.Eithera [[Char]] ();
create_sem_lab_proc sli l r1 r2 r3 r4 =
  let {
    c = SemanticLabeling.get_largest_element sli;
    ca = List.upt Nat.Zero_nat (Nat.Suc c);
    i = SemanticLabeling.sl_inter_to_inter sli;
  } in ErrorMonad.catcha
         (SemanticLabeling.check_sem_lab_proc
           (SemanticLabeling.check_sl_lab_trs i l ca)
           (SemanticLabeling.check_sl_model_lab_trs i l ca) (\ x -> False) i l
           r1 r2 r3 r4)
         (\ x -> ErrorMonad.errora (SemanticLabeling.sshow_sl_inter sli : x));

check_unlab_proc ::
  forall f l v.
    (Eq f, Show.Sshow f, Eq l, Show.Sshow l, Eq v,
      Show.Sshow v) => [(Term.Term (SemanticLabeling.Lab f l) v,
                          Term.Term (SemanticLabeling.Lab f l) v)]
                         -> [(Term.Term (SemanticLabeling.Lab f l) v,
                               Term.Term (SemanticLabeling.Lab f l) v)]
                              -> [(Term.Term (SemanticLabeling.Lab f l) v,
                                    Term.Term (SemanticLabeling.Lab f l) v)]
                                   -> [(Term.Term (SemanticLabeling.Lab f l) v,
 Term.Term (SemanticLabeling.Lab f l) v)]
-> Eithera.Eithera [[Char]] ();
check_unlab_proc p r ulP ulR =
  let {
    ulr = SemanticLabeling.unlab_of_trsI r;
  } in ErrorMonad.catcha
         (ErrorMonad.bind
           (ErrorMonad.bind
             (ErrorMonad.checkSubset
               (SemanticLabeling.map_trsI SemanticLabeling.unlab p) ulP)
             (\ x -> ErrorMonad.checkSubset ulr ulR))
           (\ x -> ErrorMonad.checkSubset ulR ulr))
         (\ x ->
           ErrorMonad.errora
             [List.append
                ['p', 'r', 'o', 'b', 'l', 'e', 'm', ' ', 'w', 'i', 't', 'h',
                  ' ', 'r', 'u', 'l', 'e', ' ']
                (Trs.showRule x)]);

}
