module Ceta = 
struct

let rec id x = x;;

type 'a eq = {eq : 'a -> 'a -> bool};;
let eq _A = _A.eq;;

type nat = Zero_nat | Suc of nat;;

type ('a, 'b) scg = Null | Scg of 'a * 'a * ('b * 'b) list * ('b * 'b) list;;

type 'a shp = Sharp of 'a | Plain of 'a;;

type xml =
  Xml of
    char list * (char list * char list) list * xml list * (char list) option;;

let rec leta s f = f s;;

type 'a ord = {less_eq : 'a -> 'a -> bool; less : 'a -> 'a -> bool};;
let less_eq _A = _A.less_eq;;
let less _A = _A.less;;

let rec nat_aux
  i n = (if Big_int.le_big_int i (Big_int.big_int_of_int 0) then n
          else nat_aux (Big_int.sub_big_int i (Big_int.big_int_of_int 1))
                 (Suc n));;

let rec nat i = nat_aux i Zero_nat;;

let rec hd (x :: xs) = x;;

let rec tl = function [] -> []
             | x :: xs -> xs;;

type ('a, 'b) term = Var of 'b | Fun of 'a * ('a, 'b) term list;;

type ('a, 'b) sub = Sub of ('b -> ('a, 'b) term);;

let rec foldl
  f a x2 = match f, a, x2 with f, a, [] -> a
    | f, a, x :: xs -> foldl f (f a x) xs;;

let rec plus_nat
  x0 n = match x0, n with Suc m, n -> plus_nat m (Suc n)
    | Zero_nat, n -> n;;

let one_nat : nat = Suc Zero_nat

let rec sz
  = function Var x -> one_nat
    | Fun (f, ts) ->
        plus_nat one_nat (foldl (fun a x -> plus_nat a (sz x)) Zero_nat ts);;

type 'a poly = PVar of nat | PNum of 'a | PSum of 'a poly list |
  PMult of 'a poly list;;

let rec fst (a, b) = a;;

let rec nth
  x0 x1 = match x0, x1 with x :: xs, Suc n -> nth xs n
    | x :: xs, Zero_nat -> x;;

let rec eqop _A a b = eq _A a b;;

type ('a, 'b) hterm = HHole | HFun of 'a * ('a, 'b) hterm list;;

let rec option_case
  f1 f2 x2 = match f1, f2, x2 with f1, f2, Some a -> f2 a
    | f1, f2, None -> f1;;

let rec mergeHTerms _A
  x0 x1 = match x0, x1 with [], [] -> Some []
    | [], uu :: uv -> None
    | uw :: ux, [] -> None
    | HHole :: hsl, h :: hsr ->
        (match mergeHTerms _A hsl hsr with None -> None
           | Some res -> Some (h :: res))
    | HFun (f, hs) :: hsl, HHole :: hsr ->
        (match mergeHTerms _A hsl hsr with None -> None
           | Some res -> Some (HFun (f, hs) :: res))
    | HFun (f, fs) :: hsl, HFun (g, gs) :: hsr ->
        (if eqop _A f g
          then (match mergeHTerms _A fs gs with None -> None
                  | Some mhs ->
                    (match mergeHTerms _A hsl hsr with None -> None
                       | Some res -> Some (HFun (f, mhs) :: res)))
          else None);;

let rec merge _A
  hl hr =
    (match mergeHTerms _A [hl] [hr] with None -> None
       | Some res -> Some (nth res Zero_nat));;

let rec mergeX _A _B
  x h xa2 = match x, h, xa2 with x, h, [] -> Some []
    | x, h, (ht, y) :: ps ->
        (if eqop _A x y
          then (match merge _B h ht with None -> None
                  | Some m -> mergeX _A _B x m ps)
          else (match mergeX _A _B x h ps with None -> None
                  | Some pairs -> Some ((ht, y) :: pairs)));;

let rec mergeAll _A _B
  = function [] -> true
    | (h, x) :: ps ->
        (match mergeX _B _A x h ps with None -> false
           | Some a -> mergeAll _A _B a);;

let rec append
  x0 ys = match x0, ys with [], ys -> ys
    | x :: xs, ys -> x :: append xs ys;;

let rec decomp
  x0 x1 = match x0, x1 with [], [] -> Some []
    | uu :: uv, [] -> None
    | [], uw :: ux -> None
    | s :: ss, t :: ts ->
        (match decomp ss ts with None -> None
           | Some pairs -> Some ((s, t) :: pairs));;

let rec gcMatchList _A
  = function [] -> Some []
    | (HHole, uu) :: ps -> gcMatchList _A ps
    | (HFun (f, hs), Fun (g, ss)) :: ps ->
        (if eqop _A f g
          then (match decomp hs ss with None -> None
                  | Some pairs -> gcMatchList _A (append pairs ps))
          else None)
    | (HFun (f, hs), Var x) :: ps ->
        (match gcMatchList _A ps with None -> None
           | Some pairs -> Some ((HFun (f, hs), x) :: pairs));;

let rec gcMatchI _A _B
  h t = (match gcMatchList _A [(h, t)] with None -> false
           | Some a -> mergeAll _A _B a);;

let rec any
  uu x1 = match uu, x1 with uu, [] -> false
    | p, x :: xs -> p x || any p xs;;

let rec map
  f x1 = match f, x1 with f, [] -> []
    | f, x :: xs -> f x :: map f xs;;

let rec capI _A _B
  uu x1 = match uu, x1 with uu, Var uv -> HHole
    | r, Fun (f, ts) ->
        let h = HFun (f, map (capI _A _B r) ts) in
        (if any (fun ra -> gcMatchI _A _B h (fst ra)) r then HHole else h);;

let rec comp f g = (fun x -> f (g x));;

let rec rev xs = foldl (fun xsa x -> x :: xsa) [] xs;;

let rec less_nat
  m x1 = match m, x1 with m, Suc n -> less_eq_nat m n
    | n, Zero_nat -> false
and less_eq_nat
  x0 n = match x0, n with Suc m, n -> less_nat m n
    | Zero_nat, n -> true;;

let rec upt i j = (if less_nat i j then i :: upt (Suc i) j else []);;

let rec list_case
  f1 f2 x2 = match f1, f2, x2 with f1, f2, a :: lista -> f2 a lista
    | f1, f2, [] -> f1;;

let rec zip
  xs x1 = match xs, x1 with xs, [] -> []
    | xs, y :: ys -> (match xs with [] -> [] | z :: zs -> (z, y) :: zip zs ys);;

type ('a, 'b) either = Left of 'a | Right of 'b;;

let rec return a = Right a;;

type nibble = Nibble0 | Nibble1 | Nibble2 | Nibble3 | Nibble4 | Nibble5 |
  Nibble6 | Nibble7 | Nibble8 | Nibble9 | NibbleA | NibbleB | NibbleC | NibbleD
  | NibbleE | NibbleF;;

let rec concat = function [] -> []
                 | x :: xs -> append x (concat xs);;

let rec rev_append
  x0 ys = match x0, ys with [], ys -> ys
    | x :: xs, ys -> rev_append xs (x :: ys);;

let rec flat_map_aux
  f x1 ys = match f, x1, ys with f, [], ys -> rev ys
    | f, x :: xs, ys -> flat_map_aux f xs (rev_append (f x) ys);;

let rec flat_map f xs = flat_map_aux f xs [];;

let sEmpty : char list = []

type 'a sshow = {sshow : 'a -> char list};;
let sshow _A = _A.sshow;;

let rec sshow_term _C _D
  = function Var x -> sshow _D x
    | Fun (f, ts) ->
        append (sshow _C f)
          ((match ts with [] -> sEmpty
             | s :: ss ->
               append ['(']
                 (append
                   (append (sshow_term _C _D s)
                     (flat_map (fun x -> concat [[',']; sshow_term _C _D x])
                       ss))
                   [')'])));;

let rec error a = Left a;;

let rec size_list
  = function [] -> Zero_nat
    | a :: lista -> plus_nat (size_list lista) (Suc Zero_nat);;

type pos = Empty | PCons of nat * pos;;

let rec minus_nat
  m n = match m, n with Suc m, Suc n -> minus_nat m n
    | Zero_nat, n -> Zero_nat
    | m, Zero_nat -> m;;

let rec split f (a, b) = f a b;;

let rec foldr
  f x1 a = match f, x1, a with f, [], a -> a
    | f, x :: xs, a -> f x (foldr f xs a);;

let rec poss_list
  = function Var x -> [Empty]
    | Fun (f, ts) ->
        let a =
          foldr (fun t ((i, ps) as a) ->
                  (minus_nat i one_nat,
                    append ps (map (fun aa -> PCons (i, aa)) (poss_list t))))
            ts (minus_nat (size_list ts) one_nat, [])
          in
        let (i, aa) = a in
        Empty :: aa;;

let rec nat_case
  f1 f2 x2 = match f1, f2, x2 with f1, f2, Suc nat -> f2 nat
    | f1, f2, Zero_nat -> f1;;

let rec drop
  n x1 = match n, x1 with n, [] -> []
    | n, x :: xs -> (match n with Zero_nat -> x :: xs | Suc m -> drop m xs);;

let rec take
  n x1 = match n, x1 with n, [] -> []
    | n, x :: xs -> (match n with Zero_nat -> [] | Suc m -> x :: take m xs);;

type ('a, 'b) ctxt = Hole |
  More of 'a * ('a, 'b) term list * ('a, 'b) ctxt * ('a, 'b) term list;;

let rec ctxt_of_pos_term
  x0 t = match x0, t with Empty, t -> Hole
    | PCons (i, ps), Fun (f, ts) ->
        More (f, take i ts, ctxt_of_pos_term ps (nth ts i), drop (Suc i) ts);;

let rec eqa _A _B (x1, y1) (x2, y2) = eqop _A x1 x2 && eqop _B y1 y2;;

let eqb _A _B = ({eq = eqa _A _B} : ('a * 'b) eq);;

let rec member _A
  x xa1 = match x, xa1 with x, [] -> false
    | x, y :: ys -> eqop _A x y || member _A x ys;;

let rec sub_apply
  x0 sigmaa = match x0, sigmaa with Var x, Sub sigma -> sigma x
    | Fun (f, ss), sigmaa -> Fun (f, map (fun t -> sub_apply t sigmaa) ss);;

let rec ctxt_apply
  x0 s = match x0, s with Hole, s -> s
    | More (f, ss1, c, ss2), s -> Fun (f, append ss1 (ctxt_apply c s :: ss2));;

let rec null = function [] -> true
               | x :: xs -> false;;

let rec list_all2
  p xs ys = match p, xs, ys with
    p, x :: xs, y :: ys -> p x y && list_all2 p xs ys
    | p, xs, [] -> null xs
    | p, [], ys -> null ys;;

let rec eq_term _C _D
  x0 x1 = match x0, x1 with
    Fun (f, ts), Fun (g, us) -> eq _C f g && list_all2 (eq_term _C _D) ts us
    | Var x, Var y -> eq _D x y
    | Fun (b, c), Var a -> false
    | Var a, Fun (b, c) -> false;;

let eq_terma _A _B = ({eq = eq_term _A _B} : ('a, 'b) term eq);;

let rec checkB b e = (if b then return () else error e);;

let rec rewrites (_A1, _A2) (_B1, _B2)
  (s, t) c sigma (l, r) ra =
    checkB
      (eqop (eq_terma _A1 _B1) s (ctxt_apply c (sub_apply l sigma)) &&
        (eqop (eq_terma _A1 _B1) t (ctxt_apply c (sub_apply r sigma)) &&
          member (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) (l, r) ra))
      (append
        (append (append ['T'; 'e'; 'r'; 'm'; ' '] (sshow_term _A2 _B2 s))
          [' '; 'd'; 'o'; 'e'; 's'; ' '; 'n'; 'o'; 't'; ' '; 'r'; 'e'; 'w'; 'r';
            'i'; 't'; 'e'; ' '; 't'; 'o'; ' '; 't'; 'e'; 'r'; 'm'; ' '])
        (sshow_term _A2 _B2 t));;

let rec either_case
  f1 f2 x2 = match f1, f2, x2 with f1, f2, Right b -> f2 b
    | f1, f2, Left a -> f1 a;;

let rec isValue p = (match p with Left l -> false | Right r -> true);;

let rec subt_at
  s x1 = match s, x1 with s, Empty -> s
    | Fun (f, ss), PCons (i, p) -> subt_at (nth ss i) p;;

let rec sub_case f1 (Sub funa) = f1 funa;;

let rec subst_safe _A _B
  vs x1 sigma = match vs, x1, sigma with vs, [], sigma -> Some sigma
    | vs, (Var x, t) :: ts, sigma ->
        (if member (eqb (eq_terma _B _A) (eq_terma _B _A)) (Var x, t) ts
          then subst_safe _A _B vs ts sigma
          else (if member _A x vs &&
                     not (eqop (eq_terma _B _A) (sub_apply (Var x) sigma) t)
                 then None
                 else (match subst_safe _A _B (x :: vs) ts sigma
                         with None -> None
                         | Some a ->
                           let Sub s = a in
                           Some (Sub (fun y ->
                                       (if eqop _A x y then t else s y))))));;

let rec decompa
  x0 x1 = match x0, x1 with [], [] -> Some []
    | uu :: uv, [] -> None
    | [], uw :: ux -> None
    | s :: ss, t :: ts ->
        (match decompa ss ts with None -> None
           | Some pairs -> Some ((s, t) :: pairs));;

let rec decompose_one _A
  x0 t = match x0, t with Var x, t -> Some [(Var x, t)]
    | Fun (v, va), Var y -> None
    | Fun (f, ts), Fun (g, ss) ->
        (if eqop _A f g then decompa ts ss else None);;

let rec decompose_n _A
  x0 s t = match x0, s, t with Zero_nat, s, t -> decompose_one _A s t
    | Suc n, s, t ->
        (match decompose_n _A n s t with None -> None
           | Some a ->
             foldl (fun aa ((sa, ta) as b) ->
                     (match aa with None -> None
                        | Some ys ->
                          (match decompose_one _A sa ta with None -> None
                             | Some xs -> Some (append xs ys))))
               (Some []) a);;

let rec decompose _A s t = decompose_n _A (sz s) s t;;

let rec decompose_list _A
  ts = foldl (fun a ((s, t) as b) ->
               (match a with None -> None
                  | Some ls ->
                    (match decompose _A s t with None -> None
                       | Some ys -> Some (append ls ys))))
         (Some []) ts;;

let rec match_all _A _B
  ts = (match decompose_list _A ts with None -> None
          | Some ls -> subst_safe _B _A [] ls (Sub (fun a -> Var a)));;

let rec getpr (_A1, _A2) (_B1, _B2)
  s t p (l, r) =
    (match match_all _A1 _B1 [(l, subt_at s p); (r, subt_at t p)]
       with None -> None
       | Some sigma ->
         (if isValue
               (rewrites (_A1, _A2) (_B1, _B2) (s, t) (ctxt_of_pos_term p s)
                 sigma (l, r) [(l, r)])
           then Some (s, (ctxt_of_pos_term p s, (sigma, (l, r)))) else None));;

let rec getp (_A1, _A2) (_B1, _B2)
  s t p x3 = match s, t, p, x3 with s, t, p, [] -> None
    | s, t, p, r :: rs ->
        (match getpr (_A1, _A2) (_B1, _B2) s t p r
           with None -> getp (_A1, _A2) (_B1, _B2) s t p rs
           | Some a -> Some a);;

let rec find_get_p (_F1, _F2) (_V1, _V2)
  s t x2 r = match s, t, x2, r with s, t, [], r -> None
    | s, t, p :: ps, r ->
        (match getp (_F1, _F2) (_V1, _V2) s t p r
           with None -> find_get_p (_F1, _F2) (_V1, _V2) s t ps r
           | Some a -> Some a);;

let rec get (_A1, _A2) (_B1, _B2)
  s t r =
    (match find_get_p (_A1, _A2) (_B1, _B2) s t (poss_list s) r
       with None ->
         error (append
                 (append
                   (append
                     (append
                       ['C'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'f';
                         'i'; 'n'; 'd'; ' '; 'r'; 'e'; 'd'; 'u'; 'c'; 't'; 'i';
                         'o'; 'n'; ' '; 'b'; 'e'; 't'; 'w'; 'e'; 'e'; 'n'; ' ';
                         't'; 'e'; 'r'; 'm'; ' ']
                       (sshow_term _A2 _B2 s))
                     [' '; 'a'; 'n'; 'd'; ' '; 't'; 'e'; 'r'; 'm'; ' '])
                   (sshow_term _A2 _B2 t))
                 ['.'])
       | Some a -> return a);;

let rec prod_case f1 (a, b) = f1 a b;;

let rec remdups _A
  = function [] -> []
    | x :: xs ->
        (if member _A x xs then remdups _A xs else x :: remdups _A xs);;

let rec compa _Ap
  es es' =
    remdups (eqb _Ap _Ap)
      (flat_map
        (fun ((x, y) as a) ->
          flat_map
            (fun ((y', z) as aa) -> (if eqop _Ap y y' then [(x, z)] else []))
            es')
        es);;

let tabC : char = '\009'

let tab : char list = [tabC]

let newlineC : char = '\010'

let newline : char list = [newlineC]

let wspace : char list = append newline (append tab [' '])

let eq_char = ({eq = (fun a b -> ((a : char) = b))} : char eq);;

let rec trim
  = function c :: s -> (if member eq_char c wspace then trim s else c :: s)
    | [] -> [];;

let rec capRM2 _A _B
  uu x1 = match uu, x1 with uu, Var uv -> HHole
    | rm, Fun (f, ts) ->
        let h = HFun (f, map (capRM2 _A _B rm) ts) in
        (if any (fun r -> gcMatchI _A _B h (fst r)) (rm f) then HHole else h);;

let rec capRM _A _B
  r = (match r with None -> (fun t -> HHole) | Some a -> capRM2 _A _B a);;

let eq_int = ({eq = Big_int.eq_big_int} : Big_int.big_int eq);;

let rec eq_nat
  x0 x1 = match x0, x1 with Suc a, Zero_nat -> false
    | Zero_nat, Suc a -> false
    | Suc nat, Suc nat' -> eq_nat nat nat'
    | Zero_nat, Zero_nat -> true;;

let eq_nata = ({eq = eq_nat} : nat eq);;

let rec eq_list _A
  x0 x1 = match x0, x1 with a :: b, [] -> false
    | [], a :: b -> false
    | a :: lista, a' :: list' -> eq _A a a' && eq_list _A lista list'
    | [], [] -> true;;

let rec eq_scg _Pp _Ap
  x0 x1 = match x0, x1 with Scg (a, b, c, d), Null -> false
    | Null, Scg (a, b, c, d) -> false
    | Scg (pp1, pp2, list1, list2), Scg (pp1', pp2', list1', list2') ->
        eq _Pp pp1 pp1' &&
          (eq _Pp pp2 pp2' &&
            (eq_list (eqb _Ap _Ap) list1 list1' &&
              eq_list (eqb _Ap _Ap) list2 list2'))
    | Null, Null -> true;;

let eq_scga _A _B = ({eq = eq_scg _A _B} : ('a, 'b) scg eq);;

let rec combine _B
  x0 s = match x0, s with
    Scg (p, q, str, wk), Scg (p', q', str', wk') ->
      Scg (p, q, remdups (eqb _B _B) (append str str'),
            remdups (eqb _B _B) (append wk wk'))
    | Null, s -> Null
    | Scg (v, va, vb, vc), Null -> Null;;

let rec list_all
  p x1 = match p, x1 with p, [] -> true
    | p, x :: xs -> p x && list_all p xs;;

let rec subsumes _A _B
  g x1 = match g, x1 with
    Scg (p, q, str, wk), Scg (p', q', str', wk') ->
      eqop _A p p' &&
        (eqop _A q q' &&
          (list_all (fun x -> member (eqb _B _B) x str') str &&
            list_all (fun x -> member (eqb _B _B) x (append str' wk')) wk))
    | g, Null -> true
    | Null, Scg (v, va, vb, vc) -> false;;

let rec filter
  p x1 = match p, x1 with p, [] -> []
    | p, x :: xs -> (if p x then x :: filter p xs else filter p xs);;

let rec subtract_list _A xs ys = filter (fun x -> not (member _A x ys)) xs;;

let rec scg_comp _B
  conn x1 g = match conn, x1, g with
    conn, Scg (p, q, str, wk), Scg (p', q', str', wk') ->
      (if not (conn q p') then Null
        else let strs =
               remdups (eqb _B _B)
                 (append (compa _B str str')
                   (append (compa _B str wk') (compa _B wk str')))
               in
             let a =
               subtract_list (eqb _B _B) (remdups (eqb _B _B) (compa _B wk wk'))
                 strs
               in
             Scg (p, q', strs, a))
    | conn, Null, g -> Null
    | conn, Scg (v, va, vb, vc), Null -> Null;;

let rec list_ex
  p x1 = match p, x1 with p, [] -> false
    | p, x :: xs -> p x || list_ex p xs;;

let rec in_situ _Ap
  = function Null -> true
    | Scg (p, q, str, wk) -> list_ex (fun ((aa, b) as a) -> eqop _Ap aa b) str;;

let rec sagiv _A _B
  conn g =
    (if in_situ _B g then true
      else let gg = scg_comp _B conn g g in
           (if subsumes _A _B gg g then false
             else sagiv _A _B conn (combine _B g gg)));;

let rec empty x = false;;

let rec args = function Var uu -> []
               | Fun (uv, ts) -> ts;;

let rec root = function Var x -> None
               | Fun (f, ts) -> Some f;;

let rec eq_shp _F
  x0 x1 = match x0, x1 with Plain b, Sharp a -> false
    | Sharp a, Plain b -> false
    | Plain f, Plain f' -> eq _F f f'
    | Sharp f, Sharp f' -> eq _F f f';;

let eq_shpa _A = ({eq = eq_shp _A} : 'a shp eq);;

type domain = Natural | Arctic | Int_mat of nat | Arc_mat of nat;;

let rec eq_option _A
  x0 x1 = match x0, x1 with Some a, None -> false
    | None, Some a -> false
    | Some a, Some a' -> eq _A a a'
    | None, None -> true;;

let eq_lista _A = ({eq = eq_list _A} : ('a list) eq);;

let rec eq_xml
  (Xml (name1, attrs1, children1, text1))
    (Xml (name2, attrs2, children2, text2)) =
    eq_list eq_char name1 name2 &&
      (eq_list (eqb (eq_lista eq_char) (eq_lista eq_char)) attrs1 attrs2 &&
        (list_all2 eq_xml children1 children2 &&
          eq_option (eq_lista eq_char) text1 text2));;

let eq_xmla = ({eq = eq_xml} : xml eq);;

type xmldoc = Xmldoc of char list * xml;;

let ord_int =
  ({less_eq = Big_int.le_big_int; less = Big_int.lt_big_int} :
    Big_int.big_int ord);;

let ord_nat = ({less_eq = less_eq_nat; less = less_nat} : nat ord);;

let rec the (Some x) = x;;

let rec plaina
  = function Var x -> Var x
    | Fun (f, ss) -> Fun (Plain f, map plaina ss);;

let rec sharpa
  = function Var x -> Var x
    | Fun (f, ss) -> Fun (Sharp f, map plaina ss);;

let dquote : char list = ['\034']

type 'a spL = SimpProj of ('a * nat) list;;

let rec times_nat
  x0 n = match x0, n with Zero_nat, n -> Zero_nat
    | Suc m, n -> plus_nat n (times_nat m n);;

let rec digit2nat
  x = (if eqop eq_char x '0' then return Zero_nat
        else (if eqop eq_char x '1' then return one_nat
               else (if eqop eq_char x '2'
                      then return (nat (Big_int.big_int_of_int 2))
                      else (if eqop eq_char x '3'
                             then return (nat (Big_int.big_int_of_int 3))
                             else (if eqop eq_char x '4'
                                    then return (nat (Big_int.big_int_of_int 4))
                                    else (if eqop eq_char x '5'
   then return (nat (Big_int.big_int_of_int 5))
   else (if eqop eq_char x '6' then return (nat (Big_int.big_int_of_int 6))
          else (if eqop eq_char x '7'
                 then return (nat (Big_int.big_int_of_int 7))
                 else (if eqop eq_char x '8'
                        then return (nat (Big_int.big_int_of_int 8))
                        else (if eqop eq_char x '9'
                               then return (nat (Big_int.big_int_of_int 9))
                               else error (append [x]
    [' '; 'i'; 's'; ' '; 'n'; 'o'; 't'; ' '; 'a'; ' '; 'd'; 'i'; 'g'; 'i';
      't'])))))))))));;

let rec bind p vq = (match p with Left a -> Left a | Right a -> vq a);;

let rec string2natHelp
  n x1 = match n, x1 with n, [] -> return n
    | n, d :: s ->
        bind (digit2nat d)
          (fun m ->
            string2natHelp
              (plus_nat (times_nat (nat (Big_int.big_int_of_int 10)) n) m) s);;

let rec string2nat
  s = (if null s
        then error ['c'; 'a'; 'n'; 'n'; 'o'; 't'; ' '; 'c'; 'o'; 'n'; 'v'; 'e';
                     'r'; 't'; ' '; 'e'; 'm'; 'p'; 't'; 'y'; ' '; 's'; 't'; 'r';
                     'i'; 'n'; 'g'; ' '; 'i'; 'n'; 't'; 'o'; ' '; 'n'; 'u'; 'm';
                     'b'; 'e'; 'r']
        else string2natHelp Zero_nat s);;

let rec replicate
  xa0 x = match xa0, x with Zero_nat, x -> []
    | Suc n, x -> x :: replicate n x;;

let rec is_none = function Some x -> false
                  | None -> true;;

let rec showAtt
  (a, v) = append a (append ['='] (append dquote (append v dquote)));;

let rec showAtts
  asa = foldr (fun a s -> append [' '] (append (showAtt a) s)) asa [];;

let rec showXMLindent
  ind i (Xml (n, a, c, t)) =
    concat
      [ind; ['<']; n; showAtts a;
        (if null c && is_none t then ['/'; '>']
          else concat
                 [['>'];
                   (if not (null c)
                     then append
                            (flat_map
                              (showXMLindent (append ind (replicate i ' ')) i)
                              c)
                            ind
                     else []);
                   (if is_none t then [] else the t); ['<'; '/']; n; ['>']])];;

let rec sshow_xml
  xml = trim (showXMLindent newline (nat (Big_int.big_int_of_int 2)) xml);;

let rec xml2text
  reqName (Xml (name, atts, cs, text)) =
    (if eqop (eq_lista eq_char) name reqName &&
          (null atts &&
            (null cs && (not (is_none text) && not (null (the text)))))
      then return (the text)
      else error (concat
                   [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'e'; 'x';
                      't'; 'r'; 'a'; 'c'; 't'; ' '; 't'; 'e'; 'x'; 't'; ' ';
                      'f'; 'o'; 'r'; ' '];
                     reqName; [' '; 'f'; 'r'; 'o'; 'm'; ' '; ' ']; newline;
                     sshow_xml (Xml (name, atts, cs, text))]));;

let rec xml2nat reqName x = bind (xml2text reqName x) string2nat;;

let rec xml3elements
  reqName p1 p2 p3 combi (Xml (name, atts, cs, text)) =
    (if eqop (eq_lista eq_char) name reqName &&
          (null atts &&
            (eqop eq_nata (size_list cs) (nat (Big_int.big_int_of_int 3)) &&
              is_none text))
      then bind (p1 (nth cs Zero_nat))
             (fun a ->
               bind (p2 (nth cs one_nat))
                 (fun b ->
                   bind (p3 (nth cs (nat (Big_int.big_int_of_int 2))))
                     (comp return (combi a b))))
      else error (concat
                   [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 't'; 'r';
                      'a'; 'n'; 's'; 'f'; 'o'; 'r'; 'm'; ' '; 't'; 'h'; 'e';
                      ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ';
                      'x'; 'm'; 'l'; ' '; 'e'; 'l'; 'e'; 'm'; 'e'; 'n'; 't';
                      ' '; '('; 'e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '];
                     reqName; [')']; newline;
                     sshow_xml (Xml (name, atts, cs, text))]));;

let rec mapMonad
  uu x1 = match uu, x1 with uu, [] -> return []
    | f, xml :: rest ->
        (match f xml with Left a -> error a
           | Right a ->
             (match mapMonad f rest with Left aa -> error aa
                | Right asa -> return (a :: asa)));;

let rec xmlManyElements
  reqName p combi (Xml (name, atts, cs, text)) =
    (if eqop (eq_lista eq_char) name reqName && (null atts && is_none text)
      then bind (mapMonad p cs) (comp return combi)
      else error (concat
                   [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 't'; 'r';
                      'a'; 'n'; 's'; 'f'; 'o'; 'r'; 'm'; ' '; 't'; 'h'; 'e';
                      ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ';
                      'x'; 'm'; 'l'; ' '; 'e'; 'l'; 'e'; 'm'; 'e'; 'n'; 't';
                      ' '; '('; 'e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '];
                     reqName; [')']; newline;
                     sshow_xml (Xml (name, atts, cs, text))]));;

let rec xml2sp
  xml2name =
    xmlManyElements
      ['a'; 'r'; 'g'; 'u'; 'm'; 'e'; 'n'; 't'; 'F'; 'i'; 'l'; 't'; 'e'; 'r']
      (xml3elements
        ['a'; 'r'; 'g'; 'u'; 'm'; 'e'; 'n'; 't'; 'F'; 'i'; 'l'; 't'; 'e'; 'r';
          'E'; 'n'; 't'; 'r'; 'y']
        xml2name (xml2nat ['a'; 'r'; 'i'; 't'; 'y'])
        (xml2nat ['c'; 'o'; 'l'; 'l'; 'a'; 'p'; 's'; 'i'; 'n'; 'g'])
        (fun f a n -> (f, minus_nat n one_nat)))
      (fun a -> SimpProj a);;

let rec fun_upd _A f a b = (fun x -> (if eqop _A x a then b else f x));;

let rec implies p q = not p || q;;

let rec sshowa _C _D
  (x, y) = concat [sshow _C x; [' '; '-'; '>'; ' ']; sshow _D y];;

let rec is_var = function Var uu -> true
                 | Fun (uv, uw) -> false;;

let rec showRule _A _B
  (l, r) =
    append (append (sshow_term _A _B l) [' '; '-'; '>'; ' '])
      (sshow_term _A _B r);;

let rec showRules _A _B r = map (showRule _A _B) r;;

let rec showDPP _A _B
  p r = ['d'; 'e'; 'p'; 'e'; 'n'; 'd'; 'e'; 'n'; 'c'; 'y'; ' '; 'p'; 'a'; 'i';
          'r'; 's'; ':'; ' ']
          :: append (showRules _A _B p)
               ([] ::
                 ['r'; 'e'; 'w'; 'r'; 'i'; 't'; 'e'; ' '; 's'; 'y'; 's'; 't';
                   'e'; 'm'; ':']
                   :: append (showRules _A _B r) [[]]);;

let rec showTRS _A _B r = append (showRules _A _B r) [[]];;

let rec forall
  p x1 = match p, x1 with p, [] -> true
    | p, x :: xs -> p x && forall p xs;;

let rec lookup _A
  uu x1 = match uu, x1 with uu, [] -> None
    | a, (a', b) :: xs -> (if eqop _A a a' then Some b else lookup _A a xs);;

let rec snd (a, b) = b;;

let rec parseError
  s allowed =
    error (append
            ['e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '; 's'; 'y'; 'm'; 'b';
              'o'; 'l'; 's'; ' ']
            (append allowed
              (append [' '; 'b'; 'u'; 't'; ' '; 'f'; 'o'; 'u'; 'n'; 'd'; ' ']
                (take (nat (Big_int.big_int_of_int 30)) s))));;

let rec consume2
  x0 allowed s = match x0, allowed, s with
    [], allowed, s ->
      parseError s (flat_map (fun sa -> append sa [',']) allowed)
    | t :: asa, allowed, s ->
        (if eqop (eq_lista eq_char) (map snd (zip t s)) t
          then return (t, trim (drop (size_list t) s))
          else consume2 asa allowed s);;

let rec consume allowed s = consume2 allowed allowed s;;

let letters : char list =
  ['a'; 'b'; 'c'; 'd'; 'e'; 'f'; 'g'; 'h'; 'i'; 'j'; 'k'; 'l'; 'm'; 'n'; 'o';
    'p'; 'q'; 'r'; 's'; 't'; 'u'; 'v'; 'w'; 'x'; 'y'; 'z'; 'A'; 'B'; 'C'; 'D';
    'E'; 'F'; 'G'; 'H'; 'I'; 'J'; 'K'; 'L'; 'M'; 'N'; 'O'; 'P'; 'Q'; 'R'; 'S';
    'T'; 'U'; 'V'; 'W'; 'X'; 'Y'; 'Z'; '_'; '0'; '1'; '2'; '3'; '4'; '5'; '6';
    '7'; '8'; '9'; '&'; ';'; ':']

let rec nat2int
  = function Zero_nat -> (Big_int.big_int_of_int 0)
    | Suc n -> Big_int.add_big_int (Big_int.big_int_of_int 1) (nat2int n);;

let rec string2int
  s = (match s
         with [] ->
           error ['c'; 'a'; 'n'; 'n'; 'o'; 't'; ' '; 'c'; 'o'; 'n'; 'v'; 'e';
                   'r'; 't'; ' '; 'e'; 'm'; 'p'; 't'; 'y'; ' '; 's'; 't'; 'r';
                   'i'; 'n'; 'g'; ' '; 'i'; 'n'; 't'; 'o'; ' '; 'n'; 'u'; 'm';
                   'b'; 'e'; 'r']
         | h :: t ->
           let a = (if eqop eq_char h '-' then (false, t) else (true, s)) in
           let (pos, ss) = a in
           bind (string2nat ss)
             (fun n ->
               (if pos || eqop eq_nata n Zero_nat then return (nat2int n)
                 else return
                        (Big_int.sub_big_int (Big_int.big_int_of_int 0)
                          (nat2int n)))));;

let rec xml2int reqName x = bind (xml2text reqName x) string2int;;

let rec list2position
  = function [] -> Empty
    | n :: ns -> PCons (n, list2position ns);;

let rec xmlChange p f x = bind (p x) (fun a -> return (f a));;

let rec xml2position
  x = xmlChange (xml2nat ['p'; 'o'; 's'; 'i'; 't'; 'i'; 'o'; 'n'])
        (fun n -> minus_nat n one_nat) x;;

let rec xml2pos
  x = xmlManyElements
        ['p'; 'o'; 's'; 'i'; 't'; 'i'; 'o'; 'n'; 'I'; 'n'; 'T'; 'e'; 'r'; 'm']
        xml2position list2position x;;

let rec string2bool
  s = (if eqop (eq_lista eq_char) s ['t'; 'r'; 'u'; 'e'] then return true
        else (if eqop (eq_lista eq_char) s ['f'; 'a'; 'l'; 's'; 'e']
               then return false
               else error (append
                            (append
                              ['c'; 'a'; 'n'; 'n'; 'o'; 't'; ' '; 'c'; 'o'; 'n';
                                'v'; 'e'; 'r'; 't'; ' ']
                              s)
                            [' '; 'i'; 'n'; 't'; 'o'; ' '; 'B'; 'o'; 'o'; 'l';
                              'e'; 'a'; 'n'])));;

let rec xml2bool reqName x = bind (xml2text reqName x) string2bool;;

let rec xml1element
  reqName p1 combi (Xml (name, atts, cs, text)) =
    (if eqop (eq_lista eq_char) name reqName &&
          (null atts && (eqop eq_nata (size_list cs) one_nat && is_none text))
      then bind (p1 (nth cs Zero_nat)) (comp return combi)
      else error (concat
                   [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 't'; 'r';
                      'a'; 'n'; 's'; 'f'; 'o'; 'r'; 'm'; ' '; 't'; 'h'; 'e';
                      ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ';
                      'x'; 'm'; 'l'; ' '; 'e'; 'l'; 'e'; 'm'; 'e'; 'n'; 't';
                      ' '; '('; 'e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '];
                     reqName; [')']; newline;
                     sshow_xml (Xml (name, atts, cs, text))]));;

let rec xml1CmanyElements
  reqName p1 p2 combi (Xml (name, atts, cs, text)) =
    (if eqop (eq_lista eq_char) name reqName &&
          (null atts && (is_none text && not (null cs)))
      then bind (p1 (hd cs))
             (fun a -> bind (mapMonad (p2 a) (tl cs)) (comp return (combi a)))
      else error (concat
                   [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 't'; 'r';
                      'a'; 'n'; 's'; 'f'; 'o'; 'r'; 'm'; ' '; 't'; 'h'; 'e';
                      ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ';
                      'x'; 'm'; 'l'; ' '; 'e'; 'l'; 'e'; 'm'; 'e'; 'n'; 't';
                      ' '; '('; 'e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '];
                     reqName; [')']; newline;
                     sshow_xml (Xml (name, atts, cs, text))]));;

let rec xml1manyElements
  reqName p1 p2 = xml1CmanyElements reqName p1 (fun a -> p2);;

let rec xml2var
  x = bind (xml2text ['v'; 'a'; 'r'] x) (comp return (fun a -> Var a));;

let rec xmlGetName (Xml (name, uu, uv, uw)) = name;;

let rec xmlOptions
  ps x =
    (match lookup (eq_lista eq_char) (xmlGetName x) ps
       with None ->
         error (concat
                 [['e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '; 'o'; 'n'; 'e';
                    ' '; 'o'; 'f'; ':'; ' '];
                   flat_map (fun p -> append (fst p) [' ']) ps; newline;
                   ['b'; 'u'; 't'; ' '; 'f'; 'o'; 'u'; 'n'; 'd']; newline;
                   sshow_xml x])
       | Some p -> p x);;

let rec xml2term
  xml2name x =
    xmlOptions
      [(['v'; 'a'; 'r'], xml2var);
        (['f'; 'u'; 'n'; 'a'; 'p'; 'p'],
          xml1manyElements ['f'; 'u'; 'n'; 'a'; 'p'; 'p'] xml2name
            (xml1element ['a'; 'r'; 'g'] (xml2term xml2name) id)
            (fun a b -> Fun (a, b)))]
      x;;

let rec xml2elements
  reqName p1 p2 combi (Xml (name, atts, cs, text)) =
    (if eqop (eq_lista eq_char) name reqName &&
          (null atts &&
            (eqop eq_nata (size_list cs) (nat (Big_int.big_int_of_int 2)) &&
              is_none text))
      then bind (p1 (nth cs Zero_nat))
             (fun a -> bind (p2 (nth cs one_nat)) (comp return (combi a)))
      else error (concat
                   [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 't'; 'r';
                      'a'; 'n'; 's'; 'f'; 'o'; 'r'; 'm'; ' '; 't'; 'h'; 'e';
                      ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ';
                      'x'; 'm'; 'l'; ' '; 'e'; 'l'; 'e'; 'm'; 'e'; 'n'; 't';
                      ' '; '('; 'e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '];
                     reqName; [')']; newline;
                     sshow_xml (Xml (name, atts, cs, text))]));;

let rec xml2rule
  xml2name =
    xml2elements ['r'; 'u'; 'l'; 'e']
      (xml1element ['l'; 'h'; 's'] (xml2term xml2name) id)
      (xml1element ['r'; 'h'; 's'] (xml2term xml2name) id) (fun a b -> (a, b));;

let rec xml2scg
  xml2name =
    xml1manyElements
      ['s'; 'i'; 'z'; 'e'; 'C'; 'h'; 'a'; 'n'; 'g'; 'e'; 'G'; 'r'; 'a'; 'p';
        'h']
      (xml2rule xml2name)
      (xml3elements ['e'; 'd'; 'g'; 'e'] xml2position
        (xml2bool ['s'; 't'; 'r'; 'i'; 'c'; 't']) xml2position
        (fun p s q -> (s, (p, q))))
      (fun lr edges ->
        (lr, (map snd (filter fst edges),
               map snd (filter (fun x -> not (fst x)) edges))));;

let rec lefta (Left l) = l;;

let rec vec_zeroI ze n = replicate n ze;;

let rec mat0I ze nr nc = replicate nc (vec_zeroI ze nr);;

let rec vec1I
  ze on n i =
    append (replicate i ze)
      (on :: replicate (minus_nat (minus_nat n one_nat) i) ze);;

let rec mat1I ze on n = map (vec1I ze on n) (upt Zero_nat n);;

let rec sshow_shp _B
  = function Sharp s -> append (sshow _B s) ['#']
    | Plain s -> sshow _B s;;

let sshow_shpa _A = ({sshow = sshow_shp _A} : 'a shp sshow);;

let rec splitterFun
  allowed x1 = match allowed, x1 with
    allowed, c :: s ->
      (if allowed c
        then let a = splitterFun allowed s in
             let (l, aa) = a in
             (c :: l, aa)
        else ([], c :: s))
    | uu, [] -> ([], []);;

let rec parseText
  s = let a = splitterFun (fun c -> not (eqop (eq_lista eq_char) [c] ['<'])) s
        in
      let (l, r) = a in
      (if null (trim l) then return (None, r)
        else return (Some (rev (trim (rev (trim l)))), r));;

let rec parseAttribute
  s = bind (consume [dquote] s)
        (fun ((uu, s1) as a) ->
          let aa =
            splitterFun (fun c -> not (eqop (eq_lista eq_char) [c] dquote)) s1
            in
          let (n, s2) = aa in
          bind (consume [dquote] s2)
            (fun ((uua, s3) as ab) -> return (n, s3)));;

let rec splitter allowed s = splitterFun (fun c -> member eq_char c allowed) s;;

let rec parseName
  s = let a = splitter letters s in
      let (l, r) = a in
      (if null l then parseError s letters else return (l, trim r));;

let rec parseAttributes
  = function [] -> return ([], [])
    | c :: s ->
        (if member eq_char c ['/'; '>'] then return ([], c :: s)
          else bind (parseName (c :: s))
                 (fun ((n, s1) as a) ->
                   bind (consume [['=']] s1)
                     (fun ((uu, s2) as aa) ->
                       bind (parseAttribute s2)
                         (fun ((ac, s3) as ab) ->
                           bind (parseAttributes s3)
                             (fun ((nas, s4) as b) ->
                               return ((n, ac) :: nas, s4))))));;

let rec parseXMLList
  s = (if null s ||
            eqop (eq_lista eq_char) (take (nat (Big_int.big_int_of_int 2)) s)
              ['<'; '/']
        then return ([], s)
        else bind (consume [['<']] s)
               (fun ((uu, s1) as a) ->
                 bind (parseName s1)
                   (fun ((n, s2) as aa) ->
                     bind (parseAttributes s2)
                       (fun ((nas, s3) as ab) ->
                         bind (consume [['/'; '>']; ['>']] s3)
                           (fun ((clo, s4) as ac) ->
                             (if eqop (eq_lista eq_char) clo ['/'; '>']
                               then bind (parseXMLList s4)
                                      (fun ((li, s5) as ad) ->
return (Xml (n, nas, [], None) :: li, s5))
                               else bind (parseText s4)
                                      (fun ((t, s5) as ad) ->
bind (parseXMLList s5)
  (fun ((chi, s6) as ae) ->
    bind (consume [append ['<'; '/'] (append n ['>'])] s6)
      (fun ((uua, s7) as af) ->
        bind (parseXMLList s7)
          (fun ((li, s8) as ag) ->
            return (Xml (n, nas, chi, t) :: li, s8)))))))))));;

let rec parseXML
  s = bind (consume [['<']] s)
        (fun ((uu, s1) as a) ->
          bind (parseName s1)
            (fun ((n, s2) as aa) ->
              bind (parseAttributes s2)
                (fun ((nas, s3) as ab) ->
                  bind (consume [['/'; '>']; ['>']] s3)
                    (fun ((clo, s4) as ac) ->
                      (if eqop (eq_lista eq_char) clo ['/'; '>']
                        then return (Xml (n, nas, [], None), trim s4)
                        else bind (parseText s4)
                               (fun ((t, s5) as ad) ->
                                 bind (parseXMLList s5)
                                   (fun ((chi, s6) as ae) ->
                                     bind (consume
    [append ['<'; '/'] (append n ['>'])] s6)
                                       (fun ((uua, s7) as af) ->
 return (Xml (n, nas, chi, t), s7)))))))));;

let rec scanUpto
  enda x1 = match enda, x1 with
    enda, [] ->
      error (append
              ['d'; 'i'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'f'; 'i'; 'n'; 'd'; ' ';
                'r'; 'e'; 'q'; 'u'; 'i'; 'r'; 'e'; 'd'; ' '; 't'; 'o'; 'k'; 'e';
                'n'; ' ']
              enda)
    | enda, c :: s ->
        (if eqop (eq_lista eq_char) (map snd (zip enda (c :: s))) enda
          then return
                 (take (size_list enda) (c :: s),
                   trim (drop (size_list enda) (c :: s)))
          else bind (scanUpto enda s)
                 (fun ((res, rest) as a) -> return (c :: res, rest)));;

let rec xml2poly
  xml2coeff x =
    xml1element ['p'; 'o'; 'l'; 'y'; 'n'; 'o'; 'm'; 'i'; 'a'; 'l']
      (xmlOptions
        [(['c'; 'o'; 'e'; 'f'; 'f'; 'i'; 'c'; 'i'; 'e'; 'n'; 't'],
           xml1element ['c'; 'o'; 'e'; 'f'; 'f'; 'i'; 'c'; 'i'; 'e'; 'n'; 't']
             xml2coeff (fun a -> PNum a));
          (['v'; 'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e'],
            xmlChange (xml2nat ['v'; 'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e'])
              (fun n -> PVar (minus_nat n one_nat)));
          (['s'; 'u'; 'm'],
            xmlManyElements ['s'; 'u'; 'm'] (xml2poly xml2coeff)
              (fun a -> PSum a));
          (['p'; 'r'; 'o'; 'd'; 'u'; 'c'; 't'],
            xmlManyElements ['p'; 'r'; 'o'; 'd'; 'u'; 'c'; 't']
              (xml2poly xml2coeff) (fun a -> PMult a))])
      id x;;

let rec xml_term _C _D
  = function Var x -> Xml (['v'; 'a'; 'r'], [], [], Some (sshow _D x))
    | Fun (f, ts) ->
        Xml (['f'; 'u'; 'n'; 'a'; 'p'; 'p'], [],
              Xml (['n'; 'a'; 'm'; 'e'], [], [], Some (sshow _C f)) ::
                map (fun t ->
                      Xml (['a'; 'r'; 'g'], [], [xml_term _C _D t], None))
                  ts,
              None);;

let rec isLeft = function Left uu -> true
                 | Right uv -> false;;

let rec righta (Right r) = r;;

let rec get_list (_A1, _A2) (_B1, _B2)
  x0 u r = match x0, u, r with
    [], u, r ->
      error ['L'; 'o'; 'o'; 'p'; ' '; 'm'; 'u'; 's'; 't'; ' '; 'b'; 'e'; ' ';
              'n'; 'o'; 'n'; '-'; 'e'; 'm'; 'p'; 't'; 'y']
    | [s], u, r -> bind (get (_A1, _A2) (_B1, _B2) s u r) (fun y -> return [y])
    | s :: t :: ts, u, r ->
        bind (get (_A1, _A2) (_B1, _B2) s t r)
          (fun d ->
            bind (get_list (_A1, _A2) (_B1, _B2) (t :: ts) u r)
              (fun y -> return (d :: y)));;

let rec get_loop (_A1, _A2) (_B1, _B2)
  x0 uu uv uw = match x0, uu, uv, uw with
    [], uu, uv, uw ->
      error ['L'; 'o'; 'o'; 'p'; ' '; 'm'; 'u'; 's'; 't'; ' '; 'b'; 'e'; ' ';
              'n'; 'o'; 'n'; '-'; 'e'; 'm'; 'p'; 't'; 'y']
    | t :: ts, d, tau, r ->
        get_list (_A1, _A2) (_B1, _B2) (t :: ts)
          (ctxt_apply d (sub_apply t tau)) r;;

let rec maxa less_eq a b = (if less_eq a b then b else a);;

let rec max _A a = maxa (less_eq _A) a;;

let rec mina less_eq a b = (if less_eq a b then a else b);;

let rec min _A a = mina (less_eq _A) a;;

let rec insert_scg _Pp _Ap
  g gs =
    (if eqop (eq_scga _Pp _Ap) g Null then None
      else (if list_ex (fun g' -> subsumes _Pp _Ap g' g) gs then None
             else Some (g :: filter (fun g' -> not (subsumes _Pp _Ap g g'))
                               gs)));;

let rec mk_trancl _A _B
  conn x1 s = match conn, x1, s with conn, [], s -> s
    | conn, g :: gs, s ->
        (match insert_scg _A _B g s with None -> mk_trancl _A _B conn gs s
           | Some s' ->
             mk_trancl _A _B conn
               (append (map (scg_comp _B conn g) s')
                 (append (map (fun f -> scg_comp _B conn f g) s') gs))
               s');;

let rec check_SCT _Pp _Ap
  conn gs =
    let a = mk_trancl _Pp _Ap conn gs [] in
    list_all (sagiv _Pp _Ap conn) a;;

let rec showList xs = flat_map (fun x -> append x newline) xs;;

let rec divmod
  m n = (if eqop eq_nata n Zero_nat || less_nat m n then (Zero_nat, m)
          else let a = divmod (minus_nat m n) n in
               let (q, aa) = a in
               (Suc q, aa));;

let rec mod_nat m n = snd (divmod m n);;

let rec div_nat m n = fst (divmod m n);;

let rec digit2string
  n = (if eqop eq_nata n Zero_nat then ['0']
        else (if eqop eq_nata n one_nat then ['1']
               else (if eqop eq_nata n (nat (Big_int.big_int_of_int 2))
                      then ['2']
                      else (if eqop eq_nata n (nat (Big_int.big_int_of_int 3))
                             then ['3']
                             else (if eqop eq_nata n
(nat (Big_int.big_int_of_int 4))
                                    then ['4']
                                    else (if eqop eq_nata n
       (nat (Big_int.big_int_of_int 5))
   then ['5']
   else (if eqop eq_nata n (nat (Big_int.big_int_of_int 6)) then ['6']
          else (if eqop eq_nata n (nat (Big_int.big_int_of_int 7)) then ['7']
                 else (if eqop eq_nata n (nat (Big_int.big_int_of_int 8))
                        then ['8']
                        else (if eqop eq_nata n (nat (Big_int.big_int_of_int 9))
                               then ['9']
                               else ['n'; 'o'; 't'; ' '; 'a'; ' '; 'd'; 'i';
                                      'g'; 'i'; 't']))))))))));;

let rec sshow_nat
  n = (if less_nat n (nat (Big_int.big_int_of_int 10)) then digit2string n
        else append (sshow_nat (div_nat n (nat (Big_int.big_int_of_int 10))))
               (digit2string (mod_nat n (nat (Big_int.big_int_of_int 10)))));;

let rec sshow_int
  i = (if Big_int.lt_big_int i (Big_int.big_int_of_int 0)
        then append ['-']
               (sshow_nat
                 (nat (Big_int.sub_big_int (Big_int.big_int_of_int 0) i)))
        else sshow_nat (nat i));;

let sshow_inta = ({sshow = sshow_int} : Big_int.big_int sshow);;

let sshow_nata = ({sshow = sshow_nat} : nat sshow);;

let rec defs_list _A
  = function [] -> []
    | (Fun (f, uu), uv) :: ts ->
        let fs = defs_list _A ts in
        (if member _A f fs then fs else f :: fs)
    | (Var uw, ux) :: ts -> defs_list _A ts;;

let rec fitLength
  uu x1 uv = match uu, x1, uv with uu, Zero_nat, uv -> []
    | a, Suc v, [] -> replicate (Suc v) a
    | a, Suc n, b :: asa -> b :: fitLength a n asa;;

let rec sum_lpoly
  uu x1 ys = match uu, x1, ys with uu, [], ys -> ys
    | uv, v :: va, [] -> v :: va
    | pl, x :: xs, y :: ys -> pl x y :: sum_lpoly pl xs ys;;

let rec isRight = function Left uu -> false
                  | Right uv -> true;;

let rec xml2label
  xml2name xml =
    bind (xmlManyElements ['l'; 'a'; 'b'; 'e'; 'l']
           (xmlOptions
             [(['n'; 'u'; 'm'; 'b'; 'e'; 'r'],
                xmlChange (xml2nat ['n'; 'u'; 'm'; 'b'; 'e'; 'r'])
                  (fun a -> Left a));
               (['n'; 'a'; 'm'; 'e'], xmlChange xml2name (fun a -> Right a));
               (['s'; 'h'; 'a'; 'r'; 'p'],
                 xmlChange xml2name (fun a -> Right a));
               (['l'; 'a'; 'b'; 'e'; 'l'; 'e'; 'd'; 'S'; 'y'; 'm'; 'b'; 'o';
                  'l'],
                 xmlChange xml2name (fun a -> Right a))])
           id xml)
      (fun xs ->
        (if forall isLeft xs then return (Left (map lefta xs))
          else (if forall isRight xs then return (Right (map righta xs))
                 else error ['m'; 'i'; 'x'; 'e'; 'd'; ' '; 'l'; 'a'; 'b'; 'e';
                              'l'; 's'])));;

let rec xml2rules
  xml2name = xmlManyElements ['r'; 'u'; 'l'; 'e'; 's'] (xml2rule xml2name) id;;

let rec xml_rules _A _C
  rls = Xml (['r'; 'u'; 'l'; 'e'; 's'], [],
              map (fun ((l, r) as a) ->
                    Xml (['r'; 'u'; 'l'; 'e'], [],
                          [Xml (['l'; 'h'; 's'], [], [xml_term _A _C l], None);
                            Xml (['r'; 'h'; 's'], [], [xml_term _A _C r],
                                  None)],
                          None))
                rls,
              None);;

let rec checkEdgRM _A _B
  rm rrm p precomputeEdges =
    (if null p
      then (match rrm
              with None ->
                (fun ((aa, b) as a) ->
                  let (s, t) = aa in
                  (fun ((u, v) as ab) -> gcMatchI _A _B (capRM _A _B rm t) u)
                    b)
              | Some a ->
                (fun ((ab, b) as aa) ->
                  let (s, t) = ab in
                  (fun ((u, v) as ac) ->
                    gcMatchI _A _B (capRM _A _B rm t) u &&
                      gcMatchI _A _B (capRM _A _B rrm u) t)
                    b))
      else let forward =
             map (fun t -> (t, capRM _A _B rm t))
               (remdups (eq_terma _A _B) (map snd p))
             in
           (match rrm
              with None ->
                (if precomputeEdges
                  then let edges =
                         flat_map
                           (fun ((s, t) as a) ->
                             map (fun ((u, v) as aa) ->
                                   ((t, u),
                                     gcMatchI _A _B
                                       (the (lookup (eq_terma _A _B) t forward))
                                       u))
                               p)
                           p
                         in
                       (fun ((aa, b) as a) ->
                         let (s, t) = aa in
                         (fun ((u, v) as ab) ->
                           (match lookup (eqb (eq_terma _A _B) (eq_terma _A _B))
                                    (t, u) edges
                              with None -> gcMatchI _A _B (capRM _A _B rm t) u
                              | Some ba -> ba))
                           b)
                  else (fun ((aa, b) as a) ->
                         let (s, t) = aa in
                         (fun ((u, v) as ab) ->
                           gcMatchI _A _B
                             ((match lookup (eq_terma _A _B) t forward
                                with None -> capRM _A _B rm t | Some ct -> ct))
                             u)
                           b))
              | Some x ->
                let backward =
                  map (fun u -> (u, capRM _A _B rrm u))
                    (remdups (eq_terma _A _B) (map fst p))
                  in
                (if precomputeEdges
                  then let edges =
                         flat_map
                           (fun ((s, t) as a) ->
                             map (fun ((u, v) as aa) ->
                                   ((t, u),
                                     gcMatchI _A _B
                                       (the (lookup (eq_terma _A _B) t forward))
                                       u
                                       && gcMatchI _A _B
    (the (lookup (eq_terma _A _B) u backward)) t))
                               p)
                           p
                         in
                       (fun ((aa, b) as a) ->
                         let (s, t) = aa in
                         (fun ((u, v) as ab) ->
                           (match lookup (eqb (eq_terma _A _B) (eq_terma _A _B))
                                    (t, u) edges
                              with None ->
                                gcMatchI _A _B (capRM _A _B rm t) u &&
                                  gcMatchI _A _B (capRM _A _B rrm u) t
                              | Some ba -> ba))
                           b)
                  else (fun ((aa, b) as a) ->
                         let (s, t) = aa in
                         (fun ((u, v) as ab) ->
                           gcMatchI _A _B
                             ((match lookup (eq_terma _A _B) t forward
                                with None -> capRM _A _B rm t | Some ct -> ct))
                             u
                             && gcMatchI _A _B
                                  ((match lookup (eq_terma _A _B) u backward
                                     with None -> capRM _A _B rrm u
                                     | Some cu -> cu))
                                  t)
                           b))));;

let rec partition
  p x1 = match p, x1 with p, [] -> ([], [])
    | p, x :: xs ->
        let a = partition p xs in
        let (yes, no) = a in
        (if p x then (x :: yes, no) else (yes, x :: no));;

let rec removeAll _A
  x xa1 = match x, xa1 with x, [] -> []
    | x, y :: xs ->
        (if eqop _A x y then removeAll _A x xs else y :: removeAll _A x xs);;

let rec rewrites_to (_A1, _A2) (_B1, _B2)
  x0 t r = match x0, t, r with
    [(s, (c, (sigma, rule)))], t, r ->
      rewrites (_A1, _A2) (_B1, _B2) (s, t) c sigma rule r
    | (s, (c, (sigma, rule))) :: (t, (d, (tau, rule'))) :: xs, u, r ->
        bind (rewrites (_A1, _A2) (_B1, _B2) (s, t) c sigma rule r)
          (fun x ->
            rewrites_to (_A1, _A2) (_B1, _B2) ((t, (d, (tau, rule'))) :: xs) u
              r);;

let rec check_loop_d (_A1, _A2) (_B1, _B2)
  x0 uu uv uw = match x0, uu, uv, uw with
    [], uu, uv, uw ->
      error ['T'; 'h'; 'e'; 'r'; 'e'; ' '; 'm'; 'u'; 's'; 't'; ' '; 'b'; 'e';
              ' '; 'a'; 't'; ' '; 'l'; 'e'; 'a'; 's'; 't'; ' '; 'o'; 'n'; 'e';
              ' '; 'r'; 'e'; 'w'; 'r'; 'i'; 't'; 'e'; ' '; 's'; 't'; 'e'; 'p']
    | v :: va, d, tau, r ->
        rewrites_to (_A1, _A2) (_B1, _B2) (v :: va)
          (ctxt_apply d (sub_apply (fst (hd (v :: va))) tau)) r;;

let rec check_loop (_A1, _A2) (_B1, _B2)
  (ts, (c, sigma)) r =
    bind (get_loop (_A1, _A2) (_B1, _B2) ts c sigma r)
      (fun ls -> check_loop_d (_A1, _A2) (_B1, _B2) ls c sigma r);;

let rec catch p eq = (match p with Left a -> eq a | Right a -> Right a);;

let rec checkLoop (_A1, _A2) (_B1, _B2)
  r (ts, (c, sub)) =
    catch ((match subst_safe _B1 _A1 [] sub (Sub (fun a -> Var a))
             with None ->
               error ['s'; 'u'; 'b'; 's'; 't'; 'i'; 't'; 'u'; 't'; 'i'; 'o';
                       'n'; ' '; 'n'; 'o'; 't'; ' '; 'u'; 'n'; 'i'; 'q'; 'u';
                       'e']
             | Some sigma ->
               check_loop (_A1, _A2) (_B1, _B2) (ts, (c, sigma)) r))
      (fun x ->
        error [['e'; 'r'; 'r'; 'o'; 'r'; ' '; 'i'; 'n'; ' '; 'c'; 'e'; 'r'; 't';
                 'i'; 'f'; 'i'; 'c'; 'a'; 't'; 'i'; 'o'; 'n'; ' '; 'o'; 'f';
                 ' '; 'l'; 'o'; 'o'; 'p'];
                x]);;

let rec vec_geI ge v w = forall (fun ((aa, b) as a) -> ge aa b) (zip v w);;

let rec mat_geI
  ge m1 m2 = forall (fun ((aa, b) as a) -> vec_geI ge aa b) (zip m1 m2);;

let rec vec_pre_gtI gt v w = any (fun ((aa, b) as a) -> gt aa b) (zip v w);;

let rec mat_pre_gtI
  gt m1 m2 = any (fun ((aa, b) as a) -> vec_pre_gtI gt aa b) (zip m1 m2);;

let rec mat_gtI ge gt m1 m2 = mat_geI ge m1 m2 && mat_pre_gtI gt m1 m2;;

let rec wf_vecI n x = eqop eq_nata (size_list x) n;;

let rec checkSupterm _A _B
  x0 uu = match x0, uu with Var x, uu -> false
    | Fun (f, ts), s ->
        member (eq_terma _A _B) s ts ||
          any (fun t -> checkSupterm _A _B t s) ts;;

let rec check_supt (_F1, _F2) (_V1, _V2)
  lr = catch (checkB (checkSupterm _F1 _V1 (fst lr) (snd lr))
               [['t'; 'h'; 'e'; ' '; 'r'; 'h'; 's'; ' '; 'i'; 's'; ' '; 'n';
                  'o'; 't'; ' '; 'a'; ' '; 'p'; 'r'; 'o'; 'p'; 'e'; 'r'; ' ';
                  's'; 'u'; 'b'; 't'; 'e'; 'r'; 'm'; ' '; 'o'; 'f'; ' '; 't';
                  'h'; 'e'; ' '; 'l'; 'h'; 's']])
         (fun x ->
           error (concat
                    [['C'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'o';
                       'r'; 'i'; 'e'; 'n'; 't'; ' '; 'r'; 'u'; 'l'; 'e'; ' '];
                      sshow_term _F2 _V2 (fst lr); [' '; '-'; '>'; ' '];
                      sshow_term _F2 _V2 (snd lr)]
                   :: x));;

let rec sshow_char x = [x];;

let sshow_chara = ({sshow = sshow_char} : char sshow);;

let rec sshow_list _A xs = flat_map (sshow _A) xs;;

let sshow_lista _A = ({sshow = sshow_list _A} : ('a list) sshow);;

let sshow_terma _A _B = ({sshow = sshow_term _A _B} : ('a, 'b) term sshow);;

let rec transpose
  nr x1 = match nr, x1 with nr, [] -> replicate nr []
    | nr, v :: m ->
        map (fun ((aa, b) as a) -> aa :: b) (zip v (transpose nr m));;

let rec scalar_prodI
  ze pl ti v w = foldr (fun xy -> pl (ti (fst xy) (snd xy))) (zip v w) ze;;

let rec matT_vec_mulI
  ze pl ti m v = map (fun w -> scalar_prodI ze pl ti w v) m;;

let rec mat_mulI
  ze pl ti nr m1 m2 = map (matT_vec_mulI ze pl ti (transpose nr m1)) m2;;

let rec list_union _A
  x0 ys = match x0, ys with [], ys -> ys
    | x :: xs, ys ->
        let zs = list_union _A xs ys in
        (if member _A x zs then zs else x :: zs);;

let rec domain_case
  f1 f2 f3 f4 x4 = match f1, f2, f3, f4, x4 with
    f1, f2, f3, f4, Arc_mat nat -> f4 nat
    | f1, f2, f3, f4, Int_mat nat -> f3 nat
    | f1, f2, f3, f4, Arctic -> f2
    | f1, f2, f3, f4, Natural -> f1;;

type ('a, 'b) lab = Lab of ('a, 'b) lab * 'b |
  FunLab of ('a, 'b) lab * ('a, 'b) lab list | UnLab of 'a;;

let rec id_labeling uu cs = Left cs;;

let rec stripXMLHeader
  s = (if eqop (eq_lista eq_char)
            (take (nat (Big_int.big_int_of_int 2)) (trim s)) ['<'; '?']
        then bind (scanUpto ['?'; '>'] s)
               (fun ((header1, s1) as a) ->
                 bind (stripXMLHeader s1)
                   (fun ((header2, s2) as aa) ->
                     return (append header1 header2, trim s2)))
        else return ([], trim s));;

let rec debug i t x = x;;

let rec parseXMLDoc
  s = debug ['0']
        ['p'; 'a'; 'r'; 's'; 'i'; 'n'; 'g'; ' '; 't'; 'x'; 't'; ' '; 't'; 'o';
          ' '; 'x'; 'm'; 'l']
        (bind (stripXMLHeader s)
          (fun ((header, rest) as a) ->
            bind (parseXML rest)
              (fun ((xml, enda) as aa) ->
                (if null enda then return (Xmldoc (header, xml))
                  else error (append
                               ['t'; 'h'; 'e'; 'r'; 'e'; ' '; 'i'; 's'; ' ';
                                 't'; 'e'; 'x'; 't'; ' '; 'a'; 'f'; 't'; 'e';
                                 'r'; ' '; 'r'; 'o'; 'o'; 't'; ' '; 'e'; 'l';
                                 'e'; 'm'; 'e'; 'n'; 't'; ':'; ' ']
                               (append newline enda))))));;

let rec xml4elements
  reqName p1 p2 p3 p4 combi (Xml (name, atts, cs, text)) =
    (if eqop (eq_lista eq_char) name reqName &&
          (null atts &&
            (eqop eq_nata (size_list cs) (nat (Big_int.big_int_of_int 4)) &&
              is_none text))
      then bind (p1 (nth cs Zero_nat))
             (fun a ->
               bind (p2 (nth cs one_nat))
                 (fun b ->
                   bind (p3 (nth cs (nat (Big_int.big_int_of_int 2))))
                     (fun c ->
                       bind (p4 (nth cs (nat (Big_int.big_int_of_int 3))))
                         (comp return (combi a b c)))))
      else error (concat
                   [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 't'; 'r';
                      'a'; 'n'; 's'; 'f'; 'o'; 'r'; 'm'; ' '; 't'; 'h'; 'e';
                      ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ';
                      'x'; 'm'; 'l'; ' '; 'e'; 'l'; 'e'; 'm'; 'e'; 'n'; 't';
                      ' '; '('; 'e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '];
                     reqName; [')']; newline;
                     sshow_xml (Xml (name, atts, cs, text))]));;

let rec xml0elements
  reqName combi (Xml (name, atts, cs, text)) =
    (if eqop (eq_lista eq_char) name reqName &&
          (null atts && (null cs && is_none text))
      then return combi
      else error (concat
                   [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 't'; 'r';
                      'a'; 'n'; 's'; 'f'; 'o'; 'r'; 'm'; ' '; 't'; 'h'; 'e';
                      ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ';
                      'x'; 'm'; 'l'; ' '; 'e'; 'l'; 'e'; 'm'; 'e'; 'n'; 't';
                      ' '; '('; 'e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '];
                     reqName; [')']; newline;
                     sshow_xml (Xml (name, atts, cs, text))]));;

let rec xml2context
  xml2name x =
    xmlOptions
      [(['b'; 'o'; 'x'], xml0elements ['b'; 'o'; 'x'] Hole);
        (['f'; 'u'; 'n'; 'C'; 'o'; 'n'; 't'; 'e'; 'x'; 't'],
          xml4elements ['f'; 'u'; 'n'; 'C'; 'o'; 'n'; 't'; 'e'; 'x'; 't']
            xml2name
            (xmlManyElements ['b'; 'e'; 'f'; 'o'; 'r'; 'e'] (xml2term xml2name)
              id)
            (xml2context xml2name)
            (xmlManyElements ['a'; 'f'; 't'; 'e'; 'r'] (xml2term xml2name) id)
            (fun a b c d -> More (a, b, c, d)))]
      x;;

type arithFun = Arg of nat | Const of nat | Sum of arithFun list |
  Max of arithFun list | Min of arithFun list | Prod of arithFun list;;

type ('a, 'b) sl_inter =
  SL_Inter of nat * ((('a, 'b) lab * nat) * arithFun) list;;

type arctic = MinInfty | Num_arc of Big_int.big_int;;

type 'a redpair = Nat_carrier of (('a * nat) * (nat * nat list)) list |
  Int_carrier of (('a * nat) * (Big_int.big_int * Big_int.big_int list)) list |
  Arctic_carrier of (('a * nat) * (arctic * arctic list)) list |
  Int_mat_carrier of
    nat * (('a * nat) *
            ((Big_int.big_int list) list *
              ((Big_int.big_int list) list) list)) list
  | Arc_mat_carrier of
      nat * (('a * nat) *
              ((arctic list) list * ((arctic list) list) list)) list;;

type ('a, 'b, 'c) dPProof = PisEmpty |
  RedPairProc of
    ('a, 'b) lab redpair *
      ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
      ('a, 'b, 'c) dPProof
  | RedPairUrProc of
      ('a, 'b) lab redpair *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ('a, 'b, 'c) dPProof
  | DepGraphProc of
      (('a, 'b, 'c) dPProof option *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list) list
  | MonoRedPairProc of
      ('a, 'b) lab redpair *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ('a, 'b, 'c) dPProof
  | MonoRedPairUrProc of
      ('a, 'b) lab redpair *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ('a, 'b, 'c) dPProof
  | UnlabProc of
      ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ('a, 'b, 'c) dPProof
  | SemlabProc of
      ('a, 'b) sl_inter *
        (('a, 'b) lab -> nat list -> ('b, (('a, 'b) lab list)) either) *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ('a, 'b, 'c) dPProof
  | RootlabProc of
      ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ('a, 'b, 'c) dPProof
  | SpscProc of
      ('a, 'b) lab spL *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ('a, 'b, 'c) dPProof
  | SizeChangeSubTerm of
      (((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) *
        ((nat * nat) list * (nat * nat) list)) list
  | SizeChangeRedPair of
      ('a, 'b) lab redpair *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        (((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) *
          ((nat * nat) list * (nat * nat) list)) list;;

let rec xml2arith_fun
  xml = xmlOptions
          [(['n'; 'a'; 't'; 'u'; 'r'; 'a'; 'l'],
             xmlChange (xml2nat ['n'; 'a'; 't'; 'u'; 'r'; 'a'; 'l'])
               (fun a -> Const a));
            (['v'; 'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e'],
              xmlChange (xml2nat ['v'; 'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e'])
                (fun n -> Arg (minus_nat n one_nat)));
            (['s'; 'u'; 'm'],
              xmlManyElements ['s'; 'u'; 'm'] xml2arith_fun (fun a -> Sum a));
            (['p'; 'r'; 'o'; 'd'; 'u'; 'c'; 't'],
              xmlManyElements ['p'; 'r'; 'o'; 'd'; 'u'; 'c'; 't'] xml2arith_fun
                (fun a -> Prod a));
            (['m'; 'i'; 'n'; 'i'; 'm'; 'u'; 'm'],
              xmlManyElements ['m'; 'i'; 'n'; 'i'; 'm'; 'u'; 'm'] xml2arith_fun
                (fun a -> Min a));
            (['m'; 'a'; 'x'; 'i'; 'm'; 'u'; 'm'],
              xmlManyElements ['m'; 'a'; 'x'; 'i'; 'm'; 'u'; 'm'] xml2arith_fun
                (fun a -> Max a))]
          xml;;

let rec xml2sl_inter
  xml2name =
    xml1element ['m'; 'o'; 'd'; 'e'; 'l']
      (xmlOptions
        [(['f'; 'i'; 'n'; 'i'; 't'; 'e'; 'M'; 'o'; 'd'; 'e'; 'l'],
           xml1manyElements
             ['f'; 'i'; 'n'; 'i'; 't'; 'e'; 'M'; 'o'; 'd'; 'e'; 'l']
             (xmlChange
               (xml2nat ['c'; 'a'; 'r'; 'r'; 'i'; 'e'; 'r'; 'S'; 'i'; 'z'; 'e'])
               (fun n -> minus_nat n one_nat))
             (xml3elements ['i'; 'n'; 't'; 'e'; 'r'; 'p'; 'r'; 'e'; 't']
               xml2name (xml2nat ['a'; 'r'; 'i'; 't'; 'y']) xml2arith_fun
               (fun f a -> (fun b -> ((f, a), b))))
             (fun n xs -> Some (SL_Inter (n, xs))));
          (['r'; 'o'; 'o'; 't'; 'L'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n'; 'g'],
            xml0elements
              ['r'; 'o'; 'o'; 't'; 'L'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n'; 'g']
              None)])
      id;;

let rec xml5elements
  reqName p1 p2 p3 p4 p5 combi (Xml (name, atts, cs, text)) =
    (if eqop (eq_lista eq_char) name reqName &&
          (null atts &&
            (eqop eq_nata (size_list cs) (nat (Big_int.big_int_of_int 5)) &&
              is_none text))
      then bind (p1 (nth cs Zero_nat))
             (fun a ->
               bind (p2 (nth cs one_nat))
                 (fun b ->
                   bind (p3 (nth cs (nat (Big_int.big_int_of_int 2))))
                     (fun c ->
                       bind (p4 (nth cs (nat (Big_int.big_int_of_int 3))))
                         (fun d ->
                           bind (p5 (nth cs (nat (Big_int.big_int_of_int 4))))
                             (comp return (combi a b c d))))))
      else error (concat
                   [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 't'; 'r';
                      'a'; 'n'; 's'; 'f'; 'o'; 'r'; 'm'; ' '; 't'; 'h'; 'e';
                      ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ';
                      'x'; 'm'; 'l'; ' '; 'e'; 'l'; 'e'; 'm'; 'e'; 'n'; 't';
                      ' '; '('; 'e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '];
                     reqName; [')']; newline;
                     sshow_xml (Xml (name, atts, cs, text))]));;

let arc_mat_name : char list =
  ['m'; 'a'; 't'; 'r'; 'i'; 'c'; 'e'; 's'; ' '; 'o'; 'v'; 'e'; 'r'; ' '; 't';
    'h'; 'e'; ' '; 'a'; 'r'; 'c'; 't'; 'i'; 'c'; ' '; 's'; 'e'; 'm'; 'i'; '-';
    'r'; 'i'; 'n'; 'g']

let rec mat_arc_posI ap m = ap (nth (nth m Zero_nat) Zero_nat);;

let int_mat_name : char list =
  ['m'; 'a'; 't'; 'r'; 'i'; 'c'; 'e'; 's'; ' '; 'o'; 'v'; 'e'; 'r'; ' '; 't';
    'h'; 'e'; ' '; 'n'; 'a'; 't'; 'u'; 'r'; 'a'; 'l'; 's'; ' '; 'w'; 'i'; 't';
    'h'; ' '; 'n'; 'e'; 'g'; 'a'; 't'; 'i'; 'v'; 'e'; ' '; 'c'; 'o'; 'n'; 's';
    't'; 'a'; 'n'; 't'; 's']

let rec vec_max0I a = map a;;

let rec mat_max0I mz = map (vec_max0I mz);;

let rec vec_plusI pl v w = map (fun xy -> pl (fst xy) (snd xy)) (zip v w);;

let rec mat_plusI
  pl m1 m2 = map (fun uv -> vec_plusI pl (fst uv) (snd uv)) (zip m1 m2);;

let rec wf_mat_rcI
  nr nc m = eqop eq_nata (size_list m) nc && forall (wf_vecI nr) m;;

let rec xml2vec_coeff
  xml2coeff =
    xmlManyElements ['v'; 'e'; 'c'; 't'; 'o'; 'r']
      (xml1element ['c'; 'o'; 'e'; 'f'; 'f'; 'i'; 'c'; 'i'; 'e'; 'n'; 't']
        xml2coeff id)
      id;;

let rec xml2mat_coeff
  xml2coeff =
    xmlManyElements ['m'; 'a'; 't'; 'r'; 'i'; 'x'] (xml2vec_coeff xml2coeff)
      id;;

let rec pos_arctic
  = function MinInfty -> false
    | Num_arc n -> Big_int.le_big_int (Big_int.big_int_of_int 0) n;;

let arctic_name : char list =
  ['a'; 'r'; 'c'; 't'; 'i'; 'c'; ' '; 's'; 'e'; 'm'; 'i'; 'r'; 'i'; 'n'; 'g']

let rec gt_arctic
  uu x1 = match uu, x1 with uu, MinInfty -> true
    | MinInfty, Num_arc uv -> false
    | Num_arc x, Num_arc y -> Big_int.lt_big_int y x;;

let rec geq_arctic
  xa x1 = match xa, x1 with xa, MinInfty -> true
    | MinInfty, Num_arc uu -> false
    | Num_arc x, Num_arc y -> Big_int.le_big_int y x;;

let rec mult_arctic
  x0 ya = match x0, ya with MinInfty, ya -> MinInfty
    | Num_arc v, MinInfty -> MinInfty
    | Num_arc x, Num_arc y -> Num_arc (Big_int.add_big_int x y);;

let rec plus_arctic
  x0 ya = match x0, ya with MinInfty, ya -> ya
    | Num_arc v, MinInfty -> Num_arc v
    | Num_arc x, Num_arc y -> Num_arc (max ord_int x y);;

let rec eq_arctic
  x0 x1 = match x0, x1 with Num_arc a, MinInfty -> false
    | MinInfty, Num_arc a -> false
    | Num_arc int, Num_arc int' -> Big_int.eq_big_int int int'
    | MinInfty, MinInfty -> true;;

let eq_arctica = ({eq = eq_arctic} : arctic eq);;

let rec xml2arctic_coeff
  x = xmlOptions
        [(['i'; 'n'; 't'; 'e'; 'g'; 'e'; 'r'],
           xmlChange (xml2int ['i'; 'n'; 't'; 'e'; 'g'; 'e'; 'r'])
             (fun a -> Num_arc a));
          (['m'; 'i'; 'n'; 'u'; 's'; 'I'; 'n'; 'f'; 'i'; 'n'; 'i'; 't'; 'y'],
            xml0elements
              ['m'; 'i'; 'n'; 'u'; 's'; 'I'; 'n'; 'f'; 'i'; 'n'; 'i'; 't'; 'y']
              MinInfty)]
        x;;

let rec get_comp_times
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = ti;;

let rec get_comp_plus
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = pl;;

let rec get_comp_1
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = on;;

let int_name : char list =
  ['n'; 'a'; 't'; 'u'; 'r'; 'a'; 'l'; 's'; ' '; 'w'; 'i'; 't'; 'h'; ' '; 'n';
    'e'; 'g'; 'a'; 't'; 'i'; 'v'; 'e'; ' '; 'c'; 'o'; 'n'; 's'; 't'; 'a'; 'n';
    't'; 's']

let rec get_comp_0
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = ze;;

let rec poly_to_lpoly _A
  uu uv uw ux x4 = match uu, uv, uw, ux, x4 with
    uu, uv, uw, ux, PNum i -> return (i, [])
    | ze, on, pl, ti, PVar x -> return (ze, append (replicate x ze) [on])
    | ze, on, pl, ti, PSum [p] -> poly_to_lpoly _A ze on pl ti p
    | ze, on, pl, ti, PSum (p :: q :: ps) ->
        bind (poly_to_lpoly _A ze on pl ti p)
          (fun ((cp, ncp) as a) ->
            bind (poly_to_lpoly _A ze on pl ti (PSum (q :: ps)))
              (fun ((cq, ncq) as aa) ->
                return (pl cp cq, sum_lpoly pl ncp ncq)))
    | ze, on, pl, ti, PMult [p] -> poly_to_lpoly _A ze on pl ti p
    | ze, on, pl, ti, PMult (p :: q :: ps) ->
        bind (poly_to_lpoly _A ze on pl ti p)
          (fun ((cp, ncp) as a) ->
            bind (poly_to_lpoly _A ze on pl ti (PMult (q :: ps)))
              (fun ((cq, ncq) as aa) ->
                (if forall (eqop _A ze) ncp
                  then return (ti cp cq, map (ti cp) ncq)
                  else (if forall (eqop _A ze) ncq
                         then return (ti cp cq, map (fun x -> ti x cq) ncp)
                         else error ['n'; 'o'; 'n'; '-'; 'l'; 'i'; 'n'; 'e';
                                      'a'; 'r'; ' '; 'p'; 'o'; 'l'; 'y'; 'n';
                                      'o'; 'm'; 'i'; 'a'; 'l'; 's'; ' '; 'n';
                                      'o'; 't'; ' '; 's'; 'u'; 'p'; 'p'; 'o';
                                      'r'; 't'; 'e'; 'd']))));;

let rec xml2int_coeff x = xml2int ['i'; 'n'; 't'; 'e'; 'g'; 'e'; 'r'] x;;

let rec xml2redpair
  xml2name =
    xmlOptions
      [(['r'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'],
         xml1element ['r'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r']
           (xml1CmanyElements
             ['i'; 'n'; 't'; 'e'; 'r'; 'p'; 'r'; 'e'; 't'; 'a'; 't'; 'i'; 'o';
               'n']
             (xml1element ['t'; 'y'; 'p'; 'e']
               (xml2elements ['p'; 'o'; 'l'; 'y'; 'n'; 'o'; 'm'; 'i'; 'a'; 'l']
                 (xml1element ['d'; 'o'; 'm'; 'a'; 'i'; 'n']
                   (xmlOptions
                     [(['n'; 'a'; 't'; 'u'; 'r'; 'a'; 'l'; 's'],
                        xml0elements ['n'; 'a'; 't'; 'u'; 'r'; 'a'; 'l'; 's']
                          Natural);
                       (['a'; 'r'; 'c'; 't'; 'i'; 'c'],
                         xml0elements ['a'; 'r'; 'c'; 't'; 'i'; 'c'] Arctic);
                       (['a'; 'r'; 'c'; 't'; 'i'; 'c'; 'B'; 'e'; 'l'; 'o'; 'w';
                          'Z'; 'e'; 'r'; 'o'],
                         xml0elements
                           ['a'; 'r'; 'c'; 't'; 'i'; 'c'; 'B'; 'e'; 'l'; 'o';
                             'w'; 'Z'; 'e'; 'r'; 'o']
                           Arctic);
                       (['m'; 'a'; 't'; 'r'; 'i'; 'c'; 'e'; 's'],
                         xml3elements ['m'; 'a'; 't'; 'r'; 'i'; 'c'; 'e'; 's']
                           (xml2nat
                             ['d'; 'i'; 'm'; 'e'; 'n'; 's'; 'i'; 'o'; 'n'])
                           (xml2nat
                             ['s'; 't'; 'r'; 'i'; 'c'; 't'; 'D'; 'i'; 'm'; 'e';
                               'n'; 's'; 'i'; 'o'; 'n'])
                           (xml1element ['d'; 'o'; 'm'; 'a'; 'i'; 'n']
                             (xmlOptions
                               [(['n'; 'a'; 't'; 'u'; 'r'; 'a'; 'l'; 's'],
                                  xml0elements
                                    ['n'; 'a'; 't'; 'u'; 'r'; 'a'; 'l'; 's']
                                    true);
                                 (['a'; 'r'; 'c'; 't'; 'i'; 'c'],
                                   xml0elements ['a'; 'r'; 'c'; 't'; 'i'; 'c']
                                     false);
                                 (['a'; 'r'; 'c'; 't'; 'i'; 'c'; 'B'; 'e'; 'l';
                                    'o'; 'w'; 'Z'; 'e'; 'r'; 'o'],
                                   xml0elements
                                     ['a'; 'r'; 'c'; 't'; 'i'; 'c'; 'B'; 'e';
                                       'l'; 'o'; 'w'; 'Z'; 'e'; 'r'; 'o']
                                     false)])
                             id)
                           (fun di sd doa ->
                             (if doa then Int_mat di else Arc_mat di)))])
                   id)
                 (xml2nat ['d'; 'e'; 'g'; 'r'; 'e'; 'e'])
                 (fun typea uu -> typea))
               id)
             (fun a ->
               (match a
                  with Natural ->
                    xml3elements ['i'; 'n'; 't'; 'e'; 'r'; 'p'; 'r'; 'e'; 't']
                      xml2name (xml2nat ['a'; 'r'; 'i'; 't'; 'y'])
                      (fun x ->
                        bind (xml2poly xml2int_coeff x)
                          (poly_to_lpoly eq_int
                            (get_comp_0
                              ((fun xa -> true),
                                ((Big_int.big_int_of_int 0),
                                  ((Big_int.big_int_of_int 1),
                                    (Big_int.add_big_int,
                                      (Big_int.mult_big_int,
(max ord_int (Big_int.big_int_of_int 0),
  ((fun xa y -> Big_int.le_big_int y xa),
    ((fun xa y -> Big_int.lt_big_int y xa),
      (true, (int_name, (fun xa -> true))))))))))))
                            (get_comp_1
                              ((fun xa -> true),
                                ((Big_int.big_int_of_int 0),
                                  ((Big_int.big_int_of_int 1),
                                    (Big_int.add_big_int,
                                      (Big_int.mult_big_int,
(max ord_int (Big_int.big_int_of_int 0),
  ((fun xa y -> Big_int.le_big_int y xa),
    ((fun xa y -> Big_int.lt_big_int y xa),
      (true, (int_name, (fun xa -> true))))))))))))
                            (get_comp_plus
                              ((fun xa -> true),
                                ((Big_int.big_int_of_int 0),
                                  ((Big_int.big_int_of_int 1),
                                    (Big_int.add_big_int,
                                      (Big_int.mult_big_int,
(max ord_int (Big_int.big_int_of_int 0),
  ((fun xa y -> Big_int.le_big_int y xa),
    ((fun xa y -> Big_int.lt_big_int y xa),
      (true, (int_name, (fun xa -> true))))))))))))
                            (get_comp_times
                              ((fun xa -> true),
                                ((Big_int.big_int_of_int 0),
                                  ((Big_int.big_int_of_int 1),
                                    (Big_int.add_big_int,
                                      (Big_int.mult_big_int,
(max ord_int (Big_int.big_int_of_int 0),
  ((fun xa y -> Big_int.le_big_int y xa),
    ((fun xa y -> Big_int.lt_big_int y xa),
      (true, (int_name, (fun xa -> true))))))))))))))
                      (fun f aa poly ->
                        Left (Left ((f, aa),
                                     (fst poly,
                                       fitLength
 (get_comp_0
   ((fun x -> true),
     ((Big_int.big_int_of_int 0),
       ((Big_int.big_int_of_int 1),
         (Big_int.add_big_int,
           (Big_int.mult_big_int,
             (max ord_int (Big_int.big_int_of_int 0),
               ((fun x y -> Big_int.le_big_int y x),
                 ((fun x y -> Big_int.lt_big_int y x),
                   (true, (int_name, (fun x -> true))))))))))))
 aa (snd poly)))))
                  | Arctic ->
                    xml3elements ['i'; 'n'; 't'; 'e'; 'r'; 'p'; 'r'; 'e'; 't']
                      xml2name (xml2nat ['a'; 'r'; 'i'; 't'; 'y'])
                      (fun x ->
                        bind (xml2poly xml2arctic_coeff x)
                          (poly_to_lpoly eq_arctica
                            (get_comp_0
                              ((fun xa -> true),
                                (MinInfty,
                                  (Num_arc (Big_int.big_int_of_int 0),
                                    (plus_arctic,
                                      (mult_arctic,
(id, (geq_arctic, (gt_arctic, (false, (arctic_name, pos_arctic)))))))))))
                            (get_comp_1
                              ((fun xa -> true),
                                (MinInfty,
                                  (Num_arc (Big_int.big_int_of_int 0),
                                    (plus_arctic,
                                      (mult_arctic,
(id, (geq_arctic, (gt_arctic, (false, (arctic_name, pos_arctic)))))))))))
                            (get_comp_plus
                              ((fun xa -> true),
                                (MinInfty,
                                  (Num_arc (Big_int.big_int_of_int 0),
                                    (plus_arctic,
                                      (mult_arctic,
(id, (geq_arctic, (gt_arctic, (false, (arctic_name, pos_arctic)))))))))))
                            (get_comp_times
                              ((fun xa -> true),
                                (MinInfty,
                                  (Num_arc (Big_int.big_int_of_int 0),
                                    (plus_arctic,
                                      (mult_arctic,
(id, (geq_arctic, (gt_arctic, (false, (arctic_name, pos_arctic)))))))))))))
                      (fun f aa poly ->
                        Left (Right
                               ((f, aa),
                                 (fst poly,
                                   fitLength
                                     (get_comp_0
                                       ((fun x -> true),
 (MinInfty,
   (Num_arc (Big_int.big_int_of_int 0),
     (plus_arctic,
       (mult_arctic,
         (id, (geq_arctic,
                (gt_arctic, (false, (arctic_name, pos_arctic)))))))))))
                                     aa (snd poly)))))
                  | Int_mat n ->
                    xml3elements ['i'; 'n'; 't'; 'e'; 'r'; 'p'; 'r'; 'e'; 't']
                      xml2name (xml2nat ['a'; 'r'; 'i'; 't'; 'y'])
                      (fun x ->
                        bind (xml2poly (xml2mat_coeff xml2int_coeff) x)
                          (poly_to_lpoly (eq_lista (eq_lista eq_int))
                            (get_comp_0
                              (wf_mat_rcI (Suc (minus_nat n one_nat))
                                 (Suc (minus_nat n one_nat)),
                                (mat0I (Big_int.big_int_of_int 0)
                                   (Suc (minus_nat n one_nat))
                                   (Suc (minus_nat n one_nat)),
                                  (mat1I (Big_int.big_int_of_int 0)
                                     (Big_int.big_int_of_int 1)
                                     (Suc (minus_nat n one_nat)),
                                    (mat_plusI Big_int.add_big_int,
                                      (mat_mulI (Big_int.big_int_of_int 0)
 Big_int.add_big_int Big_int.mult_big_int (Suc (minus_nat n one_nat)),
(mat_max0I (max ord_int (Big_int.big_int_of_int 0)),
  (mat_geI (fun xa y -> Big_int.le_big_int y xa),
    (mat_gtI (fun xa y -> Big_int.le_big_int y xa)
       (fun xa y -> Big_int.lt_big_int y xa),
      (true, (int_mat_name, mat_arc_posI (fun xa -> true))))))))))))
                            (get_comp_1
                              (wf_mat_rcI (Suc (minus_nat n one_nat))
                                 (Suc (minus_nat n one_nat)),
                                (mat0I (Big_int.big_int_of_int 0)
                                   (Suc (minus_nat n one_nat))
                                   (Suc (minus_nat n one_nat)),
                                  (mat1I (Big_int.big_int_of_int 0)
                                     (Big_int.big_int_of_int 1)
                                     (Suc (minus_nat n one_nat)),
                                    (mat_plusI Big_int.add_big_int,
                                      (mat_mulI (Big_int.big_int_of_int 0)
 Big_int.add_big_int Big_int.mult_big_int (Suc (minus_nat n one_nat)),
(mat_max0I (max ord_int (Big_int.big_int_of_int 0)),
  (mat_geI (fun xa y -> Big_int.le_big_int y xa),
    (mat_gtI (fun xa y -> Big_int.le_big_int y xa)
       (fun xa y -> Big_int.lt_big_int y xa),
      (true, (int_mat_name, mat_arc_posI (fun xa -> true))))))))))))
                            (get_comp_plus
                              (wf_mat_rcI (Suc (minus_nat n one_nat))
                                 (Suc (minus_nat n one_nat)),
                                (mat0I (Big_int.big_int_of_int 0)
                                   (Suc (minus_nat n one_nat))
                                   (Suc (minus_nat n one_nat)),
                                  (mat1I (Big_int.big_int_of_int 0)
                                     (Big_int.big_int_of_int 1)
                                     (Suc (minus_nat n one_nat)),
                                    (mat_plusI Big_int.add_big_int,
                                      (mat_mulI (Big_int.big_int_of_int 0)
 Big_int.add_big_int Big_int.mult_big_int (Suc (minus_nat n one_nat)),
(mat_max0I (max ord_int (Big_int.big_int_of_int 0)),
  (mat_geI (fun xa y -> Big_int.le_big_int y xa),
    (mat_gtI (fun xa y -> Big_int.le_big_int y xa)
       (fun xa y -> Big_int.lt_big_int y xa),
      (true, (int_mat_name, mat_arc_posI (fun xa -> true))))))))))))
                            (get_comp_times
                              (wf_mat_rcI (Suc (minus_nat n one_nat))
                                 (Suc (minus_nat n one_nat)),
                                (mat0I (Big_int.big_int_of_int 0)
                                   (Suc (minus_nat n one_nat))
                                   (Suc (minus_nat n one_nat)),
                                  (mat1I (Big_int.big_int_of_int 0)
                                     (Big_int.big_int_of_int 1)
                                     (Suc (minus_nat n one_nat)),
                                    (mat_plusI Big_int.add_big_int,
                                      (mat_mulI (Big_int.big_int_of_int 0)
 Big_int.add_big_int Big_int.mult_big_int (Suc (minus_nat n one_nat)),
(mat_max0I (max ord_int (Big_int.big_int_of_int 0)),
  (mat_geI (fun xa y -> Big_int.le_big_int y xa),
    (mat_gtI (fun xa y -> Big_int.le_big_int y xa)
       (fun xa y -> Big_int.lt_big_int y xa),
      (true, (int_mat_name, mat_arc_posI (fun xa -> true))))))))))))))
                      (fun f aa poly ->
                        Right (Left ((f, aa),
                                      (fst poly,
fitLength
  (get_comp_0
    (wf_mat_rcI (Suc (minus_nat n one_nat)) (Suc (minus_nat n one_nat)),
      (mat0I (Big_int.big_int_of_int 0) (Suc (minus_nat n one_nat))
         (Suc (minus_nat n one_nat)),
        (mat1I (Big_int.big_int_of_int 0) (Big_int.big_int_of_int 1)
           (Suc (minus_nat n one_nat)),
          (mat_plusI Big_int.add_big_int,
            (mat_mulI (Big_int.big_int_of_int 0) Big_int.add_big_int
               Big_int.mult_big_int (Suc (minus_nat n one_nat)),
              (mat_max0I (max ord_int (Big_int.big_int_of_int 0)),
                (mat_geI (fun x y -> Big_int.le_big_int y x),
                  (mat_gtI (fun x y -> Big_int.le_big_int y x)
                     (fun x y -> Big_int.lt_big_int y x),
                    (true,
                      (int_mat_name, mat_arc_posI (fun x -> true))))))))))))
  aa (snd poly)))))
                  | Arc_mat n ->
                    xml3elements ['i'; 'n'; 't'; 'e'; 'r'; 'p'; 'r'; 'e'; 't']
                      xml2name (xml2nat ['a'; 'r'; 'i'; 't'; 'y'])
                      (fun x ->
                        bind (xml2poly (xml2mat_coeff xml2arctic_coeff) x)
                          (poly_to_lpoly (eq_lista (eq_lista eq_arctica))
                            (get_comp_0
                              (wf_mat_rcI (Suc (minus_nat n one_nat))
                                 (Suc (minus_nat n one_nat)),
                                (mat0I MinInfty (Suc (minus_nat n one_nat))
                                   (Suc (minus_nat n one_nat)),
                                  (mat1I MinInfty
                                     (Num_arc (Big_int.big_int_of_int 0))
                                     (Suc (minus_nat n one_nat)),
                                    (mat_plusI plus_arctic,
                                      (mat_mulI MinInfty plus_arctic mult_arctic
 (Suc (minus_nat n one_nat)),
(mat_max0I id,
  (mat_geI geq_arctic,
    (mat_geI gt_arctic,
      (false, (arc_mat_name, mat_arc_posI pos_arctic)))))))))))
                            (get_comp_1
                              (wf_mat_rcI (Suc (minus_nat n one_nat))
                                 (Suc (minus_nat n one_nat)),
                                (mat0I MinInfty (Suc (minus_nat n one_nat))
                                   (Suc (minus_nat n one_nat)),
                                  (mat1I MinInfty
                                     (Num_arc (Big_int.big_int_of_int 0))
                                     (Suc (minus_nat n one_nat)),
                                    (mat_plusI plus_arctic,
                                      (mat_mulI MinInfty plus_arctic mult_arctic
 (Suc (minus_nat n one_nat)),
(mat_max0I id,
  (mat_geI geq_arctic,
    (mat_geI gt_arctic,
      (false, (arc_mat_name, mat_arc_posI pos_arctic)))))))))))
                            (get_comp_plus
                              (wf_mat_rcI (Suc (minus_nat n one_nat))
                                 (Suc (minus_nat n one_nat)),
                                (mat0I MinInfty (Suc (minus_nat n one_nat))
                                   (Suc (minus_nat n one_nat)),
                                  (mat1I MinInfty
                                     (Num_arc (Big_int.big_int_of_int 0))
                                     (Suc (minus_nat n one_nat)),
                                    (mat_plusI plus_arctic,
                                      (mat_mulI MinInfty plus_arctic mult_arctic
 (Suc (minus_nat n one_nat)),
(mat_max0I id,
  (mat_geI geq_arctic,
    (mat_geI gt_arctic,
      (false, (arc_mat_name, mat_arc_posI pos_arctic)))))))))))
                            (get_comp_times
                              (wf_mat_rcI (Suc (minus_nat n one_nat))
                                 (Suc (minus_nat n one_nat)),
                                (mat0I MinInfty (Suc (minus_nat n one_nat))
                                   (Suc (minus_nat n one_nat)),
                                  (mat1I MinInfty
                                     (Num_arc (Big_int.big_int_of_int 0))
                                     (Suc (minus_nat n one_nat)),
                                    (mat_plusI plus_arctic,
                                      (mat_mulI MinInfty plus_arctic mult_arctic
 (Suc (minus_nat n one_nat)),
(mat_max0I id,
  (mat_geI geq_arctic,
    (mat_geI gt_arctic,
      (false, (arc_mat_name, mat_arc_posI pos_arctic)))))))))))))
                      (fun f aa poly ->
                        Right (Right
                                ((f, aa),
                                  (fst poly,
                                    fitLength
                                      (get_comp_0
(wf_mat_rcI (Suc (minus_nat n one_nat)) (Suc (minus_nat n one_nat)),
  (mat0I MinInfty (Suc (minus_nat n one_nat)) (Suc (minus_nat n one_nat)),
    (mat1I MinInfty (Num_arc (Big_int.big_int_of_int 0))
       (Suc (minus_nat n one_nat)),
      (mat_plusI plus_arctic,
        (mat_mulI MinInfty plus_arctic mult_arctic (Suc (minus_nat n one_nat)),
          (mat_max0I id,
            (mat_geI geq_arctic,
              (mat_geI gt_arctic,
                (false, (arc_mat_name, mat_arc_posI pos_arctic)))))))))))
                                      aa (snd poly)))))))
             (fun typea pi ->
               (match typea
                  with Natural -> Int_carrier (map (comp lefta lefta) pi)
                  | Arctic -> Arctic_carrier (map (comp righta lefta) pi)
                  | Int_mat n ->
                    Int_mat_carrier
                      (minus_nat n one_nat, map (comp lefta righta) pi)
                  | Arc_mat n ->
                    Arc_mat_carrier
                      (minus_nat n one_nat, map (comp righta righta) pi))))
           id)];;

let rec xml2ndChoice
  reqName p1 choiceName p2 combi (Xml (name, atts, cs, text)) =
    (if eqop (eq_lista eq_char) name reqName &&
          (null atts &&
            (less_eq_nat (nat (Big_int.big_int_of_int 2)) (size_list cs) &&
              is_none text))
      then bind (p1 (nth cs Zero_nat))
             (fun a ->
               bind (xml2bool choiceName (nth cs one_nat))
                 (fun choice ->
                   (if choice
                     then bind (p2 (nth cs (minus_nat (size_list cs) one_nat)))
                            (fun b -> return (combi a (Some b)))
                     else return (combi a None))))
      else error (concat
                   [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 't'; 'r';
                      'a'; 'n'; 's'; 'f'; 'o'; 'r'; 'm'; ' '; 't'; 'h'; 'e';
                      ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ';
                      'x'; 'm'; 'l'; ' '; 'e'; 'l'; 'e'; 'm'; 'e'; 'n'; 't';
                      ' '; '('; 'e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '];
                     reqName; [')']; newline;
                     sshow_xml (Xml (name, atts, cs, text))]));;

let rec xml2dpProof
  ld xml2name x =
    xmlOptions
      [(['p'; 'I'; 's'; 'E'; 'm'; 'p'; 't'; 'y'],
         xml0elements ['p'; 'I'; 's'; 'E'; 'm'; 'p'; 't'; 'y'] PisEmpty);
        (['d'; 'e'; 'p'; 'G'; 'r'; 'a'; 'p'; 'h'; 'P'; 'r'; 'o'; 'c'],
          xmlManyElements
            ['d'; 'e'; 'p'; 'G'; 'r'; 'a'; 'p'; 'h'; 'P'; 'r'; 'o'; 'c']
            (xml2ndChoice ['c'; 'o'; 'm'; 'p'; 'o'; 'n'; 'e'; 'n'; 't']
              (xml1element ['d'; 'p'; 's'] (xml2rules (xml2name ld)) id)
              ['r'; 'e'; 'a'; 'l'; 'S'; 'c'; 'c'] (xml2dpProof ld xml2name)
              (fun dps prfOpt -> (prfOpt, dps)))
            (fun a -> DepGraphProc a));
        (['r'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'P'; 'r'; 'o'; 'c'],
          xml3elements ['r'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'P'; 'r'; 'o'; 'c']
            (xml2redpair (xml2name ld))
            (xml1element ['d'; 'p'; 's'] (xml2rules (xml2name ld)) id)
            (xml2dpProof ld xml2name) (fun a b c -> RedPairProc (a, b, c)));
        (['r'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'U'; 'r'; 'P'; 'r'; 'o'; 'c'],
          xml4elements
            ['r'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'U'; 'r'; 'P'; 'r'; 'o'; 'c']
            (xml2redpair (xml2name ld))
            (xml1element ['d'; 'p'; 's'] (xml2rules (xml2name ld)) id)
            (xml1element ['u'; 's'; 'a'; 'b'; 'l'; 'e'; 'R'; 'u'; 'l'; 'e'; 's']
              (xml2rules (xml2name ld)) id)
            (xml2dpProof ld xml2name)
            (fun a b c d -> RedPairUrProc (a, b, c, d)));
        (['m'; 'o'; 'n'; 'o'; 'R'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'P'; 'r'; 'o';
           'c'],
          xml4elements
            ['m'; 'o'; 'n'; 'o'; 'R'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'P'; 'r';
              'o'; 'c']
            (xml2redpair (xml2name ld))
            (xml1element ['d'; 'p'; 's'] (xml2rules (xml2name ld)) id)
            (xml1element ['t'; 'r'; 's'] (xml2rules (xml2name ld)) id)
            (xml2dpProof ld xml2name)
            (fun a b c d -> MonoRedPairProc (a, b, c, d)));
        (['m'; 'o'; 'n'; 'o'; 'R'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'U'; 'r'; 'P';
           'r'; 'o'; 'c'],
          xml5elements
            ['m'; 'o'; 'n'; 'o'; 'R'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'U'; 'r';
              'P'; 'r'; 'o'; 'c']
            (xml2redpair (xml2name ld))
            (xml1element ['d'; 'p'; 's'] (xml2rules (xml2name ld)) id)
            (xml1element ['t'; 'r'; 's'] (xml2rules (xml2name ld)) id)
            (xml1element ['u'; 's'; 'a'; 'b'; 'l'; 'e'; 'R'; 'u'; 'l'; 'e'; 's']
              (xml2rules (xml2name ld)) id)
            (xml2dpProof ld xml2name)
            (fun a b c d e -> MonoRedPairUrProc (a, b, c, d, e)));
        (['s'; 'e'; 'm'; 'l'; 'a'; 'b'; 'P'; 'r'; 'o'; 'c'],
          xml4elements ['s'; 'e'; 'm'; 'l'; 'a'; 'b'; 'P'; 'r'; 'o'; 'c']
            (xml2sl_inter (xml2name ld))
            (xml1element ['d'; 'p'; 's'] (xml2rules (xml2name (Suc ld))) id)
            (xml1element ['t'; 'r'; 's'] (xml2rules (xml2name (Suc ld))) id)
            (xml2dpProof (Suc ld) xml2name)
            (fun a ->
              (match a with None -> (fun aa b c -> RootlabProc (aa, b, c))
                 | Some i ->
                   (fun aa b c -> SemlabProc (i, id_labeling, aa, b, c)))));
        (['u'; 'n'; 'l'; 'a'; 'b'; 'P'; 'r'; 'o'; 'c'],
          xml3elements ['u'; 'n'; 'l'; 'a'; 'b'; 'P'; 'r'; 'o'; 'c']
            (xml1element ['d'; 'p'; 's']
              (xml2rules (xml2name (minus_nat ld one_nat))) id)
            (xml1element ['t'; 'r'; 's']
              (xml2rules (xml2name (minus_nat ld one_nat))) id)
            (xml2dpProof (minus_nat ld one_nat) xml2name)
            (fun a b c -> UnlabProc (a, b, c)));
        (['s'; 'u'; 'b'; 't'; 'e'; 'r'; 'm'; 'P'; 'r'; 'o'; 'c'],
          xml3elements ['s'; 'u'; 'b'; 't'; 'e'; 'r'; 'm'; 'P'; 'r'; 'o'; 'c']
            (xml2sp (xml2name ld))
            (xml1element ['d'; 'p'; 's'] (xml2rules (xml2name ld)) id)
            (xml2dpProof ld xml2name) (fun a b c -> SpscProc (a, b, c)));
        (['s'; 'i'; 'z'; 'e'; 'C'; 'h'; 'a'; 'n'; 'g'; 'e'; 'C'; 'r'; 'i'; 't';
           'e'; 'r'; 'i'; 'o'; 'n'],
          xml1manyElements
            ['s'; 'i'; 'z'; 'e'; 'C'; 'h'; 'a'; 'n'; 'g'; 'e'; 'C'; 'r'; 'i';
              't'; 'e'; 'r'; 'i'; 'o'; 'n']
            (xmlOptions
              [(['s'; 'u'; 'b'; 't'; 'e'; 'r'; 'm'; 'C'; 'r'; 'i'; 't'; 'e';
                  'r'; 'i'; 'o'; 'n'],
                 xml0elements
                   ['s'; 'u'; 'b'; 't'; 'e'; 'r'; 'm'; 'C'; 'r'; 'i'; 't'; 'e';
                     'r'; 'i'; 'o'; 'n']
                   None);
                (['r'; 'e'; 'd'; 'u'; 'c'; 't'; 'i'; 'o'; 'n'; 'P'; 'a'; 'i';
                   'r'],
                  xml2elements
                    ['r'; 'e'; 'd'; 'u'; 'c'; 't'; 'i'; 'o'; 'n'; 'P'; 'a'; 'i';
                      'r']
                    (xml2redpair (xml2name ld))
                    (xml1element
                      ['u'; 's'; 'a'; 'b'; 'l'; 'e'; 'R'; 'u'; 'l'; 'e'; 's']
                      (xml2rules (xml2name ld)) id)
                    (fun redp ur -> Some (redp, ur)))])
            (xml2scg (xml2name ld))
            (fun a ->
              (match a with None -> (fun aa -> SizeChangeSubTerm aa)
                 | Some redp_ur ->
                   (fun aa ->
                     SizeChangeRedPair (fst redp_ur, snd redp_ur, aa)))))]
      x;;

let rec foldR
  f x1 b = match f, x1, b with f, [], b -> b
    | f, x :: xs, b -> f x xs (foldR f xs b);;

let rec mul_pvarsI _A
  uu uv a x3 = match uu, uv, a, x3 with uu, uv, a, [] -> []
    | ze, ti, a, (x, b) :: vas ->
        let p = ti a b in
        let res = mul_pvarsI _A ze ti a vas in
        (if eqop _A p ze then res else (x, p) :: res);;

type ('a, 'b) l_poly = LPoly of 'b * ('a * 'b) list;;

let rec mul_lpolyI _A
  ze ti a (LPoly (b, vas)) = LPoly (ti a b, mul_pvarsI _A ze ti a vas);;

let rec list_sumI
  ze pl x2 = match ze, pl, x2 with ze, pl, [] -> ze
    | ze, pl, x :: xs -> pl x (list_sumI ze pl xs);;

let rec add_varI _A _B
  uu uv x a xa4 = match uu, uv, x, a, xa4 with uu, uv, x, a, [] -> [(x, a)]
    | ze, pl, x, a, (y, b) :: vas ->
        (if eqop _B x y
          then let s = pl a b in
               (if eqop _A s ze then vas else (x, s) :: vas)
          else (y, b) :: add_varI _A _B ze pl x a vas);;

let rec sum_pvarsI _A _B
  uu uv x2 vbs = match uu, uv, x2, vbs with uu, uv, [], vbs -> vbs
    | ze, pl, (x, a) :: vas, vbs ->
        (if eqop _A a ze then sum_pvarsI _A _B ze pl vas vbs
          else sum_pvarsI _A _B ze pl vas (add_varI _A _B ze pl x a vbs));;

let rec sum_lpolyI _A _B
  ze pl (LPoly (a, vas)) (LPoly (b, vbs)) =
    LPoly (pl a b, sum_pvarsI _A _B ze pl vas vbs);;

let rec l_poly_case f1 (LPoly (a, lista)) = f1 a lista;;

let rec pleftI _A _C
  ze on pl ti mz pi x6 = match ze, on, pl, ti, mz, pi, x6 with
    ze, on, pl, ti, mz, pi, Var x -> LPoly (ze, [(x, on)])
    | ze, on, pl, ti, mz, pi, Fun (f, ts) ->
        let a = pi f (size_list ts) in
        let (c, asa) = a in
        let LPoly (d, aa) =
          sum_lpolyI _A _C ze pl (LPoly (c, []))
            (list_sumI (LPoly (ze, [])) (sum_lpolyI _A _C ze pl)
              (map (fun at ->
                     mul_lpolyI _A ze ti (fst at)
                       (pleftI _A _C ze on pl ti mz pi (snd at)))
                (zip asa ts)))
          in
        (match aa with [] -> LPoly (mz d, [])
           | ab :: lista -> LPoly (d, ab :: lista));;

let rec check_supteq (_F1, _F2) (_V1, _V2)
  lr = catch (checkB
               (eqop (eq_terma _F1 _V1) (fst lr) (snd lr) ||
                 checkSupterm _F1 _V1 (fst lr) (snd lr))
               [['t'; 'h'; 'e'; ' '; 'r'; 'h'; 's'; ' '; 'i'; 's'; ' '; 'n';
                  'o'; 't'; ' '; 'a'; ' '; 's'; 'u'; 'b'; 't'; 'e'; 'r'; 'm';
                  ' '; 'o'; 'f'; ' '; 't'; 'h'; 'e'; ' '; 'l'; 'h'; 's']])
         (fun x ->
           error (concat
                    [['C'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'o';
                       'r'; 'i'; 'e'; 'n'; 't'; ' '; 'r'; 'u'; 'l'; 'e'; ' '];
                      sshow_term _F2 _V2 (fst lr); [' '; '-'; '>'; ' '];
                      sshow_term _F2 _V2 (snd lr)]
                   :: x));;

let rec xml2trsInput
  xml2name =
    xml1element ['i'; 'n'; 'p'; 'u'; 't']
      (xml1element ['t'; 'r'; 's'; 'I'; 'n'; 'p'; 'u'; 't']
        (xml1element ['t'; 'r'; 's'] (xml2rules xml2name) id) id)
      id;;

type ('a, 'b, 'c) tRSProof =
  DPTrans of
    ((('a, 'b) lab shp, 'c) term * (('a, 'b) lab shp, 'c) term) list *
      (('a, 'b) lab shp, 'b, 'c) dPProof
  | RuleRemoval of
      ('a, 'b) lab redpair *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ('a, 'b, 'c) tRSProof
  | Unlab of
      ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ('a, 'b, 'c) tRSProof
  | Semlab of
      ('a, 'b) sl_inter *
        (('a, 'b) lab -> nat list -> ('b, (('a, 'b) lab list)) either) *
        ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ('a, 'b, 'c) tRSProof
  | Rootlab of
      ((('a, 'b) lab, 'c) term * (('a, 'b) lab, 'c) term) list *
        ('a, 'b, 'c) tRSProof
  | RisEmpty;;

let rec xml2trsProof
  xml2name xml2sname xml2lsname x =
    xmlOptions
      [(['r'; 'I'; 's'; 'E'; 'm'; 'p'; 't'; 'y'],
         xml0elements ['r'; 'I'; 's'; 'E'; 'm'; 'p'; 't'; 'y'] RisEmpty);
        (['d'; 'p'; 'T'; 'r'; 'a'; 'n'; 's'],
          xml3elements ['d'; 'p'; 'T'; 'r'; 'a'; 'n'; 's']
            (xml1element ['d'; 'p'; 's'] (xml2rules xml2sname) id)
            (xml2bool
              ['m'; 'a'; 'r'; 'k'; 'e'; 'd'; 'S'; 'y'; 'm'; 'b'; 'o'; 'l'; 's'])
            (xml2dpProof Zero_nat xml2lsname)
            (fun dps uu -> (fun a -> DPTrans (dps, a))));
        (['r'; 'u'; 'l'; 'e'; 'R'; 'e'; 'm'; 'o'; 'v'; 'a'; 'l'],
          xml3elements ['r'; 'u'; 'l'; 'e'; 'R'; 'e'; 'm'; 'o'; 'v'; 'a'; 'l']
            (xml2redpair xml2name)
            (xml1element ['t'; 'r'; 's'] (xml2rules xml2name) id)
            (xml2trsProof xml2name xml2sname xml2lsname)
            (fun a b c -> RuleRemoval (a, b, c)));
        (['u'; 'n'; 'l'; 'a'; 'b'],
          xml2elements ['u'; 'n'; 'l'; 'a'; 'b']
            (xml1element ['t'; 'r'; 's'] (xml2rules xml2name) id)
            (xml2trsProof xml2name xml2sname xml2lsname)
            (fun a b -> Unlab (a, b)));
        (['s'; 'e'; 'm'; 'l'; 'a'; 'b'],
          xml3elements ['s'; 'e'; 'm'; 'l'; 'a'; 'b'] (xml2sl_inter xml2name)
            (xml1element ['t'; 'r'; 's'] (xml2rules xml2name) id)
            (xml2trsProof xml2name xml2sname xml2lsname)
            (fun a ->
              (match a with None -> (fun aa b -> Rootlab (aa, b))
                 | Some i -> (fun aa b -> Semlab (i, id_labeling, aa, b)))))]
      x;;

let rec checkF f g x = (if f x then return () else error (g x));;

let rec checkAllMI
  f xs =
    foldr (fun x iq ->
            bind (catch (f x) (fun xa -> error (x, xa))) (fun xa -> iq))
      xs (return ());;

let rec check_edges
  ss g c d =
    catch (checkAllMI
            (fun n ->
              catch (checkAllMI
                      (fun m ->
                        checkB (not (g (m, n)))
                          (concat
                            [['e'; 'd'; 'g'; 'e'; ' '; 'f'; 'r'; 'o'; 'm'; ' '];
                              ss m; [' '; 't'; 'o'; ' ']; ss n]))
                      d)
                (fun x -> error (snd x)))
            c)
      (fun x -> error (snd x));;

let rec prightI _A _C
  ze on pl ti mz pi x6 = match ze, on, pl, ti, mz, pi, x6 with
    ze, on, pl, ti, mz, pi, Var x -> LPoly (ze, [(x, on)])
    | ze, on, pl, ti, mz, pi, Fun (f, ts) ->
        let a = pi f (size_list ts) in
        let (c, asa) = a in
        let LPoly (d, aa) =
          sum_lpolyI _A _C ze pl (LPoly (c, []))
            (list_sumI (LPoly (ze, [])) (sum_lpolyI _A _C ze pl)
              (map (fun at ->
                     mul_lpolyI _A ze ti (fst at)
                       (prightI _A _C ze on pl ti mz pi (snd at)))
                (zip asa ts)))
          in
        LPoly (mz d, aa);;

type ('a, 'b) tRSDisproof =
  Loop of
    (('a, 'b) term list *
      (('a, 'b) ctxt * (('a, 'b) term * ('a, 'b) term) list))
  | NotWellFormed;;

let rec funs_term_list'
  = function Var uu -> []
    | Fun (f, ts) -> f :: flat_map funs_term_list' ts;;

let rec funs_rule_list'
  r = append (funs_term_list' (fst r)) (funs_term_list' (snd r));;

let rec funs_trs_list _F trs = remdups _F (flat_map funs_rule_list' trs);;

let rec xml2plainName
  x = xmlOptions
        [(['n'; 'a'; 'm'; 'e'],
           xmlChange (xml2text ['n'; 'a'; 'm'; 'e']) (fun a -> UnLab a));
          (['l'; 'a'; 'b'; 'e'; 'l'; 'e'; 'd'; 'S'; 'y'; 'm'; 'b'; 'o'; 'l'],
            xml2elements
              ['l'; 'a'; 'b'; 'e'; 'l'; 'e'; 'd'; 'S'; 'y'; 'm'; 'b'; 'o'; 'l']
              xml2plainName (xml2label xml2plainName)
              (fun f a ->
                (match a with Left aa -> Lab (f, aa)
                   | Right aa -> FunLab (f, aa))))]
        x;;

let rec xml2sharpName
  x = (if eqop (eq_lista eq_char) (xmlGetName x) ['s'; 'h'; 'a'; 'r'; 'p']
        then xml1element ['s'; 'h'; 'a'; 'r'; 'p'] xml2plainName
               (fun a -> Sharp a) x
        else xmlChange xml2plainName (fun a -> Plain a) x);;

let rec length_unique _A
  = function [] -> Zero_nat
    | x :: xs ->
        (if member _A x xs then length_unique _A xs
          else Suc (length_unique _A xs));;

type certificationResult = Certified of bool | Error of char list;;

let rec vars_term_list'
  = function Var x -> [x]
    | Fun (uu, ts) -> flat_map vars_term_list' ts;;

let rec vars_term_list _B t = remdups _B (vars_term_list' t);;

let rec enum_vectors
  c x1 = match c, x1 with c, [] -> [[]]
    | c, x :: xs ->
        let a = enum_vectors c xs in
        flat_map (fun vec -> map (fun ca -> (x, ca) :: vec) c) a;;

let rec assign_from_vector _A vec x = the (lookup _A x vec);;

let rec eq_lab _A _B
  x0 x1 = match x0, x1 with
    FunLab (fa, ls), FunLab (g, ks) ->
      eq_lab _A _B fa g && list_all2 (eq_lab _A _B) ls ks
    | UnLab c, FunLab (a, ba) -> false
    | UnLab c, Lab (a, b) -> false
    | FunLab (ca, d), Lab (a, b) -> false
    | FunLab (a, ba), UnLab c -> false
    | Lab (a, b), UnLab c -> false
    | Lab (a, b), FunLab (ca, d) -> false
    | UnLab f, UnLab f' -> eq _A f f'
    | Lab (lab, l), Lab (lab', l') -> eq_lab _A _B lab lab' && eq _B l l';;

let eq_laba _A _B = ({eq = eq_lab _A _B} : ('a, 'b) lab eq);;

let rec sshow_lab _C _D
  = function UnLab f -> sshow _C f
    | Lab (fa, l) ->
        append (append (append (sshow_lab _C _D fa) ['[']) (sshow _D l)) [']']
    | FunLab (fa, la) ->
        append
          (append (append (sshow_lab _C _D fa) ['['])
            (flat_map (sshow_lab _C _D) la))
          [']'];;

let sshow_laba _A _B = ({sshow = sshow_lab _A _B} : ('a, 'b) lab sshow);;

let rec label
  f x1 = match f, x1 with f, Left l -> Lab (f, l)
    | f, Right la -> FunLab (f, la);;

let rec eval_lab
  i l alpha x3 = match i, l, alpha, x3 with
    i, l, alpha, Var x -> (alpha x, Var x)
    | i, l, alpha, Fun (f, ts) ->
        let clts = map (eval_lab i l alpha) ts in
        let cs = map fst clts in
        let c = i f cs in
        let lts = map snd clts in
        let la = l f cs in
        (c, Fun (label f la, lts));;

let rec check_sl_rule_ass (_A1, _A2) (_B1, _B2) (_C1, _C2) (_D1, _D2)
  mc i la lR alpha (l, r) =
    let cl_ll = eval_lab i la alpha l in
    let cr_lr = eval_lab i la alpha r in
    bind (checkB (implies mc (eqop _C1 (fst cl_ll) (fst cr_lr)))
           (append
             (append
               (append
                 (append
                   (append ['r'; 'u'; 'l'; 'e'; ' ']
                     (showRule (sshow_laba _A2 _B2) _D2 (l, r)))
                   [' '; 'v'; 'i'; 'o'; 'l'; 'a'; 't'; 'e'; 's'; ' '; 'm'; 'o';
                     'd'; 'e'; 'l'; ' '; 'c'; 'o'; 'n'; 'd'; 'i'; 't'; 'i'; 'o';
                     'n'; ','; ' '; '['; 'l'; 'h'; 's'; ']'; ' '; '='; ' '])
                 (sshow _C2 (fst cl_ll)))
               [','; ' '; '['; 'r'; 'h'; 's'; ']'; ' '; '='; ' '])
             (sshow _C2 (fst cr_lr))))
      (fun x ->
        checkB
          (member
            (eqb (eq_terma (eq_laba _A1 _B1) _D1)
              (eq_terma (eq_laba _A1 _B1) _D1))
            (snd cl_ll, snd cr_lr) lR)
          (append
            (append
              ['l'; 'a'; 'b'; 'e'; 'l'; 'e'; 'd'; ' '; 'r'; 'u'; 'l'; 'e'; ' ']
              (showRule (sshow_laba _A2 _B2) _D2 (snd cl_ll, snd cr_lr)))
            [' '; 'm'; 'i'; 's'; 's'; 'i'; 'n'; 'g']));;

let rec check_sl_rule (_A1, _A2) (_B1, _B2) (_C1, _C2) (_D1, _D2)
  i la c mc lR (l, r) =
    catch (checkAllMI
            (fun alpha ->
              check_sl_rule_ass (_A1, _A2) (_B1, _B2) (_C1, _C2) (_D1, _D2) mc i
                la lR alpha (l, r))
            (map (assign_from_vector _D1)
              (enum_vectors c
                (list_union _D1 (vars_term_list _D1 l)
                  (vars_term_list _D1 r)))))
      (fun x -> error (snd x));;

let rec check_sl_model_lab_trs (_F1, _F2) (_L1, _L2) (_C1, _C2) (_V1, _V2)
  i l c lR r =
    catch (checkAllMI
            (check_sl_rule (_F1, _F2) (_L1, _L2) (_C1, _C2) (_V1, _V2) i l c
              true lR)
            r)
      (fun x -> error (snd x));;

let rec rl_fun uu cs = Right cs;;

let rec rl_inter_list _A fs d g uu = (if member _A g fs then g else d);;

let rec check_sl_lab_trs (_F1, _F2) (_L1, _L2) (_C1, _C2) (_V1, _V2)
  i l c lP p =
    catch (checkAllMI
            (check_sl_rule (_F1, _F2) (_L1, _L2) (_C1, _C2) (_V1, _V2) i l c
              false lP)
            p)
      (fun x -> error (snd x));;

let rec get_lab = function Lab (f, l) -> Left l
                  | FunLab (f, la) -> Right la;;

let rec unlab
  = function Lab (f, l) -> f
    | FunLab (f, la) -> f
    | UnLab v -> UnLab v;;

let rec map_term
  fg x1 = match fg, x1 with fg, Var x -> Var x
    | fg, Fun (f, ts) -> Fun (fg f, map (map_term fg) ts);;

let rec check_sl_only_lab_and_decr (_B1, _B2) (_A1, _A2) (_C1, _C2)
  gr r =
    (fun xs ->
      catch (checkAllMI
              (fun ((ll, lr) as a) ->
                let l = map_term unlab ll in
                let ra = map_term unlab lr in
                checkB
                  (if eqop (eq_terma (eq_laba _A1 _B1) _C1) l ra
                    then not (is_var ll) &&
                           (not (is_var lr) &&
                             gr (get_lab (the (root ll)),
                                  get_lab (the (root lr))))
                           || eqop (eq_terma (eq_laba _A1 _B1) _C1) ll lr &&
                                member
                                  (eqb (eq_terma (eq_laba _A1 _B1) _C1)
                                    (eq_terma (eq_laba _A1 _B1) _C1))
                                  (l, ra) r
                    else member
                           (eqb (eq_terma (eq_laba _A1 _B1) _C1)
                             (eq_terma (eq_laba _A1 _B1) _C1))
                           (l, ra) r)
                  (append
                    (append
                      ['l'; 'a'; 'b'; 'e'; 'l'; 'e'; 'd'; ' '; 'r'; 'u'; 'l';
                        'e'; ' ']
                      (showRule (sshow_laba _A2 _B2) _C2 (l, ra)))
                    [' '; 'i'; 's'; ' '; 'n'; 'e'; 'i'; 't'; 'h'; 'e'; 'r'; ' ';
                      'd'; 'e'; 'c'; 'r'; 'e'; 'a'; 's'; 'i'; 'n'; 'g'; ' ';
                      'n'; 'o'; 'r'; ' '; 'a'; ' '; 'l'; 'a'; 'b'; 'e'; 'l';
                      'e'; 'd'; ' '; 'v'; 'e'; 'r'; 's'; 'i'; 'o'; 'n'; ' ';
                      'o'; 'f'; ' '; 'o'; 'r'; 'i'; 'g'; 'i'; 'n'; 'a'; 'l';
                      ' '; 'r'; 'u'; 'l'; 'e']))
              xs)
        (fun x -> error (snd x)));;

let rec check_sem_lab_tt (_F1, _F2) (_L1, _L2) (_V1, _V2)
  check_lab check_model_lab gr i l r lR =
    catch (bind (check_model_lab lR r)
            (fun x ->
              check_sl_only_lab_and_decr (_L1, _L2) (_F1, _F2) (_V1, _V2) gr r
                lR))
      (fun x ->
        error [['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'i'; 't'; 'h'; ' ';
                 'l'; 'a'; 'b'; 'e'; 'l'; 'e'; 'd'; ' '; 'T'; 'R'; 'S'; ':'];
                x]);;

let rec rl_tt (_F1, _F2) (_L1, _L2)
  r lR =
    let fs = funs_trs_list (eq_laba _F1 _L1) r in
    bind (checkB (not (null fs))
           [['r'; 'o'; 'o'; 't'; '-'; 'l'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n'; 'g';
              ' '; 'r'; 'e'; 'q'; 'u'; 'i'; 'r'; 'e'; 's'; ' '; 'a'; 't'; ' ';
              'l'; 'e'; 'a'; 's'; 't'; ' '; 'o'; 'n'; 'e'; ' '; 'f'; 'u'; 'n';
              'c'; 't'; 'i'; 'o'; 'n'; ' '; 's'; 'y'; 'm'; 'b'; 'o'; 'l'; ' ';
              'i'; 'n'; ' '; 't'; 'h'; 'e'; ' '; 's'; 'i'; 'g'; 'n'; 'a'; 't';
              'u'; 'r'; 'e']])
      (fun x ->
        let d = hd fs in
        check_sem_lab_tt (_F1, _F2) (_L1, _L2)
          (eq_lista eq_char, sshow_lista sshow_chara)
          (check_sl_lab_trs (_F1, _F2) (_L1, _L2)
            (eq_laba _F1 _L1, sshow_laba _F2 _L2)
            (eq_lista eq_char, sshow_lista sshow_chara)
            (rl_inter_list (eq_laba _F1 _L1) fs d) rl_fun fs)
          (check_sl_model_lab_trs (_F1, _F2) (_L1, _L2)
            (eq_laba _F1 _L1, sshow_laba _F2 _L2)
            (eq_lista eq_char, sshow_lista sshow_chara)
            (rl_inter_list (eq_laba _F1 _L1) fs d) rl_fun fs)
          empty (rl_inter_list (eq_laba _F1 _L1) fs d) rl_fun r lR);;

let rec indent xs = newline :: map (append [' '; ' ']) xs;;

let terminationGoal : xml =
  Xml (['g'; 'o'; 'a'; 'l'], [],
        [Xml (['t'; 'e'; 'r'; 'm'; 'i'; 'n'; 'a'; 't'; 'i'; 'o'; 'n'], [], [],
               None)],
        None)

let rec xml2substitution
  xml2name =
    xmlManyElements ['s'; 'u'; 'b'; 's'; 't'; 'i'; 't'; 'u'; 't'; 'i'; 'o'; 'n']
      (xml2elements ['s'; 'u'; 'b'; 's'; 't'; 'E'; 'n'; 't'; 'r'; 'y'] xml2var
        (xml2term xml2name) (fun a b -> (a, b)))
      id;;

let rec xmlMany2Elements
  reqName p1 p2 p3 combi (Xml (name, atts, cs, text)) =
    let ds = rev cs in
    (if eqop (eq_lista eq_char) name reqName &&
          (null atts &&
            (is_none text &&
              less_eq_nat (nat (Big_int.big_int_of_int 2)) (size_list cs)))
      then bind (mapMonad p1 (rev (tl (tl ds))))
             (fun a ->
               bind (p2 (nth ds one_nat))
                 (fun b ->
                   bind (p3 (nth ds Zero_nat)) (comp return (combi a b))))
      else error (concat
                   [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 't'; 'r';
                      'a'; 'n'; 's'; 'f'; 'o'; 'r'; 'm'; ' '; 't'; 'h'; 'e';
                      ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ';
                      'x'; 'm'; 'l'; ' '; 'e'; 'l'; 'e'; 'm'; 'e'; 'n'; 't';
                      ' '; '('; 'e'; 'x'; 'p'; 'e'; 'c'; 't'; 'e'; 'd'; ' '];
                     reqName; [')']; newline;
                     sshow_xml (Xml (name, atts, cs, text))]));;

let rec xml2trsDisproof
  xml2name =
    xmlOptions
      [(['v'; 'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e'; 'C'; 'o'; 'n'; 'd'; 'i'; 't';
          'i'; 'o'; 'n'],
         xml0elements
           ['v'; 'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e'; 'C'; 'o'; 'n'; 'd'; 'i';
             't'; 'i'; 'o'; 'n']
           NotWellFormed);
        (['l'; 'o'; 'o'; 'p'],
          xmlMany2Elements ['l'; 'o'; 'o'; 'p']
            (xml3elements ['r'; 'e'; 'd'; 'e'; 'x'] (xml2term xml2name) xml2pos
              (xml2rule xml2name) (fun t p r -> t))
            (xml2substitution xml2name) (xml2context xml2name)
            (fun ts sigma c -> Loop (ts, (c, sigma))))];;

let nonTerminationGoal : xml =
  Xml (['g'; 'o'; 'a'; 'l'], [],
        [Xml (['n'; 'o'; 'n'; 'T'; 'e'; 'r'; 'm'; 'i'; 'n'; 'a'; 't'; 'i'; 'o';
                'n'],
               [], [], None)],
        None)

type ('a, 'b, 'c) certificationProblem =
  CertificationProblem of
    bool * (('a, 'c) term * ('a, 'c) term) list *
      (('a, 'b, 'c) tRSProof, ('a, 'c) tRSDisproof) either;;

let rec xml2certProblem
  xml2uname xml2name xml2sname xml2lsname (Xml (name, uu, cs, text)) =
    (if eqop (eq_lista eq_char) name
          ['c'; 'e'; 'r'; 't'; 'i'; 'f'; 'i'; 'c'; 'a'; 't'; 'i'; 'o'; 'n'; 'P';
            'r'; 'o'; 'b'; 'l'; 'e'; 'm']
          && (is_none text &&
               less_eq_nat (nat (Big_int.big_int_of_int 3)) (size_list cs))
      then bind (xml2trsInput xml2uname (nth cs one_nat))
             (fun trs ->
               (if eqop eq_xmla (nth cs Zero_nat) terminationGoal
                 then bind (xml1element ['p'; 'r'; 'o'; 'o'; 'f']
                             (xml2trsProof xml2name xml2sname xml2lsname) id
                             (nth cs (nat (Big_int.big_int_of_int 2))))
                        (fun p ->
                          return (CertificationProblem (true, trs, Left p)))
                 else (if eqop eq_xmla (nth cs Zero_nat) nonTerminationGoal
                        then bind (xml1element ['p'; 'r'; 'o'; 'o'; 'f']
                                    (xml2trsDisproof xml2uname) id
                                    (nth cs (nat (Big_int.big_int_of_int 2))))
                               (fun p ->
                                 return
                                   (CertificationProblem (false, trs, Right p)))
                        else error ['u'; 'n'; 'k'; 'n'; 'o'; 'w'; 'n'; ' '; 'g';
                                     'o'; 'a'; 'l'; ' '; 't'; 'y'; 'p'; 'e';
                                     ' '; 'i'; 'n'; ' '; 'c'; 'e'; 'r'; 't';
                                     'i'; 'f'; 'i'; 'c'; 'a'; 't'; 'i'; 'o';
                                     'n'; ' '; 'p'; 'r'; 'o'; 'b'; 'l'; 'e';
                                     'm'])))
      else error ['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'p'; 'a';
                   'r'; 's'; 'e'; ' '; 'r'; 'o'; 'o'; 't'; ' '; 'e'; 'l'; 'e';
                   'm'; 'e'; 'n'; 't']);;

let rec computeDPsOfTerm _A _B
  d l lshp x3 = match d, l, lshp, x3 with d, l, lshp, Var uu -> []
    | d, l, lshp, Fun (f, ts) ->
        append
          (if member _A f d && not (checkSupterm _A _B l (Fun (f, ts)))
            then [(lshp, sharpa (Fun (f, ts)))] else [])
          (foldr
            (fun t ->
              list_union
                (eqb (eq_terma (eq_shpa _A) _B) (eq_terma (eq_shpa _A) _B))
                (computeDPsOfTerm _A _B d l lshp t))
            ts []);;

let rec computeDPsOfRule _A _B
  d x1 = match d, x1 with
    d, (Fun (f, ss), r) ->
      computeDPsOfTerm _A _B d (Fun (f, ss)) (sharpa (Fun (f, ss))) r
    | d, (Var x, r) -> [];;

let rec computeDPs _A _B
  r = let d = defs_list _A r in
      foldr (fun lr ->
              list_union
                (eqb (eq_terma (eq_shpa _A) _B) (eq_terma (eq_shpa _A) _B))
                (computeDPsOfRule _A _B d lr))
        r [];;

let rec checkSubset _A
  xs ys =
    catch (checkAllMI (checkF (fun x -> member _A x ys) id) xs)
      (fun x -> error (snd x));;

let rec checkWfTRS _F (_V1, _V2)
  r = catch (bind (catch
                    (checkAllMI
                      (fun lr ->
                        checkB (not (is_var (fst lr)))
                          (concat
                            [['t'; 'h'; 'e'; ' '; 'l'; 'e'; 'f'; 't'; ' '; 's';
                               'i'; 'd'; 'e'; ' '; 'o'; 'f'; ' '; 'r'; 'u'; 'l';
                               'e'; ' '];
                              showRule _F _V2 lr;
                              [' '; 'i'; 's'; ' '; 'a'; ' '; 'v'; 'a'; 'r'; 'i';
                                'a'; 'b'; 'l'; 'e']]))
                      r)
                    (fun x -> error (snd x)))
              (fun x ->
                catch (checkAllMI
                        (fun lr ->
                          catch (checkSubset _V1 (vars_term_list _V1 (snd lr))
                                  (vars_term_list _V1 (fst lr)))
                            (fun xa ->
                              error (concat
                                      [['t'; 'h'; 'e'; ' '; 'v'; 'a'; 'r'; 'i';
 'a'; 'b'; 'l'; 'e'; ' '];
sshow _V2 xa;
[' '; 'o'; 'c'; 'c'; 'u'; 'r'; 's'; ' '; 'o'; 'n'; 'l'; 'y'; ' '; 'i'; 'n'; ' ';
  't'; 'h'; 'e'; ' '; 'r'; 'i'; 'g'; 'h'; 't'; '-'; 'h'; 'a'; 'n'; 'd'; ' ';
  's'; 'i'; 'd'; 'e'; ' '; 'o'; 'f'; ' '; 'r'; 'u'; 'l'; 'e'; ' '];
showRule _F _V2 lr])))
                        r)
                  (fun xa -> error (snd xa))))
        (fun x ->
          error [['t'; 'h'; 'e'; ' '; 'T'; 'R'; 'S'; ' '; 'i'; 's'; ' '; 'n';
                   'o'; 't'; ' '; 'w'; 'e'; 'l'; 'l'; '-'; 'f'; 'o'; 'r'; 'm';
                   'e'; 'd'];
                  x]);;

let rec checkDPs (_F1, _F2) (_V1, _V2)
  r p = catch (bind (checkWfTRS _F2 (_V1, _V2) r)
                (fun x ->
                  catch (checkSubset
                          (eqb (eq_terma (eq_shpa _F1) _V1)
                            (eq_terma (eq_shpa _F1) _V1))
                          (computeDPs _F1 _V1 r) p)
                    (fun xa ->
                      error [concat
                               [['t'; 'h'; 'e'; ' '; 'D'; 'P'; ' '];
                                 showRule (sshow_shpa _F2) _V2 xa;
                                 [' '; 'd'; 'o'; 'e'; 's'; ' '; 'n'; 'o'; 't';
                                   ' '; 'a'; 'p'; 'p'; 'e'; 'a'; 'r'; ' '; 'i';
                                   'n'; ' '; 't'; 'h'; 'e'; ' '; 'D'; 'P'; ' ';
                                   'p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm']]])))
          (fun x ->
            error (['t'; 'h'; 'e'; ' '; 'D'; 'P'; '-'; 't'; 'r'; 'a'; 'n'; 's';
                     'f'; 'o'; 'r'; 'm'; 'a'; 't'; 'i'; 'o'; 'n'; ' '; 'i'; 's';
                     ' '; 'n'; 'o'; 't'; ' '; 'a'; 'p'; 'p'; 'l'; 'i'; 'e'; 'd';
                     ' '; 'c'; 'o'; 'r'; 'r'; 'e'; 'c'; 't'; 'l'; 'y'; '.']
                    :: x));;

let rec create_afI _A _B
  ze x1 f n i = match ze, x1, f, n, i with ze, [], f, n, i -> true
    | ze, ((g, m), (c, coeffs)) :: rest, f, n, i ->
        (if eqop _B f g && eqop eq_nata m n
          then less_nat i (size_list coeffs) && not (eqop _A (nth coeffs i) ze)
          else create_afI _A _B ze rest f n i);;

let nat_name : char list = ['n'; 'a'; 't'; 'u'; 'r'; 'a'; 'l'; 's']

let rec check_sem_lab_proc (_F1, _F2) (_L1, _L2) (_V1, _V2)
  check_lab check_model_lab gr i l p r lP lR =
    bind (bind (catch (check_lab lP p)
                 (fun x ->
                   error [['n'; 'o'; 't'; ' '; 'a'; 'l'; 'l'; ' '; 'l'; 'a';
                            'b'; 'e'; 'l'; 'e'; 'd'; ' '; 'D'; 'P'; 's'; ' ';
                            'p'; 'r'; 'e'; 's'; 'e'; 'n'; 't'; ':'];
                           x]))
           (fun x ->
             catch (check_model_lab lR r)
               (fun xa ->
                 error [['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'i'; 't';
                          'h'; ' '; 'l'; 'a'; 'b'; 'e'; 'l'; 'e'; 'd'; ' '; 'T';
                          'R'; 'S'; ':'];
                         xa])))
      (fun x ->
        catch (check_sl_only_lab_and_decr (_L1, _L2) (_F1, _F2) (_V1, _V2) gr r
                lR)
          (fun xa -> error [xa]));;

let rec list_diff _A
  xs x1 = match xs, x1 with xs, [] -> xs
    | xs, y :: ys -> list_diff _A (removeAll _A y xs) ys;;

let rec rl_proc (_F1, _F2) (_L1, _L2)
  p r lP lR =
    let fs =
      list_union (eq_laba _F1 _L1) (funs_trs_list (eq_laba _F1 _L1) r)
        (list_diff (eq_laba _F1 _L1) (funs_trs_list (eq_laba _F1 _L1) p)
          (defs_list (eq_laba _F1 _L1) p))
      in
    bind (checkB (not (null fs))
           [['r'; 'o'; 'o'; 't'; '-'; 'l'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n'; 'g';
              ' '; 'r'; 'e'; 'q'; 'u'; 'i'; 'r'; 'e'; 's'; ' '; 'a'; 't'; ' ';
              'l'; 'e'; 'a'; 's'; 't'; ' '; 'o'; 'n'; 'e'; ' '; 'f'; 'u'; 'n';
              'c'; 't'; 'i'; 'o'; 'n'; ' '; 's'; 'y'; 'm'; 'b'; 'o'; 'l'; ' ';
              'i'; 'n'; ' '; 't'; 'h'; 'e'; ' '; 's'; 'i'; 'g'; 'n'; 'a'; 't';
              'u'; 'r'; 'e']])
      (fun x ->
        let d = hd fs in
        check_sem_lab_proc (_F1, _F2) (_L1, _L2)
          (eq_lista eq_char, sshow_lista sshow_chara)
          (check_sl_lab_trs (_F1, _F2) (_L1, _L2)
            (eq_laba _F1 _L1, sshow_laba _F2 _L2)
            (eq_lista eq_char, sshow_lista sshow_chara)
            (rl_inter_list (eq_laba _F1 _L1) fs d) rl_fun fs)
          (check_sl_model_lab_trs (_F1, _F2) (_L1, _L2)
            (eq_laba _F1 _L1, sshow_laba _F2 _L2)
            (eq_lista eq_char, sshow_lista sshow_chara)
            (rl_inter_list (eq_laba _F1 _L1) fs d) rl_fun fs)
          empty (rl_inter_list (eq_laba _F1 _L1) fs d) rl_fun p r lP lR);;

let rec xml2labSharpName
  = function Zero_nat -> xmlChange xml2sharpName (fun a -> UnLab a)
    | Suc n ->
        xml2elements
          ['l'; 'a'; 'b'; 'e'; 'l'; 'e'; 'd'; 'S'; 'y'; 'm'; 'b'; 'o'; 'l']
          (xml2labSharpName n) (xml2label (xml2labSharpName n))
          (fun f a ->
            (match a with Left aa -> Lab (f, aa)
               | Right aa -> FunLab (f, aa)));;

let rec xmldoc2certProblem
  (Xmldoc (header, xml)) =
    debug ['0']
      ['p'; 'a'; 'r'; 's'; 'i'; 'n'; 'g'; ' '; 'x'; 'm'; 'l'; ' '; 't'; 'o';
        ' '; 'c'; 'e'; 'r'; 't'; 'i'; 'f'; 'i'; 'c'; 'a'; 't'; 'i'; 'o'; 'n';
        ' '; 'p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm']
      (xml2certProblem (xml2text ['n'; 'a'; 'm'; 'e']) xml2plainName
        xml2sharpName xml2labSharpName xml);;

let rec parseCertProblem s = bind (parseXMLDoc s) xmldoc2certProblem;;

let rec computeID r = map (fun lr -> (plaina (fst lr), plaina (snd lr))) r;;

let rec get_comp_ge
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = ge;;

let rec get_comp_gt
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = gt;;

let rec show_pvars (_A1, _A2) _B
  on x1 = match on, x1 with on, [] -> []
    | on, (x, c) :: vas ->
        append (if eqop _A1 on c then [] else sshow _A2 c) (sshow _B x) ::
          show_pvars (_A1, _A2) _B on vas;;

let rec show_lpoly (_A1, _A2) _B
  ze on (LPoly (c, cs)) =
    (match show_pvars (_A1, _A2) _B on cs with [] -> sshow _A2 c
       | s :: ss ->
         append
           (append
             (if eqop _A1 c ze then []
               else append (sshow _A2 c) [' '; '+'; ' '])
             s)
           (flat_map (append [' '; '+'; ' ']) ss));;

let rec lookup_rest _A
  x xa1 = match x, xa1 with x, [] -> None
    | x, (y, c) :: ycs ->
        (if eqop _A x y then Some (c, ycs)
          else (match lookup_rest _A x ycs with None -> None
                  | Some a -> let (d, yccs) = a in
                              Some (d, (y, c) :: yccs)));;

let rec search_len
  t s n =
    (if eqop (eq_lista eq_char) (take n t) s then Some ([], drop n t)
      else (match t with [] -> None
              | h :: tt ->
                (match search_len tt s n with None -> None
                   | Some a -> let (bef, aft) = a in
                               Some (h :: bef, aft))));;

let rec reverse_trs r = map (fun lr -> (snd lr, fst lr)) r;;

let rec check_pvarsI _B
  ze ge vas x3 = match ze, ge, vas, x3 with
    ze, ge, vas, [] ->
      catch (checkAllMI (fun va -> checkB (ge (snd va) ze) (fst va)) vas)
        (fun x -> error (snd x))
    | ze, ge, vas, (x, b) :: vbs ->
        let a =
          (match lookup_rest _B x vas with None -> (ze, vas)
             | Some a -> let (aa, ba) = a in
                         (aa, ba))
          in
        let (a', vas') = a in
        bind (checkB (ge a' b) x) (fun xa -> check_pvarsI _B ze ge vas' vbs);;

let rec check_lpoly_sI (_A1, _A2) (_B1, _B2)
  mode ze on ge gt (LPoly (a, vas)) (LPoly (b, vbs)) =
    catch (bind (checkB (gt a b)
                  ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'h'; 'e'; 'n';
                    ' '; 'c'; 'o'; 'm'; 'p'; 'a'; 'r'; 'i'; 'n'; 'g'; ' '; 'c';
                    'o'; 'n'; 's'; 't'; 'a'; 'n'; 't'; ' '; 'p'; 'a'; 'r'; 't'])
            (fun x ->
              catch (check_pvarsI _B1 ze (if mode then ge else gt) vas vbs)
                (fun xa ->
                  error (append
                          ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'h';
                            'e'; 'n'; ' '; 'c'; 'o'; 'm'; 'p'; 'a'; 'r'; 'i';
                            'n'; 'g'; ' '; 'c'; 'o'; 'e'; 'f'; 'f'; 'i'; 'c';
                            'i'; 'e'; 'n'; 't'; 's'; ' '; 'o'; 'f'; ' '; 'v';
                            'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e'; ' ']
                          (sshow _B2 xa)))))
      (fun x ->
        error [append
                 (append
                   (append
                     ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'h'; 'e';
                       'n'; ' '; 'c'; 'o'; 'm'; 'p'; 'a'; 'r'; 'i'; 'n'; 'g';
                       ' ']
                     (show_lpoly (_A1, _A2) _B2 ze on (LPoly (a, vas))))
                   [' '; '>'; ' '])
                 (show_lpoly (_A1, _A2) _B2 ze on (LPoly (b, vbs)));
                x]);;

let rec check_polo_sI (_A1, _A2) _B (_C1, _C2)
  mode ze on pl ti mz ge gt pi (s, t) =
    let left = pleftI _A1 _C1 ze on pl ti mz pi s in
    let right = prightI _A1 _C1 ze on pl ti mz pi t in
    catch (check_lpoly_sI (_A1, _A2) (_C1, _C2) mode ze on ge gt left right)
      (fun x ->
        error (append
                 (append
                   (append
                     ['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'e';
                       'n'; 's'; 'u'; 'r'; 'e'; ' ']
                     (sshow_term _B _C2 s))
                   [' '; '>'; ' '])
                 (sshow_term _B _C2 t)
                :: x));;

let rec get_comp_apos
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = ap;;

let rec get_comp_max0
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = mz;;

let rec get_comp_mono
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = mo;;

let rec sshow_arctic
  = function Num_arc x -> sshow_int x
    | MinInfty -> ['-'; 'i'; 'n'; 'f'];;

let sshow_arctica = ({sshow = sshow_arctic} : arctic sshow);;

let rec fresh_strings_list
  name offset used n =
    take n
      (filter
        (fun s ->
          not (member (eq_lista eq_char) s (remdups (eq_lista eq_char) used)))
        (map (fun i -> append name (sshow_nat (plus_nat i offset)))
          (upt Zero_nat
            (plus_nat n (length_unique (eq_lista eq_char) used)))));;

let rec checkAllCombinations
  c xs =
    foldR (fun x ys res ->
            catch (checkAllMI
                    (fun y ->
                      bind (catch (c x y) (fun xa -> error ((x, y), xa)))
                        (fun xa -> res))
                    ys)
              (fun xa -> error (snd xa)))
      xs (return ());;

let rec check_graph_decomp
  ss g rcs =
    bind (catch (checkAllCombinations (check_edges ss g) (map snd rcs))
           (fun x -> error (snd x)))
      (fun x ->
        catch (checkAllMI (fun c -> check_edges ss g c c)
                (map snd (filter (fun rc -> not (fst rc)) rcs)))
          (fun xa -> error (snd xa)));;

let rec check_lpoly_nsI (_A1, _A2) (_B1, _B2)
  ze on ge (LPoly (a, vas)) (LPoly (b, vbs)) =
    catch (bind (checkB (ge a b)
                  ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'h'; 'e'; 'n';
                    ' '; 'c'; 'o'; 'm'; 'p'; 'a'; 'r'; 'i'; 'n'; 'g'; ' '; 'c';
                    'o'; 'n'; 's'; 't'; 'a'; 'n'; 't'; ' '; 'p'; 'a'; 'r'; 't';
                    's'])
            (fun x ->
              catch (check_pvarsI _B1 ze ge vas vbs)
                (fun xa ->
                  error (append
                          ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'h';
                            'e'; 'n'; ' '; 'c'; 'o'; 'm'; 'p'; 'a'; 'r'; 'i';
                            'n'; 'g'; ' '; 'c'; 'o'; 'e'; 'f'; 'f'; 'i'; 'c';
                            'i'; 'e'; 'n'; 't'; 's'; ' '; 'o'; 'f'; ' '; 'v';
                            'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e'; ' ']
                          (sshow _B2 xa)))))
      (fun x ->
        error [append
                 (append
                   (append
                     ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'h'; 'e';
                       'n'; ' '; 'c'; 'o'; 'm'; 'p'; 'a'; 'r'; 'i'; 'n'; 'g';
                       ' ']
                     (show_lpoly (_A1, _A2) _B2 ze on (LPoly (a, vas))))
                   [' '; '>'; '='; ' '])
                 (show_lpoly (_A1, _A2) _B2 ze on (LPoly (b, vbs)));
                x]);;

let rec check_polo_nsI (_A1, _A2) _B (_C1, _C2)
  ze on pl ti mz ge pi (s, t) =
    let left = pleftI _A1 _C1 ze on pl ti mz pi s in
    let right = prightI _A1 _C1 ze on pl ti mz pi t in
    catch (check_lpoly_nsI (_A1, _A2) (_C1, _C2) ze on ge left right)
      (fun x ->
        error (append
                 (append
                   (append
                     ['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'e';
                       'n'; 's'; 'u'; 'r'; 'e'; ' ']
                     (sshow_term _B _C2 s))
                   [' '; '>'; '='; ' '])
                 (sshow_term _B _C2 t)
                :: x));;

let rec to_lpoly_inter _B
  on i f n =
    (match lookup (eqb _B eq_nata) (f, n) i with None -> (on, replicate n on)
       | Some cas -> cas);;

let rec checkRedPairProc _A _B
  checkSS checkNSS name representation p rP r =
    let a = partition (fun foo -> isValue (checkSS foo)) p in
    let (sp, nSp) = a in
    catch (bind (bind (catch
                        (catch (checkAllMI checkNSS r) (fun x -> error (snd x)))
                        (fun x ->
                          error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w';
                                   'h'; 'e'; 'n'; ' '; 'o'; 'r'; 'i'; 'e'; 'n';
                                   't'; 'i'; 'n'; 'g'; ' '; 'T'; 'R'; 'S']
                                  :: x)))
                  (fun x ->
                    catch (catch (checkAllMI checkNSS nSp)
                            (fun xa -> error (snd xa)))
                      (fun xa ->
                        error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w';
                                 'h'; 'e'; 'n'; ' '; 'o'; 'r'; 'i'; 'e'; 'n';
                                 't'; 'i'; 'n'; 'g'; ' '; 'D'; 'P'; 's']
                                :: xa))))
            (fun x ->
              catch (checkSubset (eqb (eq_terma _A _B) (eq_terma _A _B)) nSp rP)
                (fun xa ->
                  error ((match checkSS xa with Left err -> err
                           | Right ok ->
                             [['i'; 'n'; 't'; 'e'; 'r'; 'n'; 'a'; 'l'; ' '; 'e';
                                'r'; 'r'; 'o'; 'r'];
                               ['p'; 'l'; 'e'; 'a'; 's'; 'e'; ' '; 'c'; 'o';
                                 'n'; 't'; 'a'; 'c'; 't'; ' '; 'o'; 'n'; 'e';
                                 ' '; 'o'; 'f'; ' '; 't'; 'h'; 'e'; ' '; 'C';
                                 'e'; 'T'; 'A'; ' '; 'm'; 'e'; 'm'; 'b'; 'e';
                                 'r'; 's'];
                               ['h'; 't'; 't'; 'p'; ':'; '/'; '/'; 'c'; 'l';
                                 '-'; 'i'; 'n'; 'f'; 'o'; 'r'; 'm'; 'a'; 't';
                                 'i'; 'k'; '.'; 'u'; 'i'; 'b'; 'k'; '.'; 'a';
                                 'c'; '.'; 'a'; 't'; '/'; 's'; 'o'; 'f'; 't';
                                 'w'; 'a'; 'r'; 'e'; '/'; 'c'; 'e'; 't'; 'a'];
                               ['c'; 'e'; 't'; 'a'; '@'; 'i'; 'n'; 'f'; 'o';
                                 'r'; 'm'; 'a'; 't'; 'i'; 'k'; '.'; 'u'; 'i';
                                 'b'; 'k'; '.'; 'a'; 'c'; '.'; 'a'; 't']])))))
      (fun x ->
        error (['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'a'; 'p'; 'p';
                 'l'; 'y'; ' '; 't'; 'h'; 'e'; ' '; 'r'; 'e'; 'd'; 'u'; 'c';
                 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a'; 'i'; 'r'; ' '; 'p'; 'r';
                 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r'; ' '; 'w'; 'i'; 't'; 'h';
                 ' '; 't'; 'h'; 'e'; ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i';
                 'n'; 'g']
                :: append representation ([] :: x)));;

let rec matchCapRMBelow _A _B
  l rm (Fun (f, ts)) = gcMatchI _A _B (HFun (f, map (capRM _A _B rm) ts)) l;;

let rec checkUrClosedTRM_af (_A1, _A2) (_B1, _B2)
  uu uv uw x3 = match uu, uv, uw, x3 with uu, uv, uw, Var x -> return ()
    | rm, ur, pi, Fun (f, ts) ->
        let n = size_list ts in
        bind (catch
               (checkAllMI
                 (fun i ->
                   (if pi f n i
                     then checkUrClosedTRM_af (_A1, _A2) (_B1, _B2) rm ur pi
                            (nth ts i)
                     else return ()))
                 (upt Zero_nat n))
               (fun x -> error (snd x)))
          (fun x ->
            catch (checkAllMI
                    (fun lr ->
                      checkB
                        (member (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) lr
                           ur
                          || not (matchCapRMBelow _A1 _B1 (fst lr) (Some rm)
                                   (Fun (f, ts))))
                        (append
                          (append
                            (append
                              (append
                                ['d'; 'u'; 'e'; ' '; 't'; 'o'; ' '; 't'; 'h';
                                  'e'; ' '; 's'; 'u'; 'b'; 't'; 'e'; 'r'; 'm';
                                  ' ']
                                (sshow_term _A2 _B2 (Fun (f, ts))))
                              [' '; 'o'; 'f'; ' '; 's'; 'o'; 'm'; 'e'; ' '; 'u';
                                's'; 'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'h'; 's';
                                ','; ' '; 'r'; 'u'; 'l'; 'e'; ' '])
                            (showRule _A2 _B2 lr))
                          [' '; 's'; 'h'; 'o'; 'u'; 'l'; 'd'; ' '; 'b'; 'e';
                            ' '; 'u'; 's'; 'a'; 'b'; 'l'; 'e'; '.']))
                    (rm f))
              (fun xa -> error (snd xa)));;

let rec check_sct_ur_af_proc (_A1, _A2) (_B1, _B2)
  checkSS checkNSS representation p gs r rm rrm ur pi =
    let edgeCheck = checkEdgRM _A1 _B1 rm rrm p true in
    catch ((match rm
             with None ->
               error [['T'; 'R'; 'S'; ' '; 'h'; 'a'; 's'; ' '; 'v'; 'a'; 'r';
                        'i'; 'a'; 'b'; 'l'; 'e'; 's'; ' '; 'a'; 's'; ' '; 'l';
                        'e'; 'f'; 't'; '-'; 'h'; 'a'; 'n'; 'd'; ' '; 's'; 'i';
                        'd'; 'e'; 's']]
             | Some rma ->
               bind (bind (bind (bind (catch
(catch
  (checkAllMI
    (fun lr -> checkUrClosedTRM_af (_A1, _A2) (_B1, _B2) rma ur pi (snd lr)) ur)
  (fun x -> error (snd x)))
(fun x ->
  error [['e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h'; 'e'; 'n'; ' '; 'c'; 'h'; 'e';
           'c'; 'k'; 'i'; 'n'; 'g'; ' '; 'c'; 'l'; 'o'; 's'; 'u'; 'r'; 'e'; ' ';
           'p'; 'r'; 'o'; 'p'; 'e'; 'r'; 't'; 'i'; 'e'; 's'; ' '; 'o'; 'f'; ' ';
           'r'; 'h'; 's'; ' '; 'o'; 'f'; ' '; 'u'; 's'; 'a'; 'b'; 'l'; 'e'; ' ';
           'r'; 'u'; 'l'; 'e'; 's'];
          x]))
                                  (fun x ->
                                    catch (checkSubset
    (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) p (map fst gs))
                                      (fun xa ->
error [append
         ['t'; 'h'; 'e'; 'r'; 'e'; ' '; 'i'; 's'; ' '; 'n'; 'o'; ' '; 's'; 'i';
           'z'; 'e'; '-'; 'c'; 'h'; 'a'; 'n'; 'g'; 'e'; ' '; 'g'; 'r'; 'a'; 'p';
           'h'; ' '; 'f'; 'o'; 'r'; ' '; 'D'; 'P'; ' ']
         (showRule _A2 _B2 xa)])))
                            (fun x ->
                              catch (checkAllMI
                                      (fun ((aa, b) as a) ->
let (s, t) = aa in
(fun ((stri, nstri) as ab) ->
  catch (bind (bind (bind (checkB
                            (not (is_var s) &&
                              (not (is_var t) && null (rma (the (root t)))))
                            [['r'; 'o'; 'o'; 't'; 's'; ' '; 'o'; 'f'; ' '; 'D';
                               'P'; ' '; 'm'; 'u'; 's'; 't'; ' '; 'b'; 'e'; ' ';
                               'c'; 'o'; 'n'; 's'; 't'; 'r'; 'u'; 'c'; 't'; 'o';
                               'r'; 's']])
                      (fun xa ->
                        catch (checkAllMI
                                (fun i ->
                                  checkB (less_nat i (size_list (args s)))
                                    [append
                                       ['l'; 'e'; 'f'; 't'; '-'; 'i'; 'n'; 'd';
 'e'; 'x'; ' '; 't'; 'o'; ' '; 'l'; 'a'; 'r'; 'g'; 'e']
                                       (sshow_nat i)])
                                (remdups eq_nata (map fst (append stri nstri))))
                          (fun xb -> error (snd xb))))
                (fun xa ->
                  catch (checkAllMI
                          (fun j ->
                            checkB
                              (less_nat j (size_list (args t)) &&
                                isValue
                                  (checkUrClosedTRM_af (_A1, _A2) (_B1, _B2) rma
                                    ur pi (nth (args t) j)))
                              [append
                                 ['r'; 'i'; 'g'; 'h'; 't'; '-'; 'i'; 'n'; 'd';
                                   'e'; 'x'; ' '; 't'; 'o'; ' '; 'l'; 'a'; 'r';
                                   'g'; 'e'; ' '; 'o'; 'r'; ' '; 'a'; 'r'; 'g';
                                   'u'; 'm'; 'e'; 'n'; 't'; ' '; 'v'; 'i'; 'o';
                                   'l'; 'a'; 't'; 'e'; 's'; ' '; 'u'; 's'; 'a';
                                   'b'; 'l'; 'e'; '-'; 'r'; 'u'; 'l'; 'e'; 's';
                                   ' '; 'c'; 'o'; 'n'; 'd'; 'i'; 't'; 'i'; 'o';
                                   'n']
                                 (sshow_nat j)])
                          (remdups eq_nata (map snd (append stri nstri))))
                    (fun xb -> error (snd xb))))
          (fun xa ->
            let ss = args s in
            let ts = args t in
            bind (catch
                   (checkAllMI
                     (fun ((i, j) as ac) ->
                       checkB (isValue (checkSS (nth ss i, nth ts j)))
                         [append
                            (append
                              (append
                                ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w';
                                  'i'; 't'; 'h'; ' '; 'e'; 'd'; 'g'; 'e'; ' ']
                                (sshow_nat i))
                              [' '; '-'; 'S'; '-'; '>'; ' '])
                            (sshow_nat j)])
                     stri)
                   (fun xb -> error (snd xb)))
              (fun xb ->
                catch (checkAllMI
                        (fun ((i, j) as ac) ->
                          checkB (isValue (checkNSS (nth ss i, nth ts j)))
                            [append
                               (append
                                 (append
                                   ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w';
                                     'i'; 't'; 'h'; ' '; 'e'; 'd'; 'g'; 'e';
                                     ' ']
                                   (sshow_nat i))
                                 [' '; '-'; 'N'; 'S'; '-'; '>'; ' '])
                               (sshow_nat j)])
                        nstri)
                  (fun xc -> error (snd xc)))))
    (fun xa ->
      error (append
               ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; 's'; ' '; 'w'; 'i'; 't'; 'h';
                 ' '; 'D'; 'P'; ' ']
               (showRule _A2 _B2 (s, t))
              :: xa)))
  b)
                                      gs)
                                (fun xa -> error (snd xa))))
                      (fun x ->
                        catch (catch (checkAllMI checkNSS ur)
                                (fun xa -> error (snd xa)))
                          (fun xa ->
                            error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w';
                                     'h'; 'e'; 'n'; ' '; 'o'; 'r'; 'i'; 'e';
                                     'n'; 't'; 'i'; 'n'; 'g'; ' '; 'u'; 's';
                                     'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'u'; 'l';
                                     'e'; 's']
                                    :: xa))))
                 (fun x ->
                   checkB
                     (check_SCT (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1))
                       eq_nata (fun st uv -> edgeCheck (st, uv))
                       (map (fun ((st, aa) as a) ->
                              let (ab, b) = aa in
                              Scg (st, st, ab, b))
                         gs))
                     [['s'; 'i'; 'z'; 'e'; '-'; 'c'; 'h'; 'a'; 'n'; 'g'; 'e';
                        ' '; 'a'; 'n'; 'a'; 'l'; 'y'; 's'; 'i'; 's'; ' '; 'f';
                        'a'; 'i'; 'l'; 'e'; 'd']])))
      (fun x ->
        error (['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'a'; 'p'; 'p';
                 'l'; 'y'; ' '; 't'; 'h'; 'e'; ' '; 's'; 'i'; 'z'; 'e'; '-';
                 'c'; 'h'; 'a'; 'n'; 'g'; 'e'; ' '; 'p'; 'r'; 'o'; 'c'; 'e';
                 's'; 's'; 'o'; 'r'; ' '; 'w'; 'i'; 't'; 'h'; ' '; 't'; 'h';
                 'e'; ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ']
                :: append representation
                     (['f'; 'o'; 'r'; ' '; 't'; 'h'; 'e'; ' '; 'f'; 'o'; 'l';
                        'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' '; 'r'; 'e'; 'a'; 's';
                        'o'; 'n']
                       :: x)));;

let rec checkNotWellFormed _F (_V1, _V2)
  r = checkB (not (isValue (checkWfTRS _F (_V1, _V2) r)))
        [['T'; 'h'; 'e'; ' '; 'T'; 'R'; 'S'; ' '; 'i'; 's'; ' '; 'w'; 'e'; 'l';
           'l'; ' '; 'f'; 'o'; 'r'; 'm'; 'e'; 'd']];;

let rec checkTRSDisproof (_A1, _A2) (_B1, _B2)
  i r x2 = match i, r, x2 with
    i, r, Loop l ->
      debug i ['L'; 'o'; 'o'; 'p']
        (catch (checkLoop (_A1, _A2) (_B1, _B2) r l)
          (fun x ->
            error (append i
                     [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'i'; 'n'; ' ';
                       'l'; 'o'; 'o'; 'p'; ' '; 'p'; 'r'; 'o'; 'o'; 'f']
                    :: x)))
    | i, r, NotWellFormed ->
        debug i
          ['n'; 'o'; 't'; ' '; 'w'; 'e'; 'l'; 'l'; '-'; 'f'; 'o'; 'r'; 'm'; 'e';
            'd']
          (catch (checkNotWellFormed _A2 (_B1, _B2) r)
            (fun x ->
              error (append i
                       [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'i'; 'n'; ' ';
                         'n'; 'o'; 't'; '-'; 'w'; 'e'; 'l'; 'l'; '-'; 'f'; 'o';
                         'r'; 'm'; 'e'; 'd'; ' '; 'p'; 'r'; 'o'; 'o'; 'f']
                      :: x)));;

let rec init_lab f = UnLab f;;

let rec map_rule fg lr = (map_term fg (fst lr), map_term fg (snd lr));;

let rec sshow_arithFun
  = function Arg i -> append ['x'; '_'] (sshow_nat (Suc i))
    | Const n -> sshow_nat n
    | Sum (f :: fs) ->
        append (sshow_arithFun f)
          (flat_map (fun g -> append [' '; '+'; ' '] (sshow_arithFun g)) fs)
    | Prod (f :: fs) ->
        append
          (append (append ['('] (sshow_arithFun f))
            (flat_map (fun g -> append [' '; '*'; ' '] (sshow_arithFun g)) fs))
          [')']
    | Min (f :: fs) ->
        append
          (append (append ['m'; 'i'; 'n'; '('] (sshow_arithFun f))
            (flat_map (fun g -> append [','] (sshow_arithFun g)) fs))
          [')']
    | Max (f :: fs) ->
        append
          (append (append ['m'; 'a'; 'x'; '('] (sshow_arithFun f))
            (flat_map (fun g -> append [','] (sshow_arithFun g)) fs))
          [')'];;

let rec sshow_sl_inter _C _D
  (SL_Inter (n, fnas)) =
    showList
      (append
         (append
           ['c'; 'a'; 'r'; 'r'; 'i'; 'e'; 'r'; ' '; '{'; '0'; ','; '.'; '.';
             '.'; ',']
           (sshow_nat n))
         ['}']
        :: map (fun ((aa, b) as a) ->
                 let (f, na) = aa in
                 (fun ab ->
                   append
                     (append
                       (append (append (append ['['] (sshow_lab _C _D f)) ['/'])
                         (sshow_nat na))
                       [']'; ' '; '='; ' '])
                     (sshow_arithFun ab))
                   b)
             fnas);;

let rec take_default
  def x1 uu = match def, x1, uu with def, [], uu -> def
    | uv, x :: xs, Zero_nat -> x
    | def, uw :: xs, Suc i -> take_default def xs i;;

let rec eval_arithFun_unbound
  nats x1 = match nats, x1 with nats, Arg i -> take_default Zero_nat nats i
    | nats, Const n -> n
    | nats, Sum [] -> Zero_nat
    | nats, Sum (f :: fs) ->
        plus_nat (eval_arithFun_unbound nats f)
          (eval_arithFun_unbound nats (Sum fs))
    | nats, Prod [] -> one_nat
    | nats, Prod (f :: fs) ->
        times_nat (eval_arithFun_unbound nats f)
          (eval_arithFun_unbound nats (Prod fs))
    | nats, Max [f] -> eval_arithFun_unbound nats f
    | nats, Max (f :: v :: va) ->
        max ord_nat (eval_arithFun_unbound nats f)
          (eval_arithFun_unbound nats (Max (v :: va)))
    | nats, Min [f] -> eval_arithFun_unbound nats f
    | nats, Min (f :: v :: va) ->
        min ord_nat (eval_arithFun_unbound nats f)
          (eval_arithFun_unbound nats (Min (v :: va)));;

let rec eval_arithFun c f nats = mod_nat (eval_arithFun_unbound nats f) c;;

let rec sl_inter_to_inter _A _B
  (SL_Inter (c, ls)) fl cs =
    (match lookup (eqb (eq_laba _A _B) eq_nata) (fl, size_list cs) ls
       with None -> Zero_nat | Some f -> eval_arithFun (Suc c) f cs);;

let rec get_largest_element (SL_Inter (n, uu)) = n;;

let rec create_sem_lab_tt (_F1, _F2) (_L1, _L2) (_V1, _V2)
  sli l r lr =
    let c = get_largest_element sli in
    let ca = upt Zero_nat (Suc c) in
    let i = sl_inter_to_inter _F1 _L1 sli in
    catch (check_sem_lab_tt (_F1, _F2) (_L1, _L2) (_V1, _V2)
            (check_sl_lab_trs (_F1, _F2) (_L1, _L2) (eq_nata, sshow_nata)
              (_V1, _V2) i l ca)
            (check_sl_model_lab_trs (_F1, _F2) (_L1, _L2) (eq_nata, sshow_nata)
              (_V1, _V2) i l ca)
            (fun x -> false) i l r lr)
      (fun x -> error (sshow_sl_inter _F2 _L2 sli :: x));;

let rec map_trsI fg r = map (map_rule fg) r;;

let rec unlab_of_trsI _F _L _V
  r = remdups (eqb (eq_terma (eq_laba _F _L) _V) (eq_terma (eq_laba _F _L) _V))
        (map_trsI unlab
          (filter
            (fun ((l, ra) as a) ->
              eqop (eq_terma (eq_laba _F _L) _V) l ra ||
                not (eqop (eq_terma (eq_laba _F _L) _V) (map_term unlab l)
                      (map_term unlab ra)))
            r));;

let rec unlab_trs_tt (_F1, _F2) (_L1, _L2) (_V1, _V2)
  r rR =
    let u = unlab_of_trsI _F1 _L1 _V1 r in
    catch (bind (checkSubset
                  (eqb (eq_terma (eq_laba _F1 _L1) _V1)
                    (eq_terma (eq_laba _F1 _L1) _V1))
                  rR u)
            (fun x ->
              checkSubset
                (eqb (eq_terma (eq_laba _F1 _L1) _V1)
                  (eq_terma (eq_laba _F1 _L1) _V1))
                u rR))
      (fun x ->
        error (append (showRules (sshow_laba _F2 _L2) _V2 rR)
                (append
                  [[' '; ' '; 'd'; 'o'; 'e'; 's'; ' '; 'n'; 'o'; 't'; ' '; 'c';
                     'o'; 'r'; 'r'; 'e'; 's'; 'p'; 'o'; 'n'; 'd'; ' '; 't'; 'o';
                     ' '; 'u'; 'n'; 'l'; 'a'; 'b'; 'e'; 'l'; 'e'; 'd'; ' '; 'T';
                     'R'; 'S'; ' '; ' ']]
                  (showRules (sshow_laba _F2 _L2) _V2 u))));;

let rec get_comp_description
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = de;;

let rec create_lpoly_repr (_A1, _A2) (_B1, _B2)
  c i = let pi = to_lpoly_inter _B1 (get_comp_1 c) i in
        append
          [append
             ['p'; 'o'; 'l'; 'y'; 'n'; 'o'; 'm'; 'i'; 'a'; 'l'; ' '; 'i'; 'n';
               't'; 'e'; 'r'; 'p'; 'r'; 'e'; 't'; 'r'; 'a'; 't'; 'i'; 'o'; 'n';
               ' '; 'o'; 'v'; 'e'; 'r'; ' ']
             (get_comp_description c)]
          (map (fun ((f, n) as a) ->
                 let t =
                   Fun (f, map (fun aa -> Var aa)
                             (fresh_strings_list ['x'; '_'] one_nat [] n))
                   in
                 append
                   (append
                     (append ['P'; 'o'; 'l'; '(']
                       (sshow_term _B2 (sshow_lista sshow_chara) t))
                     [')'; ' '; '='; ' '])
                   (show_lpoly (_A1, _A2) (sshow_lista sshow_chara)
                     (get_comp_0 c) (get_comp_1 c)
                     (pleftI _A1 (eq_lista eq_char) (get_comp_0 c)
                       (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
                       (get_comp_max0 c) pi t)))
            (remdups (eqb _B1 eq_nata) (map fst i)));;

let rec check_poly_monoI _A _B
  ze ge cm =
    (fun xs ->
      catch (checkAllMI
              (fun ((aa, b) as a) ->
                let (f, n) = aa in
                (fun ((c, cs) as ab) ->
                  catch (bind (bind (checkB (ge c ze)
                                      [append
 (append
   (append
     ['c'; 'o'; 'n'; 's'; 't'; 'a'; 'n'; 't'; ' '; 'p'; 'a'; 'r'; 't'; ' ']
     (sshow _A c))
   [' '; 'm'; 'u'; 's'; 't'; ' '; 'b'; 'e'; ' '; 'a'; 't'; ' '; 'l'; 'e'; 'a';
     's'; 't'; ' '])
 (sshow _A ze)])
                                (fun x ->
                                  checkB (less_eq_nat n (size_list cs))
                                    [['t'; 'h'; 'e'; ' '; 'l'; 'a'; 's'; 't';
                                       ' '; 'a'; 'r'; 'g'; 'u'; 'm'; 'e'; 'n';
                                       't'; ' '; 'i'; 's'; ' '; 'i'; 'g'; 'n';
                                       'o'; 'r'; 'e'; 'd']]))
                          (fun x ->
                            catch (checkAllMI
                                    (fun d ->
                                      checkB (cm d)
[append
   (append ['c'; 'o'; 'e'; 'f'; 'f'; 'i'; 'c'; 'i'; 'e'; 'n'; 't'; ' ']
     (sshow _A d))
   [' '; 'i'; 's'; ' '; 'n'; 'o'; 't'; ' '; 'a'; 'l'; 'l'; 'o'; 'w'; 'e'; 'd']])
                                    cs)
                              (fun xa -> error (snd xa))))
                    (fun x ->
                      error (append
                               (append
                                 (append
                                   ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w';
                                     'i'; 't'; 'h'; ' '; 'm'; 'o'; 'n'; 'o';
                                     't'; 'o'; 'n'; 'i'; 'c'; 'i'; 't'; 'y';
                                     ' '; 'd'; 'u'; 'e'; ' '; 't'; 'o'; ' ';
                                     'i'; 'n'; 't'; 'e'; 'r'; 'p'; 'r'; 'e';
                                     't'; 'a'; 't'; 'i'; 'o'; 'n'; ' '; 'o';
                                     'f'; ' ']
                                   (sshow _B f))
                                 ['/'])
                               (sshow_nat n)
                              :: x)))
                  b)
              xs)
        (fun x -> error (snd x)));;

let rec checkRuleRemovalProc _A _B
  checkMono checkSS checkNSS name representation r rR =
    let a = partition (fun foo -> isValue (checkSS foo)) r in
    let (sp, nSp) = a in
    catch (bind (bind (bind (catch checkMono
                              (fun x ->
                                error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' ';
 'w'; 'i'; 't'; 'h'; ' '; 'm'; 'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; 'i'; 't';
 'y'; ' '; 'o'; 'f'; ' '; 's'; 't'; 'r'; 'i'; 'c'; 't'; ' '; 'o'; 'r'; 'd'; 'e';
 'r']
:: x)))
                        (fun x ->
                          catch (catch (checkAllMI checkNSS r)
                                  (fun xa -> error (snd xa)))
                            (fun xa ->
                              error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' ';
                                       'w'; 'h'; 'e'; 'n'; ' '; 'o'; 'r'; 'i';
                                       'e'; 'n'; 't'; 'i'; 'n'; 'g'; ' '; 'r';
                                       'u'; 'l'; 'e'; 's']
                                      :: xa))))
                  (fun x ->
                    catch (checkSubset (eqb (eq_terma _A _B) (eq_terma _A _B))
                            nSp rR)
                      (fun xa ->
                        error ((match checkSS xa with Left err -> err
                                 | Right ok ->
                                   [['i'; 'n'; 't'; 'e'; 'r'; 'n'; 'a'; 'l';
                                      ' '; 'e'; 'r'; 'r'; 'o'; 'r'];
                                     ['p'; 'l'; 'e'; 'a'; 's'; 'e'; ' '; 'c';
                                       'o'; 'n'; 't'; 'a'; 'c'; 't'; ' '; 'o';
                                       'n'; 'e'; ' '; 'o'; 'f'; ' '; 't'; 'h';
                                       'e'; ' '; 'C'; 'e'; 'T'; 'A'; ' '; 'm';
                                       'e'; 'm'; 'b'; 'e'; 'r'; 's'];
                                     ['h'; 't'; 't'; 'p'; ':'; '/'; '/'; 'c';
                                       'l'; '-'; 'i'; 'n'; 'f'; 'o'; 'r'; 'm';
                                       'a'; 't'; 'i'; 'k'; '.'; 'u'; 'i'; 'b';
                                       'k'; '.'; 'a'; 'c'; '.'; 'a'; 't'; '/';
                                       's'; 'o'; 'f'; 't'; 'w'; 'a'; 'r'; 'e';
                                       '/'; 'c'; 'e'; 't'; 'a'];
                                     ['c'; 'e'; 't'; 'a'; '@'; 'i'; 'n'; 'f';
                                       'o'; 'r'; 'm'; 'a'; 't'; 'i'; 'k'; '.';
                                       'u'; 'i'; 'b'; 'k'; '.'; 'a'; 'c'; '.';
                                       'a'; 't']])))))
            (fun x ->
              catch (checkSubset (eqb (eq_terma _A _B) (eq_terma _A _B)) rR r)
                (fun xa ->
                  error ((match checkSS xa with Left err -> err
                           | Right ok ->
                             [['i'; 'n'; 't'; 'e'; 'r'; 'n'; 'a'; 'l'; ' '; 'e';
                                'r'; 'r'; 'o'; 'r'];
                               ['p'; 'l'; 'e'; 'a'; 's'; 'e'; ' '; 'c'; 'o';
                                 'n'; 't'; 'a'; 'c'; 't'; ' '; 'o'; 'n'; 'e';
                                 ' '; 'o'; 'f'; ' '; 't'; 'h'; 'e'; ' '; 'C';
                                 'e'; 'T'; 'A'; ' '; 'm'; 'e'; 'm'; 'b'; 'e';
                                 'r'; 's'];
                               ['h'; 't'; 't'; 'p'; ':'; '/'; '/'; 'c'; 'l';
                                 '-'; 'i'; 'n'; 'f'; 'o'; 'r'; 'm'; 'a'; 't';
                                 'i'; 'k'; '.'; 'u'; 'i'; 'b'; 'k'; '.'; 'a';
                                 'c'; '.'; 'a'; 't'; '/'; 's'; 'o'; 'f'; 't';
                                 'w'; 'a'; 'r'; 'e'; '/'; 'c'; 'e'; 't'; 'a'];
                               ['c'; 'e'; 't'; 'a'; '@'; 'i'; 'n'; 'f'; 'o';
                                 'r'; 'm'; 'a'; 't'; 'i'; 'k'; '.'; 'u'; 'i';
                                 'b'; 'k'; '.'; 'a'; 'c'; '.'; 'a'; 't']])))))
      (fun x ->
        error (['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'a'; 'p'; 'p';
                 'l'; 'y'; ' '; 't'; 'h'; 'e'; ' '; 'r'; 'u'; 'l'; 'e'; ' ';
                 'r'; 'e'; 'm'; 'o'; 'v'; 'a'; 'l'; ' '; 'p'; 'r'; 'o'; 'c';
                 'e'; 's'; 's'; 'o'; 'r'; ' '; 'w'; 'i'; 't'; 'h'; ' '; 't';
                 'h'; 'e'; ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g']
                :: append representation ([] :: x)));;

let rec get_comp_wf_elem
  (wfe, (ze, (on, (pl, (ti, (mz, (ge, (gt, (mo, (de, ap)))))))))) = wfe;;

let rec check_lpoly_coeffs _A _B
  wfe apos ch i =
    catch (checkAllMI
            (fun ((aa, b) as a) ->
              let (f, n) = aa in
              (fun ((c, cs) as ab) ->
                catch (bind (bind (checkB (wfe c)
                                    [append
                                       (append
 ['c'; 'o'; 'n'; 's'; 't'; 'a'; 'n'; 't'; ' '; 'p'; 'a'; 'r'; 't'; ' ']
 (sshow _A c))
                                       [' '; 'i'; 's'; ' '; 'n'; 'o'; 't'; ' ';
 'w'; 'e'; 'l'; 'l'; '-'; 'f'; 'o'; 'r'; 'm'; 'e'; 'd']])
                              (fun x ->
                                checkB (apos c || any apos (take n cs))
                                  [['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't';
                                     ' '; 'f'; 'i'; 'n'; 'd'; ' '; 'p'; 'o';
                                     's'; 'i'; 't'; 'i'; 'v'; 'e'; ' '; 'e';
                                     'n'; 't'; 'r'; 'y'; ' '; 'w'; 'h'; 'i';
                                     'c'; 'h'; ' '; 'i'; 's'; ' '; 'r'; 'e';
                                     'q'; 'u'; 'i'; 'r'; 'e'; 'd'; ' '; 'f';
                                     'o'; 'r'; ' '; 'a'; 'r'; 'c'; 't'; 'i';
                                     'c'; ' '; 'i'; 'n'; 't'; 'e'; 'r'; 'p';
                                     'r'; 'e'; 't'; 'a'; 't'; 'i'; 'o'; 'n';
                                     's']]))
                        (fun x ->
                          catch (checkAllMI
                                  (fun ac ->
                                    checkB (ch ac && wfe ac)
                                      [append
 (append ['c'; 'o'; 'e'; 'f'; 'f'; 'i'; 'c'; 'i'; 'e'; 'n'; 't'; ' ']
   (sshow _A ac))
 [' '; 'i'; 's'; ' '; 'n'; 'o'; 't'; ' '; 'a'; 'l'; 'l'; 'o'; 'w'; 'e'; 'd']])
                                  cs)
                            (fun xa -> error (snd xa))))
                  (fun x ->
                    error (append
                             (append
                               (append
                                 ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w';
                                   'i'; 't'; 'h'; ' '; 'i'; 'n'; 't'; 'e'; 'r';
                                   'p'; 'r'; 'e'; 't'; 'a'; 't'; 'i'; 'o'; 'n';
                                   ' '; 'o'; 'f'; ' ']
                                 (sshow _B f))
                               ['/'])
                             (sshow_nat n)
                            :: x)))
                b)
            i)
      (fun x -> error (snd x));;

let rec create_rule_removal_proc_comp (_A1, _A2) (_B1, _B2) (_C1, _C2)
  c cm i r rR =
    let pi = to_lpoly_inter _B1 (get_comp_1 c) i in
    bind (check_lpoly_coeffs _A2 _B2 (get_comp_wf_elem c) (get_comp_apos c)
           (fun x -> get_comp_ge c x (get_comp_0 c)) i)
      (fun x ->
        checkRuleRemovalProc _B1 _C1
          (check_poly_monoI _A2 _B2 (get_comp_0 c) (get_comp_ge c) cm i)
          (check_polo_sI (_A1, _A2) _B2 (_C1, _C2) (get_comp_mono c)
            (get_comp_0 c) (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) (get_comp_gt c) pi)
          (check_polo_nsI (_A1, _A2) _B2 (_C1, _C2) (get_comp_0 c)
            (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) pi)
          (append
            (append
              ['p'; 'o'; 'l'; 'y'; 'n'; 'o'; 'm'; 'i'; 'a'; 'l'; ' '; 'o'; 'r';
                'd'; 'e'; 'r'; ' '; '(']
              (get_comp_description c))
            [')'])
          (create_lpoly_repr (_A1, _A2) (_B1, _B2) c i) r rR);;

let rec create_rule_removal_proc (_A1, _A2) (_B1, _B2)
  = function
    Int_carrier ia ->
      create_rule_removal_proc_comp (eq_int, sshow_inta) (_A1, _A2) (_B1, _B2)
        ((fun x -> true),
          ((Big_int.big_int_of_int 0),
            ((Big_int.big_int_of_int 1),
              (Big_int.add_big_int,
                (Big_int.mult_big_int,
                  (max ord_int (Big_int.big_int_of_int 0),
                    ((fun x y -> Big_int.le_big_int y x),
                      ((fun x y -> Big_int.lt_big_int y x),
                        (true, (int_name, (fun x -> true)))))))))))
        (Big_int.lt_big_int (Big_int.big_int_of_int 0)) ia
    | Nat_carrier i ->
        create_rule_removal_proc_comp (eq_nata, sshow_nata) (_A1, _A2)
          (_B1, _B2)
          ((fun x -> true),
            (Zero_nat,
              (one_nat,
                (plus_nat,
                  (times_nat,
                    (id, ((fun x y -> less_eq_nat y x),
                           ((fun x y -> less_nat y x),
                             (true, (nat_name, (fun x -> true)))))))))))
          (fun x -> not (eqop eq_nata x Zero_nat)) i
    | Arctic_carrier id ->
        (fun uu uua ->
          error [['a'; 'r'; 'c'; 't'; 'i'; 'c'; ' '; 'n'; 'o'; 't'; ' '; 'm';
                   'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; ' '; 'f'; 'o'; 'r';
                   ' '; 'T'; 'R'; 'S'; 's']])
    | Arc_mat_carrier (n, ic) ->
        (fun uu uua ->
          error [['a'; 'r'; 'c'; 't'; 'i'; 'c'; ' '; 'n'; 'o'; 't'; ' '; 'm';
                   'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; ' '; 'f'; 'o'; 'r';
                   ' '; 'T'; 'R'; 'S'; 's']])
    | Int_mat_carrier (n, ib) ->
        (fun uu uua ->
          error [['m'; 'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; ' '; 'm'; 'a';
                   't'; 'r'; 'i'; 'x'; '-'; 'o'; 'r'; 'd'; 'e'; 'r'; 's'; ' ';
                   'n'; 'o'; 't'; ' '; 'y'; 'e'; 't'; ' '; 's'; 'u'; 'p'; 'p';
                   'o'; 'r'; 't'; 'e'; 'd']]);;

let rec computeRuleMap _A
  = function [] -> Some (fun f -> [])
    | (Fun (f, ts), r) :: rules ->
        (match computeRuleMap _A rules with None -> None
           | Some rm -> Some (fun_upd _A rm f ((Fun (f, ts), r) :: rm f)))
    | (Var uu, uv) :: rules -> None;;

let rec create_sct_ur_af_proc_comp (_A1, _A2) (_B1, _B2) (_C1, _C2)
  c i p r rm rrm ur graphs =
    let pi = to_lpoly_inter _B1 (get_comp_1 c) i in
    bind (check_lpoly_coeffs _A2 _B2 (get_comp_wf_elem c) (get_comp_apos c)
           (fun x -> get_comp_ge c x (get_comp_0 c)) i)
      (fun x ->
        check_sct_ur_af_proc (_B1, _B2) (_C1, _C2)
          (check_polo_sI (_A1, _A2) _B2 (_C1, _C2) (get_comp_mono c)
            (get_comp_0 c) (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) (get_comp_gt c) pi)
          (check_polo_nsI (_A1, _A2) _B2 (_C1, _C2) (get_comp_0 c)
            (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) pi)
          (append
             (append
               ['p'; 'o'; 'l'; 'y'; 'n'; 'o'; 'm'; 'i'; 'a'; 'l'; ' '; 'o'; 'r';
                 'd'; 'e'; 'r'; ' '; '(']
               (get_comp_description c))
             [')']
            :: create_lpoly_repr (_A1, _A2) (_B1, _B2) c i)
          p graphs r rm rrm ur (create_afI _A1 _B1 (get_comp_0 c) i));;

let rec create_sct_ur_af_proc (_A1, _A2) (_B1, _B2)
  = function
    Int_carrier ia ->
      create_sct_ur_af_proc_comp (eq_int, sshow_inta) (_A1, _A2) (_B1, _B2)
        ((fun x -> true),
          ((Big_int.big_int_of_int 0),
            ((Big_int.big_int_of_int 1),
              (Big_int.add_big_int,
                (Big_int.mult_big_int,
                  (max ord_int (Big_int.big_int_of_int 0),
                    ((fun x y -> Big_int.le_big_int y x),
                      ((fun x y -> Big_int.lt_big_int y x),
                        (true, (int_name, (fun x -> true)))))))))))
        ia
    | Nat_carrier i ->
        create_sct_ur_af_proc_comp (eq_nata, sshow_nata) (_A1, _A2) (_B1, _B2)
          ((fun x -> true),
            (Zero_nat,
              (one_nat,
                (plus_nat,
                  (times_nat,
                    (id, ((fun x y -> less_eq_nat y x),
                           ((fun x y -> less_nat y x),
                             (true, (nat_name, (fun x -> true)))))))))))
          i
    | Arctic_carrier ida ->
        create_sct_ur_af_proc_comp (eq_arctica, sshow_arctica) (_A1, _A2)
          (_B1, _B2)
          ((fun x -> true),
            (MinInfty,
              (Num_arc (Big_int.big_int_of_int 0),
                (plus_arctic,
                  (mult_arctic,
                    (id, (geq_arctic,
                           (gt_arctic,
                             (false, (arctic_name, pos_arctic))))))))))
          ida
    | Int_mat_carrier (n, ib) ->
        create_sct_ur_af_proc_comp
          (eq_lista (eq_lista eq_int), sshow_lista (sshow_lista sshow_inta))
          (_A1, _A2) (_B1, _B2)
          (wf_mat_rcI (Suc n) (Suc n),
            (mat0I (Big_int.big_int_of_int 0) (Suc n) (Suc n),
              (mat1I (Big_int.big_int_of_int 0) (Big_int.big_int_of_int 1)
                 (Suc n),
                (mat_plusI Big_int.add_big_int,
                  (mat_mulI (Big_int.big_int_of_int 0) Big_int.add_big_int
                     Big_int.mult_big_int (Suc n),
                    (mat_max0I (max ord_int (Big_int.big_int_of_int 0)),
                      (mat_geI (fun x y -> Big_int.le_big_int y x),
                        (mat_gtI (fun x y -> Big_int.le_big_int y x)
                           (fun x y -> Big_int.lt_big_int y x),
                          (true,
                            (int_mat_name,
                              mat_arc_posI (fun x -> true)))))))))))
          ib
    | Arc_mat_carrier (n, ic) ->
        create_sct_ur_af_proc_comp
          (eq_lista (eq_lista eq_arctica),
            sshow_lista (sshow_lista sshow_arctica))
          (_A1, _A2) (_B1, _B2)
          (wf_mat_rcI (Suc n) (Suc n),
            (mat0I MinInfty (Suc n) (Suc n),
              (mat1I MinInfty (Num_arc (Big_int.big_int_of_int 0)) (Suc n),
                (mat_plusI plus_arctic,
                  (mat_mulI MinInfty plus_arctic mult_arctic (Suc n),
                    (mat_max0I id,
                      (mat_geI geq_arctic,
                        (mat_geI gt_arctic,
                          (false,
                            (arc_mat_name, mat_arc_posI pos_arctic))))))))))
          ic;;

let rec check_sct_subterm_proc (_A1, _A2) (_B1, _B2)
  p gs r rm rrm =
    let edge_check = checkEdgRM _A1 _B1 rm rrm p true in
    catch ((match rm
             with None ->
               error [['T'; 'R'; 'S'; ' '; 'h'; 'a'; 's'; ' '; 'v'; 'a'; 'r';
                        'i'; 'a'; 'b'; 'l'; 'e'; 's'; ' '; 'a'; 's'; ' '; 'l';
                        'e'; 'f'; 't'; '-'; 'h'; 'a'; 'n'; 'd'; ' '; 's'; 'i';
                        'd'; 'e'; 's']]
             | Some rma ->
               bind (bind (catch
                            (checkSubset
                              (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) p
                              (map fst gs))
                            (fun x ->
                              error [append
                                       ['t'; 'h'; 'e'; 'r'; 'e'; ' '; 'i'; 's';
 ' '; 'n'; 'o'; ' '; 's'; 'i'; 'z'; 'e'; '-'; 'c'; 'h'; 'a'; 'n'; 'g'; 'e'; ' ';
 'g'; 'r'; 'a'; 'p'; 'h'; ' '; 'f'; 'o'; 'r'; ' '; 'D'; 'P'; ' ']
                                       (showRule _A2 _B2 x)]))
                      (fun x ->
                        catch (checkAllMI
                                (fun ((aa, b) as a) ->
                                  let (s, t) = aa in
                                  (fun ((stri, nstri) as ab) ->
                                    catch (bind
    (checkB (not (is_var s) && (not (is_var t) && null (rma (the (root t)))))
      [['r'; 'o'; 'o'; 't'; 's'; ' '; 'o'; 'f'; ' '; 'D'; 'P'; ' '; 'm'; 'u';
         's'; 't'; ' '; 'b'; 'e'; ' '; 'c'; 'o'; 'n'; 's'; 't'; 'r'; 'u'; 'c';
         't'; 'o'; 'r'; 's']])
    (fun xa ->
      let ss = args s in
      let ts = args t in
      let n = size_list ss in
      let m = size_list ts in
      bind (catch
             (checkAllMI
               (fun ((i, j) as ac) ->
                 checkB
                   (less_nat i n &&
                     (less_nat j m &&
                       isValue
                         (check_supt (_A1, _A2) (_B1, _B2)
                           (nth ss i, nth ts j))))
                   [append
                      (append
                        (append
                          ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'i';
                            't'; 'h'; ' '; 'e'; 'd'; 'g'; 'e'; ' ']
                          (sshow_nat i))
                        [' '; '|'; '>'; ' '])
                      (sshow_nat j)])
               stri)
             (fun xb -> error (snd xb)))
        (fun xb ->
          catch (checkAllMI
                  (fun ((i, j) as ac) ->
                    checkB
                      (less_nat i n &&
                        (less_nat j m &&
                          isValue
                            (check_supteq (_A1, _A2) (_B1, _B2)
                              (nth ss i, nth ts j))))
                      [append
                         (append
                           (append
                             ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'i';
                               't'; 'h'; ' '; 'e'; 'd'; 'g'; 'e'; ' ']
                             (sshow_nat i))
                           [' '; '|'; '>'; '='; ' '])
                         (sshow_nat j)])
                  nstri)
            (fun xc -> error (snd xc)))))
                                      (fun xa ->
error (append
         ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; 's'; ' '; 'w'; 'i'; 't'; 'h'; ' ';
           'D'; 'P'; ' ']
         (showRule _A2 _B2 (s, t))
        :: xa)))
                                    b)
                                gs)
                          (fun xa -> error (snd xa))))
                 (fun x ->
                   checkB
                     (check_SCT (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1))
                       eq_nata (fun st uv -> edge_check (st, uv))
                       (map (fun ((st, aa) as a) ->
                              let (ab, b) = aa in
                              Scg (st, st, ab, b))
                         gs))
                     [['s'; 'i'; 'z'; 'e'; '-'; 'c'; 'h'; 'a'; 'n'; 'g'; 'e';
                        ' '; 'a'; 'n'; 'a'; 'l'; 'y'; 's'; 'i'; 's'; ' '; 'f';
                        'a'; 'i'; 'l'; 'e'; 'd']])))
      (fun x ->
        error (['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'a'; 'p'; 'p';
                 'l'; 'y'; ' '; 't'; 'h'; 'e'; ' '; 's'; 'i'; 'z'; 'e'; '-';
                 'c'; 'h'; 'a'; 'n'; 'g'; 'e'; ' '; 'p'; 'r'; 'o'; 'c'; 'e';
                 's'; 's'; 'o'; 'r'; ' '; 'w'; 'i'; 't'; 'h'; ' '; 's'; 'u';
                 'b'; 't'; 'e'; 'r'; 'm'; '-'; 'r'; 'e'; 'l'; 'a'; 't'; 'i';
                 'o'; 'n'; ' '; 'f'; 'o'; 'r'; ' '; 't'; 'h'; 'e'; ' '; 'f';
                 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' '; 'r'; 'e'; 'a';
                 's'; 'o'; 'n']
                :: x));;

let rec create_sp_representation _A
  = function SimpProj [] -> []
    | SimpProj ((f, n) :: sp) ->
        concat [['p'; 'i'; '(']; sshow _A f; [')'; ' '; '='; ' ']; sshow_nat n]
          :: create_sp_representation _A (SimpProj sp);;

let rec check_for_var _F _V
  rule =
    catch (checkB (not (is_var (fst rule)))
            [['t'; 'h'; 'e'; ' '; 'l'; 'h'; 's'; ' '; 'i'; 's'; ' '; 'a'; ' ';
               'v'; 'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e']])
      (fun x ->
        error (concat
                 [sshow_term _F _V (fst rule);
                   [' '; 'i'; 's'; ' '; 'a'; ' '; 'v'; 'a'; 'r'; 'i'; 'a'; 'b';
                     'l'; 'e']]
                :: x));;

let rec lift2term
  p x1 = match p, x1 with p, Var x -> Var x
    | p, Fun (f, ss) -> nth ss (p f);;

let rec check_supteq_proj (_F1, _F2) (_V1, _V2)
  p lr =
    catch (checkB
            (eqop (eq_terma _F1 _V1) (lift2term p (fst lr))
               (lift2term p (snd lr))
              || checkSupterm _F1 _V1 (lift2term p (fst lr))
                   (lift2term p (snd lr)))
            [['t'; 'h'; 'e'; ' '; 'p'; 'r'; 'o'; 'j'; 'e'; 'c'; 't'; 'e'; 'd';
               ' '; 'r'; 'h'; 's'; ' '; 'i'; 's'; ' '; 'n'; 'o'; 't'; ' '; 'a';
               ' '; 's'; 'u'; 'b'; 't'; 'e'; 'r'; 'm'; ' '; 'o'; 'f'; ' '; 't';
               'h'; 'e'; ' '; 'p'; 'r'; 'o'; 'j'; 'e'; 'c'; 't'; 'e'; 'd'; ' ';
               'l'; 'h'; 's']])
      (fun x ->
        error (concat
                 [['C'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'o'; 'r';
                    'i'; 'e'; 'n'; 't'; ' '; 'r'; 'u'; 'l'; 'e'; ' '];
                   sshow_term _F2 _V2 (fst lr); [' '; '-'; '>'; ' '];
                   sshow_term _F2 _V2 (snd lr)]
                :: concat
                     [['i'; '.'; 'e'; '.'; ','; ' '];
                       sshow_term _F2 _V2 (lift2term p (fst lr));
                       [' '; '|'; '>'; '='; ' '];
                       sshow_term _F2 _V2 (lift2term p (snd lr))]
                     :: x));;

let rec check_projection (_A1, _A2) _B
  ds p x2 = match ds, p, x2 with
    ds, p, (Fun (f, ss), Fun (g, ts)) ->
      bind (bind (bind (bind (bind (checkB (less_nat Zero_nat (size_list ss))
                                     [['l'; 'h'; 's'; ' '; 'i'; 's'; ' '; 'a';
' '; 'c'; 'o'; 'n'; 's'; 't'; 'a'; 'n'; 't']])
                               (fun x ->
                                 checkB (not (member _A1 f ds))
                                   [concat
                                      [sshow _A2 f;
[' '; 'i'; 's'; ' '; 'd'; 'e'; 'f'; 'i'; 'n'; 'e'; 'd'; ' '; 'i'; 'n'; ' ';
  'R']]]))
                         (fun x ->
                           checkB (less_nat (p f) (size_list ss))
                             [concat
                                [sshow_nat (p f);
                                  [' '; 'i'; 's'; ' '; 'n'; 'o'; 't'; ' '; 'a';
                                    'n'; ' '; 'a'; 'r'; 'g'; 'u'; 'm'; 'e'; 'n';
                                    't'; ' '; 'p'; 'o'; 's'; 'i'; 't'; 'i'; 'o';
                                    'n'; ' '; 'o'; 'f'; ' '];
                                  sshow_term _A2 _B (Fun (f, ss))]]))
                   (fun x ->
                     checkB (less_nat Zero_nat (size_list ts))
                       [['r'; 'h'; 's'; ' '; 'i'; 's'; ' '; 'a'; ' '; 'c'; 'o';
                          'n'; 's'; 't'; 'a'; 'n'; 't']]))
             (fun x ->
               checkB (not (member _A1 g ds))
                 [concat
                    [sshow _A2 g;
                      [' '; 'i'; 's'; ' '; 'd'; 'e'; 'f'; 'i'; 'n'; 'e'; 'd';
                        ' '; 'i'; 'n'; ' '; 'R']]]))
        (fun x ->
          checkB (less_nat (p g) (size_list ts))
            [concat
               [sshow_nat (p g);
                 [' '; 'i'; 's'; ' '; 'n'; 'o'; 't'; ' '; 'a'; 'n'; ' '; 'a';
                   'r'; 'g'; 'u'; 'm'; 'e'; 'n'; 't'; ' '; 'p'; 'o'; 's'; 'i';
                   't'; 'i'; 'o'; 'n'; ' '; 'o'; 'f'; ' '];
                 sshow_term _A2 _B (Fun (g, ts))]])
    | ds, p, (Var v, t) ->
        error [['c'; 'a'; 'n'; 'n'; 'o'; 't'; ' '; 'p'; 'r'; 'o'; 'j'; 'e'; 'c';
                 't'; ' '; 'a'; ' '; 'v'; 'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e']]
    | ds, p, (s, Var v) ->
        error [['c'; 'a'; 'n'; 'n'; 'o'; 't'; ' '; 'p'; 'r'; 'o'; 'j'; 'e'; 'c';
                 't'; ' '; 'a'; ' '; 'v'; 'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e']];;

let rec check_supt_proj (_F1, _F2) (_V1, _V2)
  p lr =
    catch (checkB
            (checkSupterm _F1 _V1 (lift2term p (fst lr)) (lift2term p (snd lr)))
            [['t'; 'h'; 'e'; ' '; 'p'; 'r'; 'o'; 'j'; 'e'; 'c'; 't'; 'e'; 'd';
               ' '; 'r'; 'h'; 's'; ' '; 'i'; 's'; ' '; 'n'; 'o'; 't'; ' '; 'a';
               ' '; 'p'; 'r'; 'o'; 'p'; 'e'; 'r'; ' '; 's'; 'u'; 'b'; 't'; 'e';
               'r'; 'm'; ' '; 'o'; 'f'; ' '; 't'; 'h'; 'e'; ' '; 'p'; 'r'; 'o';
               'j'; 'e'; 'c'; 't'; 'e'; 'd'; ' '; 'l'; 'h'; 's']])
      (fun x ->
        error (concat
                 [['C'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'o'; 'r';
                    'i'; 'e'; 'n'; 't'; ' '; 'r'; 'u'; 'l'; 'e'; ' '];
                   sshow_term _F2 _V2 (fst lr); [' '; '-'; '>'; ' '];
                   sshow_term _F2 _V2 (snd lr)]
                :: concat
                     [['i'; '.'; 'e'; '.'; ','; ' '];
                       sshow_term _F2 _V2 (lift2term p (fst lr));
                       [' '; '|'; '>'; ' '];
                       sshow_term _F2 _V2 (lift2term p (snd lr))]
                     :: x));;

let rec check_spsc_proc (_A1, _A2) (_B1, _B2)
  proj rep p rP r =
    let ds = defs_list _A1 r in
    let a =
      partition
        (fun ra -> isValue (check_supt_proj (_A1, _A2) (_B1, _B2) proj ra)) p
      in
    let (sp, nSp) = a in
    catch (bind (bind (bind (catch
                              (catch
                                (checkAllMI
                                  (check_projection (_A1, _A2) _B2 ds proj) p)
                                (fun x -> error (snd x)))
                              (fun x ->
                                error (['t'; 'h'; 'e'; ' '; 's'; 'u'; 'p'; 'p';
 'l'; 'i'; 'e'; 'd'; ' '; 'p'; 'r'; 'o'; 'j'; 'e'; 'c'; 't'; 'i'; 'o'; 'n'; ' ';
 'i'; 's'; ' '; 'm'; 'a'; 'l'; 'f'; 'o'; 'r'; 'm'; 'e'; 'd']
:: x)))
                        (fun x ->
                          catch (catch
                                  (checkAllMI
                                    (check_supteq_proj (_A1, _A2) (_B1, _B2)
                                      proj)
                                    nSp)
                                  (fun xa -> error (snd xa)))
                            (fun xa ->
                              error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' ';
                                       'w'; 'h'; 'e'; 'n'; ' '; 'o'; 'r'; 'i';
                                       'e'; 'n'; 't'; 'i'; 'n'; 'g'; ' '; 'D';
                                       'P'; 's']
                                      :: xa))))
                  (fun x ->
                    catch (catch (checkAllMI (check_for_var _A2 _B2) r)
                            (fun xa -> error (snd xa)))
                      (fun xa ->
                        error (['o'; 'n'; 'e'; ' '; 'o'; 'f'; ' '; 't'; 'h';
                                 'e'; ' '; 'l'; 'h'; 's'; 's'; ' '; 'o'; 'f';
                                 ' '; 'R'; ' '; 'i'; 's'; ' '; 'a'; ' '; 'v';
                                 'a'; 'r'; 'i'; 'a'; 'b'; 'l'; 'e']
                                :: xa))))
            (fun x ->
              catch (checkSubset (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) nSp
                      rP)
                (fun xa ->
                  error ((match check_supt_proj (_A1, _A2) (_B1, _B2) proj xa
                           with Left err -> err
                           | Right ok ->
                             [['i'; 'n'; 't'; 'e'; 'r'; 'n'; 'a'; 'l'; ' '; 'e';
                                'r'; 'r'; 'o'; 'r'];
                               ['p'; 'l'; 'e'; 'a'; 's'; 'e'; ' '; 'c'; 'o';
                                 'n'; 't'; 'a'; 'c'; 't'; ' '; 'o'; 'n'; 'e';
                                 ' '; 'o'; 'f'; ' '; 't'; 'h'; 'e'; ' '; 'C';
                                 'e'; 'T'; 'A'; ' '; 'm'; 'e'; 'm'; 'b'; 'e';
                                 'r'; 's'];
                               ['h'; 't'; 't'; 'p'; ':'; '/'; '/'; 'c'; 'l';
                                 '-'; 'i'; 'n'; 'f'; 'o'; 'r'; 'm'; 'a'; 't';
                                 'i'; 'k'; '.'; 'u'; 'i'; 'b'; 'k'; '.'; 'a';
                                 'c'; '.'; 'a'; 't'; '/'; 's'; 'o'; 'f'; 't';
                                 'w'; 'a'; 'r'; 'e'; '/'; 'c'; 'e'; 't'; 'a'];
                               ['c'; 'e'; 't'; 'a'; '@'; 'i'; 'n'; 'f'; 'o';
                                 'r'; 'm'; 'a'; 't'; 'i'; 'k'; '.'; 'u'; 'i';
                                 'b'; 'k'; '.'; 'a'; 'c'; '.'; 'a'; 't']])))))
      (fun x ->
        error (['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'a'; 'p'; 'p';
                 'l'; 'y'; ' '; 't'; 'h'; 'e'; ' '; 's'; 'u'; 'b'; 't'; 'e';
                 'r'; 'm'; ' '; 'c'; 'r'; 'i'; 't'; 'e'; 'r'; 'i'; 'o'; 'n';
                 ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r'; ' '; 'w';
                 'i'; 't'; 'h'; ' '; 't'; 'h'; 'e'; ' '; 'f'; 'o'; 'l'; 'l';
                 'o'; 'w'; 'i'; 'n'; 'g'; ' '; 's'; 'i'; 'm'; 'p'; 'l'; 'e';
                 ' '; 'p'; 'r'; 'o'; 'j'; 'e'; 'c'; 't'; 'i'; 'o'; 'n']
                :: append rep ([] :: x)));;

let rec create_sp _A
  (SimpProj sp) f = (match lookup _A f sp with None -> Zero_nat | Some n -> n);;

let rec create_spsc_proc (_A1, _A2) (_B1, _B2)
  sp p rP r =
    let pi = create_sp _A1 sp in
    check_spsc_proc (_A1, _A2) (_B1, _B2) pi (create_sp_representation _A2 sp) p
      rP r;;

let rec create_sem_lab_proc (_F1, _F2) (_L1, _L2) (_V1, _V2)
  sli l r1 r2 r3 r4 =
    let c = get_largest_element sli in
    let ca = upt Zero_nat (Suc c) in
    let i = sl_inter_to_inter _F1 _L1 sli in
    catch (check_sem_lab_proc (_F1, _F2) (_L1, _L2) (_V1, _V2)
            (check_sl_lab_trs (_F1, _F2) (_L1, _L2) (eq_nata, sshow_nata)
              (_V1, _V2) i l ca)
            (check_sl_model_lab_trs (_F1, _F2) (_L1, _L2) (eq_nata, sshow_nata)
              (_V1, _V2) i l ca)
            (fun x -> false) i l r1 r2 r3 r4)
      (fun x -> error (sshow_sl_inter _F2 _L2 sli :: x));;

let rec check_unlab_proc (_F1, _F2) (_L1, _L2) (_V1, _V2)
  p r ulP ulR =
    let ulr = unlab_of_trsI _F1 _L1 _V1 r in
    catch (bind (bind (checkSubset
                        (eqb (eq_terma (eq_laba _F1 _L1) _V1)
                          (eq_terma (eq_laba _F1 _L1) _V1))
                        (map_trsI unlab p) ulP)
                  (fun x ->
                    checkSubset
                      (eqb (eq_terma (eq_laba _F1 _L1) _V1)
                        (eq_terma (eq_laba _F1 _L1) _V1))
                      ulr ulR))
            (fun x ->
              checkSubset
                (eqb (eq_terma (eq_laba _F1 _L1) _V1)
                  (eq_terma (eq_laba _F1 _L1) _V1))
                ulR ulr))
      (fun x ->
        error [append
                 ['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'i'; 't'; 'h';
                   ' '; 'r'; 'u'; 'l'; 'e'; ' ']
                 (showRule (sshow_laba _F2 _L2) _V2 x)]);;

let rec checkUrClosedTRM (_A1, _A2) (_B1, _B2)
  uu uv x2 = match uu, uv, x2 with uu, uv, Var x -> return ()
    | rm, ur, Fun (f, ts) ->
        bind (catch
               (checkAllMI (checkUrClosedTRM (_A1, _A2) (_B1, _B2) rm ur) ts)
               (fun x -> error (snd x)))
          (fun x ->
            catch (checkAllMI
                    (fun lr ->
                      checkB
                        (member (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) lr
                           ur
                          || not (matchCapRMBelow _A1 _B1 (fst lr) (Some rm)
                                   (Fun (f, ts))))
                        (append
                          (append
                            (append
                              (append
                                ['d'; 'u'; 'e'; ' '; 't'; 'o'; ' '; 't'; 'h';
                                  'e'; ' '; 's'; 'u'; 'b'; 't'; 'e'; 'r'; 'm';
                                  ' ']
                                (sshow_term _A2 _B2 (Fun (f, ts))))
                              [' '; 'o'; 'f'; ' '; 's'; 'o'; 'm'; 'e'; ' '; 'u';
                                's'; 'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'h'; 's';
                                ','; ' '; 'r'; 'u'; 'l'; 'e'; ' '])
                            (showRule _A2 _B2 lr))
                          [' '; 's'; 'h'; 'o'; 'u'; 'l'; 'd'; ' '; 'b'; 'e';
                            ' '; 'u'; 's'; 'a'; 'b'; 'l'; 'e'; '.']))
                    (rm f))
              (fun xa -> error (snd xa)));;

let rec matchCapBelowI _A _B
  l r (Fun (f, ts)) = gcMatchI _A _B (HFun (f, map (capI _A _B r) ts)) l;;

let rec checkUrClosedT (_A1, _A2) (_B1, _B2)
  uu uv x2 = match uu, uv, x2 with uu, uv, Var x -> return ()
    | r, ur, Fun (f, ts) ->
        bind (catch (checkAllMI (checkUrClosedT (_A1, _A2) (_B1, _B2) r ur) ts)
               (fun x -> error (snd x)))
          (fun x ->
            catch (checkAllMI
                    (fun lr ->
                      checkB
                        (member (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) lr
                           ur
                          || not (matchCapBelowI _A1 _B1 (fst lr) r
                                   (Fun (f, ts))))
                        (append
                          (append
                            (append
                              (append
                                ['d'; 'u'; 'e'; ' '; 't'; 'o'; ' '; 't'; 'h';
                                  'e'; ' '; 's'; 'u'; 'b'; 't'; 'e'; 'r'; 'm';
                                  ' ']
                                (sshow_term _A2 _B2 (Fun (f, ts))))
                              [' '; 'o'; 'f'; ' '; 's'; 'o'; 'm'; 'e'; ' '; 'u';
                                's'; 'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'h'; 's';
                                ','; ' '; 'r'; 'u'; 'l'; 'e'; ' '])
                            (showRule _A2 _B2 lr))
                          [' '; 's'; 'h'; 'o'; 'u'; 'l'; 'd'; ' '; 'b'; 'e';
                            ' '; 'u'; 's'; 'a'; 'b'; 'l'; 'e'; '.']))
                    r)
              (fun xa -> error (snd xa)));;

let rec checkUrPClosed (_A1, _A2) (_B1, _B2)
  r ur p =
    bind (catch
           (catch
             (checkAllMI
               (fun lr -> checkUrClosedT (_A1, _A2) (_B1, _B2) r ur (snd lr))
               ur)
             (fun x -> error (snd x)))
           (fun x ->
             error [['e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h'; 'e'; 'n'; ' '; 'c';
                      'h'; 'e'; 'c'; 'k'; 'i'; 'n'; 'g'; ' '; 'c'; 'l'; 'o';
                      's'; 'u'; 'r'; 'e'; ' '; 'p'; 'r'; 'o'; 'p'; 'e'; 'r';
                      't'; 'i'; 'e'; 's'; ' '; 'o'; 'f'; ' '; 'r'; 'h'; 's';
                      ' '; 'o'; 'f'; ' '; 'u'; 's'; 'a'; 'b'; 'l'; 'e'; ' ';
                      'r'; 'u'; 'l'; 'e'; 's'];
                     x]))
      (fun x ->
        catch (catch
                (checkAllMI
                  (fun st -> checkUrClosedT (_A1, _A2) (_B1, _B2) r ur (snd st))
                  p)
                (fun xa -> error (snd xa)))
          (fun xa ->
            error [['e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h'; 'e'; 'n'; ' '; 'c';
                     'h'; 'e'; 'c'; 'k'; 'i'; 'n'; 'g'; ' '; 'c'; 'l'; 'o'; 's';
                     'u'; 'r'; 'e'; ' '; 'p'; 'r'; 'o'; 'p'; 'e'; 'r'; 't'; 'i';
                     'e'; 's'; ' '; 'o'; 'f'; ' '; 'r'; 'h'; 's'; ' '; 'o'; 'f';
                     ' '; 'D'; 'P'; 's'];
                    xa]));;

let rec checkUrPClosedRM (_A1, _A2) (_B1, _B2)
  r rmo ur p =
    (match rmo with None -> checkUrPClosed (_A1, _A2) (_B1, _B2) r ur p
       | Some rm ->
         bind (catch
                (catch
                  (checkAllMI
                    (fun lr ->
                      checkUrClosedTRM (_A1, _A2) (_B1, _B2) rm ur (snd lr))
                    ur)
                  (fun x -> error (snd x)))
                (fun x ->
                  error [['e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h'; 'e'; 'n'; ' ';
                           'c'; 'h'; 'e'; 'c'; 'k'; 'i'; 'n'; 'g'; ' '; 'c';
                           'l'; 'o'; 's'; 'u'; 'r'; 'e'; ' '; 'p'; 'r'; 'o';
                           'p'; 'e'; 'r'; 't'; 'i'; 'e'; 's'; ' '; 'o'; 'f';
                           ' '; 'r'; 'h'; 's'; ' '; 'o'; 'f'; ' '; 'u'; 's';
                           'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'u'; 'l'; 'e'; 's'];
                          x]))
           (fun x ->
             catch (catch
                     (checkAllMI
                       (fun st ->
                         checkUrClosedTRM (_A1, _A2) (_B1, _B2) rm ur (snd st))
                       p)
                     (fun xa -> error (snd xa)))
               (fun xa ->
                 error [['e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h'; 'e'; 'n'; ' ';
                          'c'; 'h'; 'e'; 'c'; 'k'; 'i'; 'n'; 'g'; ' '; 'c'; 'l';
                          'o'; 's'; 'u'; 'r'; 'e'; ' '; 'p'; 'r'; 'o'; 'p'; 'e';
                          'r'; 't'; 'i'; 'e'; 's'; ' '; 'o'; 'f'; ' '; 'r'; 'h';
                          's'; ' '; 'o'; 'f'; ' '; 'D'; 'P'; 's'];
                         xa])));;

let rec checkMonoRedPairUrProc (_A1, _A2) (_B1, _B2)
  checkMono checkSS checkNSS name representation p rP r rm rR ur =
    let a = partition (fun foo -> isValue (checkSS foo)) p in
    let (sp, nSp) = a in
    let aa = partition (fun foo -> isValue (checkSS foo)) ur in
    let (sr, nSr) = aa in
    catch (bind (bind (bind (bind (bind (bind
  (catch checkMono
    (fun x ->
      error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'i'; 't'; 'h'; ' ';
               'm'; 'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; 'i'; 't'; 'y'; ' ';
               'o'; 'f'; ' '; 's'; 't'; 'r'; 'i'; 'c'; 't'; ' '; 'o'; 'r'; 'd';
               'e'; 'r']
              :: x)))
  (fun x ->
    catch (checkUrPClosedRM (_A1, _A2) (_B1, _B2) r rm ur p)
      (fun xa ->
        error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'i'; 't'; 'h'; ' ';
                 'c'; 'l'; 'o'; 's'; 'u'; 'r'; 'e'; ' '; 'p'; 'r'; 'o'; 'p';
                 'e'; 'r'; 't'; 'i'; 'e'; 's'; ' '; 'o'; 'f'; ' '; 'u'; 's';
                 'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'u'; 'l'; 'e'; 's']
                :: xa))))
                                    (fun x ->
                                      catch
(catch (checkAllMI checkNSS nSr) (fun xa -> error (snd xa)))
(fun xa ->
  error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'h'; 'e'; 'n'; ' '; 'o';
           'r'; 'i'; 'e'; 'n'; 't'; 'i'; 'n'; 'g'; ' '; 'u'; 's'; 'a'; 'b'; 'l';
           'e'; ' '; 'r'; 'u'; 'l'; 'e'; 's']
          :: xa))))
                              (fun x ->
                                catch (catch (checkAllMI checkNSS nSp)
(fun xa -> error (snd xa)))
                                  (fun xa ->
                                    error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm';
     ' '; 'w'; 'h'; 'e'; 'n'; ' '; 'o'; 'r'; 'i'; 'e'; 'n'; 't'; 'i'; 'n'; 'g';
     ' '; 'D'; 'P'; 's']
    :: xa))))
                        (fun x ->
                          catch (checkSubset
                                  (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) rR
                                  r)
                            (fun xa ->
                              error [['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ':';
                                       ' '; 'a'; 'd'; 'd'; 'i'; 't'; 'i'; 'o';
                                       'n'; 'a'; 'l'; ' '; 'r'; 'u'; 'l'; 'e';
                                       '('; 's'; ')'; ' '; 't'; 'o'; ' '; 'T';
                                       'R'; 'S'; ' '; 'a'; 'd'; 'd'; 'e';
                                       'd']])))
                  (fun x ->
                    catch (checkSubset
                            (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) nSr rR)
                      (fun xa ->
                        error ((match checkSS xa with Left err -> err
                                 | Right ok ->
                                   [['i'; 'n'; 't'; 'e'; 'r'; 'n'; 'a'; 'l';
                                      ' '; 'e'; 'r'; 'r'; 'o'; 'r'];
                                     ['p'; 'l'; 'e'; 'a'; 's'; 'e'; ' '; 'c';
                                       'o'; 'n'; 't'; 'a'; 'c'; 't'; ' '; 'o';
                                       'n'; 'e'; ' '; 'o'; 'f'; ' '; 't'; 'h';
                                       'e'; ' '; 'C'; 'e'; 'T'; 'A'; ' '; 'm';
                                       'e'; 'm'; 'b'; 'e'; 'r'; 's'];
                                     ['h'; 't'; 't'; 'p'; ':'; '/'; '/'; 'c';
                                       'l'; '-'; 'i'; 'n'; 'f'; 'o'; 'r'; 'm';
                                       'a'; 't'; 'i'; 'k'; '.'; 'u'; 'i'; 'b';
                                       'k'; '.'; 'a'; 'c'; '.'; 'a'; 't'; '/';
                                       's'; 'o'; 'f'; 't'; 'w'; 'a'; 'r'; 'e';
                                       '/'; 'c'; 'e'; 't'; 'a'];
                                     ['c'; 'e'; 't'; 'a'; '@'; 'i'; 'n'; 'f';
                                       'o'; 'r'; 'm'; 'a'; 't'; 'i'; 'k'; '.';
                                       'u'; 'i'; 'b'; 'k'; '.'; 'a'; 'c'; '.';
                                       'a'; 't']])))))
            (fun x ->
              catch (checkSubset (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) nSp
                      rP)
                (fun xa ->
                  error ((match checkSS xa with Left err -> err
                           | Right ok ->
                             [['i'; 'n'; 't'; 'e'; 'r'; 'n'; 'a'; 'l'; ' '; 'e';
                                'r'; 'r'; 'o'; 'r'];
                               ['p'; 'l'; 'e'; 'a'; 's'; 'e'; ' '; 'c'; 'o';
                                 'n'; 't'; 'a'; 'c'; 't'; ' '; 'o'; 'n'; 'e';
                                 ' '; 'o'; 'f'; ' '; 't'; 'h'; 'e'; ' '; 'C';
                                 'e'; 'T'; 'A'; ' '; 'm'; 'e'; 'm'; 'b'; 'e';
                                 'r'; 's'];
                               ['h'; 't'; 't'; 'p'; ':'; '/'; '/'; 'c'; 'l';
                                 '-'; 'i'; 'n'; 'f'; 'o'; 'r'; 'm'; 'a'; 't';
                                 'i'; 'k'; '.'; 'u'; 'i'; 'b'; 'k'; '.'; 'a';
                                 'c'; '.'; 'a'; 't'; '/'; 's'; 'o'; 'f'; 't';
                                 'w'; 'a'; 'r'; 'e'; '/'; 'c'; 'e'; 't'; 'a'];
                               ['c'; 'e'; 't'; 'a'; '@'; 'i'; 'n'; 'f'; 'o';
                                 'r'; 'm'; 'a'; 't'; 'i'; 'k'; '.'; 'u'; 'i';
                                 'b'; 'k'; '.'; 'a'; 'c'; '.'; 'a'; 't']])))))
      (fun x ->
        error (['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'a'; 'p'; 'p';
                 'l'; 'y'; ' '; 't'; 'h'; 'e'; ' '; '('; 'm'; 'o'; 'n'; 'o';
                 't'; 'o'; 'n'; 'i'; 'c'; ')'; ' '; 'r'; 'e'; 'd'; 'u'; 'c';
                 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a'; 'i'; 'r'; ' '; 'p'; 'r';
                 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r'; ' '; 'w'; 'i'; 't'; 'h';
                 ' '; 't'; 'h'; 'e'; ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i';
                 'n'; 'g']
                :: append representation
                     ([] ::
                       ['a'; 'n'; 'd'; ' '; 't'; 'h'; 'e'; ' '; 'f'; 'o'; 'l';
                         'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' '; 'u'; 's'; 'a'; 'b';
                         'l'; 'e'; ' '; 'r'; 'u'; 'l'; 'e'; 's']
                         :: append (showRules _A2 _B2 ur) ([] :: x))));;

let rec create_mono_redpair_ur_proc_comp (_A1, _A2) (_B1, _B2) (_C1, _C2)
  c cm i p rP r rm rR ur =
    let pi = to_lpoly_inter _B1 (get_comp_1 c) i in
    bind (check_lpoly_coeffs _A2 _B2 (get_comp_wf_elem c) (get_comp_apos c)
           (fun x -> get_comp_ge c x (get_comp_0 c)) i)
      (fun x ->
        checkMonoRedPairUrProc (_B1, _B2) (_C1, _C2)
          (check_poly_monoI _A2 _B2 (get_comp_0 c) (get_comp_ge c) cm i)
          (check_polo_sI (_A1, _A2) _B2 (_C1, _C2) (get_comp_mono c)
            (get_comp_0 c) (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) (get_comp_gt c) pi)
          (check_polo_nsI (_A1, _A2) _B2 (_C1, _C2) (get_comp_0 c)
            (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) pi)
          (append
            (append
              ['p'; 'o'; 'l'; 'y'; 'n'; 'o'; 'm'; 'i'; 'a'; 'l'; ' '; 'o'; 'r';
                'd'; 'e'; 'r'; ' '; '(']
              (get_comp_description c))
            [')'])
          (create_lpoly_repr (_A1, _A2) (_B1, _B2) c i) p rP r rm rR ur);;

let rec create_mono_redpair_ur_proc (_A1, _A2) (_B1, _B2)
  = function
    Int_carrier ia ->
      create_mono_redpair_ur_proc_comp (eq_int, sshow_inta) (_A1, _A2)
        (_B1, _B2)
        ((fun x -> true),
          ((Big_int.big_int_of_int 0),
            ((Big_int.big_int_of_int 1),
              (Big_int.add_big_int,
                (Big_int.mult_big_int,
                  (max ord_int (Big_int.big_int_of_int 0),
                    ((fun x y -> Big_int.le_big_int y x),
                      ((fun x y -> Big_int.lt_big_int y x),
                        (true, (int_name, (fun x -> true)))))))))))
        (Big_int.lt_big_int (Big_int.big_int_of_int 0)) ia
    | Nat_carrier i ->
        create_mono_redpair_ur_proc_comp (eq_nata, sshow_nata) (_A1, _A2)
          (_B1, _B2)
          ((fun x -> true),
            (Zero_nat,
              (one_nat,
                (plus_nat,
                  (times_nat,
                    (id, ((fun x y -> less_eq_nat y x),
                           ((fun x y -> less_nat y x),
                             (true, (nat_name, (fun x -> true)))))))))))
          (fun x -> not (eqop eq_nata x Zero_nat)) i
    | Arctic_carrier id ->
        (fun uu uua uub uuc uud uue ->
          error [['a'; 'r'; 'c'; 't'; 'i'; 'c'; ' '; 'n'; 'o'; 't'; ' '; 'm';
                   'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; ' '; 'f'; 'o'; 'r';
                   ' '; 'T'; 'R'; 'S'; 's']])
    | Arc_mat_carrier (n, ic) ->
        (fun uu uua uub uuc uud uue ->
          error [['a'; 'r'; 'c'; 't'; 'i'; 'c'; ' '; 'n'; 'o'; 't'; ' '; 'm';
                   'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; ' '; 'f'; 'o'; 'r';
                   ' '; 'T'; 'R'; 'S'; 's']])
    | Int_mat_carrier (n, ib) ->
        (fun uu uua uub uuc uud uue ->
          error [['m'; 'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; ' '; 'm'; 'a';
                   't'; 'r'; 'i'; 'x'; '-'; 'o'; 'r'; 'd'; 'e'; 'r'; 's'; ' ';
                   'n'; 'o'; 't'; ' '; 'y'; 'e'; 't'; ' '; 's'; 'u'; 'p'; 'p';
                   'o'; 'r'; 't'; 'e'; 'd']]);;

let rec checkMonoRedPairProc _A _B
  checkMono checkSS checkNSS name representation p rP r rR =
    let a = partition (fun foo -> isValue (checkSS foo)) p in
    let (sp, nSp) = a in
    let aa = partition (fun foo -> isValue (checkSS foo)) r in
    let (sr, nSr) = aa in
    catch (bind (bind (bind (bind (bind (catch checkMono
  (fun x ->
    error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'i'; 't'; 'h'; ' ';
             'm'; 'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; 'i'; 't'; 'y'; ' ';
             'o'; 'f'; ' '; 's'; 't'; 'r'; 'i'; 'c'; 't'; ' '; 'o'; 'r'; 'd';
             'e'; 'r']
            :: x)))
                                    (fun x ->
                                      catch
(catch (checkAllMI checkNSS nSr) (fun xa -> error (snd xa)))
(fun xa ->
  error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w'; 'h'; 'e'; 'n'; ' '; 'o';
           'r'; 'i'; 'e'; 'n'; 't'; 'i'; 'n'; 'g'; ' '; 'T'; 'R'; 'S']
          :: xa))))
                              (fun x ->
                                catch (catch (checkAllMI checkNSS nSp)
(fun xa -> error (snd xa)))
                                  (fun xa ->
                                    error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm';
     ' '; 'w'; 'h'; 'e'; 'n'; ' '; 'o'; 'r'; 'i'; 'e'; 'n'; 't'; 'i'; 'n'; 'g';
     ' '; 'D'; 'P'; 's']
    :: xa))))
                        (fun x ->
                          catch (checkSubset
                                  (eqb (eq_terma _A _B) (eq_terma _A _B)) rR r)
                            (fun xa ->
                              error [['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ':';
                                       ' '; 'a'; 'd'; 'd'; 'i'; 't'; 'i'; 'o';
                                       'n'; 'a'; 'l'; ' ']])))
                  (fun x ->
                    catch (checkSubset (eqb (eq_terma _A _B) (eq_terma _A _B))
                            nSr rR)
                      (fun xa ->
                        error ((match checkSS xa with Left err -> err
                                 | Right ok ->
                                   [['i'; 'n'; 't'; 'e'; 'r'; 'n'; 'a'; 'l';
                                      ' '; 'e'; 'r'; 'r'; 'o'; 'r'];
                                     ['p'; 'l'; 'e'; 'a'; 's'; 'e'; ' '; 'c';
                                       'o'; 'n'; 't'; 'a'; 'c'; 't'; ' '; 'o';
                                       'n'; 'e'; ' '; 'o'; 'f'; ' '; 't'; 'h';
                                       'e'; ' '; 'C'; 'e'; 'T'; 'A'; ' '; 'm';
                                       'e'; 'm'; 'b'; 'e'; 'r'; 's'];
                                     ['h'; 't'; 't'; 'p'; ':'; '/'; '/'; 'c';
                                       'l'; '-'; 'i'; 'n'; 'f'; 'o'; 'r'; 'm';
                                       'a'; 't'; 'i'; 'k'; '.'; 'u'; 'i'; 'b';
                                       'k'; '.'; 'a'; 'c'; '.'; 'a'; 't'; '/';
                                       's'; 'o'; 'f'; 't'; 'w'; 'a'; 'r'; 'e';
                                       '/'; 'c'; 'e'; 't'; 'a'];
                                     ['c'; 'e'; 't'; 'a'; '@'; 'i'; 'n'; 'f';
                                       'o'; 'r'; 'm'; 'a'; 't'; 'i'; 'k'; '.';
                                       'u'; 'i'; 'b'; 'k'; '.'; 'a'; 'c'; '.';
                                       'a'; 't']])))))
            (fun x ->
              catch (checkSubset (eqb (eq_terma _A _B) (eq_terma _A _B)) nSp rP)
                (fun xa ->
                  error ((match checkSS xa with Left err -> err
                           | Right ok ->
                             [['i'; 'n'; 't'; 'e'; 'r'; 'n'; 'a'; 'l'; ' '; 'e';
                                'r'; 'r'; 'o'; 'r'];
                               ['p'; 'l'; 'e'; 'a'; 's'; 'e'; ' '; 'c'; 'o';
                                 'n'; 't'; 'a'; 'c'; 't'; ' '; 'o'; 'n'; 'e';
                                 ' '; 'o'; 'f'; ' '; 't'; 'h'; 'e'; ' '; 'C';
                                 'e'; 'T'; 'A'; ' '; 'm'; 'e'; 'm'; 'b'; 'e';
                                 'r'; 's'];
                               ['h'; 't'; 't'; 'p'; ':'; '/'; '/'; 'c'; 'l';
                                 '-'; 'i'; 'n'; 'f'; 'o'; 'r'; 'm'; 'a'; 't';
                                 'i'; 'k'; '.'; 'u'; 'i'; 'b'; 'k'; '.'; 'a';
                                 'c'; '.'; 'a'; 't'; '/'; 's'; 'o'; 'f'; 't';
                                 'w'; 'a'; 'r'; 'e'; '/'; 'c'; 'e'; 't'; 'a'];
                               ['c'; 'e'; 't'; 'a'; '@'; 'i'; 'n'; 'f'; 'o';
                                 'r'; 'm'; 'a'; 't'; 'i'; 'k'; '.'; 'u'; 'i';
                                 'b'; 'k'; '.'; 'a'; 'c'; '.'; 'a'; 't']])))))
      (fun x ->
        error (['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'a'; 'p'; 'p';
                 'l'; 'y'; ' '; 't'; 'h'; 'e'; ' '; 'r'; 'e'; 'd'; 'u'; 'c';
                 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a'; 'i'; 'r'; ' '; 'p'; 'r';
                 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r'; ' '; 'w'; 'i'; 't'; 'h';
                 ' '; 't'; 'h'; 'e'; ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i';
                 'n'; 'g']
                :: append representation ([] :: x)));;

let rec create_mono_redpair_proc_comp (_A1, _A2) (_B1, _B2) (_C1, _C2)
  c cm i p rP r rR =
    let pi = to_lpoly_inter _B1 (get_comp_1 c) i in
    bind (check_lpoly_coeffs _A2 _B2 (get_comp_wf_elem c) (get_comp_apos c)
           (fun x -> get_comp_ge c x (get_comp_0 c)) i)
      (fun x ->
        checkMonoRedPairProc _B1 _C1
          (check_poly_monoI _A2 _B2 (get_comp_0 c) (get_comp_ge c) cm i)
          (check_polo_sI (_A1, _A2) _B2 (_C1, _C2) (get_comp_mono c)
            (get_comp_0 c) (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) (get_comp_gt c) pi)
          (check_polo_nsI (_A1, _A2) _B2 (_C1, _C2) (get_comp_0 c)
            (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) pi)
          (append
            (append
              ['p'; 'o'; 'l'; 'y'; 'n'; 'o'; 'm'; 'i'; 'a'; 'l'; ' '; 'o'; 'r';
                'd'; 'e'; 'r'; ' '; '(']
              (get_comp_description c))
            [')'])
          (create_lpoly_repr (_A1, _A2) (_B1, _B2) c i) p rP r rR);;

let rec create_mono_redpair_proc (_A1, _A2) (_B1, _B2)
  = function
    Int_carrier ia ->
      create_mono_redpair_proc_comp (eq_int, sshow_inta) (_A1, _A2) (_B1, _B2)
        ((fun x -> true),
          ((Big_int.big_int_of_int 0),
            ((Big_int.big_int_of_int 1),
              (Big_int.add_big_int,
                (Big_int.mult_big_int,
                  (max ord_int (Big_int.big_int_of_int 0),
                    ((fun x y -> Big_int.le_big_int y x),
                      ((fun x y -> Big_int.lt_big_int y x),
                        (true, (int_name, (fun x -> true)))))))))))
        (Big_int.lt_big_int (Big_int.big_int_of_int 0)) ia
    | Nat_carrier i ->
        create_mono_redpair_proc_comp (eq_nata, sshow_nata) (_A1, _A2)
          (_B1, _B2)
          ((fun x -> true),
            (Zero_nat,
              (one_nat,
                (plus_nat,
                  (times_nat,
                    (id, ((fun x y -> less_eq_nat y x),
                           ((fun x y -> less_nat y x),
                             (true, (nat_name, (fun x -> true)))))))))))
          (fun x -> not (eqop eq_nata x Zero_nat)) i
    | Arctic_carrier id ->
        (fun uu uua uub uuc ->
          error [['a'; 'r'; 'c'; 't'; 'i'; 'c'; ' '; 'n'; 'o'; 't'; ' '; 'm';
                   'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; ' '; 'f'; 'o'; 'r';
                   ' '; 'T'; 'R'; 'S'; 's']])
    | Arc_mat_carrier (n, ic) ->
        (fun uu uua uub uuc ->
          error [['a'; 'r'; 'c'; 't'; 'i'; 'c'; ' '; 'n'; 'o'; 't'; ' '; 'm';
                   'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; ' '; 'f'; 'o'; 'r';
                   ' '; 'T'; 'R'; 'S'; 's']])
    | Int_mat_carrier (n, ib) ->
        (fun uu uua uub uuc ->
          error [['m'; 'o'; 'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; ' '; 'm'; 'a';
                   't'; 'r'; 'i'; 'x'; '-'; 'o'; 'r'; 'd'; 'e'; 'r'; 's'; ' ';
                   'n'; 'o'; 't'; ' '; 'y'; 'e'; 't'; ' '; 's'; 'u'; 'p'; 'p';
                   'o'; 'r'; 't'; 'e'; 'd']]);;

let rec checkAllMIndex
  g f xs =
    snd (foldl (fun ((n, mon) as a) x -> (g n x, bind mon (fun xa -> f n x)))
          (one_nat, return ()) xs);;

let rec checkDepGraphProc (_A1, _A2) (_B1, _B2)
  p dps rm rrm =
    bind (catch
           (checkSubset (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) p
             (flat_map snd dps))
           (fun x ->
             error [concat
                      [['D'; 'e'; 'p'; 'e'; 'n'; 'd'; 'e'; 'n'; 'c'; 'y'; ' ';
                         'P'; 'a'; 'i'; 'r'; ' '];
                        showRule _A2 _B2 x;
                        [' '; 'i'; 's'; ' '; 'm'; 'i'; 's'; 's'; 'i'; 'n'; 'g';
                          ' '; 'i'; 'n'; ' '; 'd'; 'e'; 'c'; 'o'; 'm'; 'p'; 'o';
                          's'; 'i'; 't'; 'i'; 'o'; 'n']]]))
      (fun x ->
        catch (check_graph_decomp
                (comp (sshowa (sshow_terma _A2 _B2) (sshow_terma _A2 _B2)) fst)
                (fun ((aa, b) as a) ->
                  let (ab, ba) = aa in
                  let (uu, t) = ab in
                  (fun ((uua, ct) as ac) ((ae, bb) as ad) ->
                    let (u, uub) = ae in
                    (fun ((cu, uuc) as af) ->
                      gcMatchI _A1 _B1 ct u && gcMatchI _A1 _B1 cu t)
                      bb)
                    ba
                    b)
                (map (fun ((real, cs) as a) ->
                       (real,
                         map (fun ((s, t) as aa) ->
                               ((s, t),
                                 (capRM _A1 _B1 rrm s, capRM _A1 _B1 rm t)))
                           cs))
                  dps))
          (fun xa ->
            error [['o'; 'u'; 'r'; ' '; 'e'; 's'; 't'; 'i'; 'm'; 'a'; 't'; 'i';
                     'o'; 'n'; ' '; '('; 'E'; 'D'; 'G'; '*'; '*'; '*'; ')'; ' ';
                     'c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 's'; 'h';
                     'o'; 'w'; ' '; 't'; 'h'; 'a'; 't'; ' '; 'y'; 'o'; 'u'; ' ';
                     'g'; 'a'; 'v'; 'e'; ' '; 'a'; ' '; 'v'; 'a'; 'l'; 'i'; 'd';
                     ' '; 'd'; 'e'; 'c'; 'o'; 'm'; 'p'; 'o'; 's'; 'i'; 't'; 'i';
                     'o'; 'n'; ' '; 'd'; 'u'; 'e'; ' '; 't'; 'o'; ' '; 't'; 'h';
                     'e'; ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' ';
                     'r'; 'e'; 'a'; 's'; 'o'; 'n'];
                    xa]));;

let rec checkUrClosedT_af (_A1, _A2) (_B1, _B2)
  uu uv uw x3 = match uu, uv, uw, x3 with uu, uv, uw, Var x -> return ()
    | r, ur, pi, Fun (f, ts) ->
        let n = size_list ts in
        bind (catch
               (checkAllMI
                 (fun i ->
                   (if pi f n i
                     then checkUrClosedT_af (_A1, _A2) (_B1, _B2) r ur pi
                            (nth ts i)
                     else return ()))
                 (upt Zero_nat n))
               (fun x -> error (snd x)))
          (fun x ->
            catch (checkAllMI
                    (fun lr ->
                      checkB
                        (member (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) lr
                           ur
                          || not (matchCapBelowI _A1 _B1 (fst lr) r
                                   (Fun (f, ts))))
                        (append
                          (append
                            (append
                              (append
                                ['d'; 'u'; 'e'; ' '; 't'; 'o'; ' '; 't'; 'h';
                                  'e'; ' '; 's'; 'u'; 'b'; 't'; 'e'; 'r'; 'm';
                                  ' ']
                                (sshow_term _A2 _B2 (Fun (f, ts))))
                              [' '; 'o'; 'f'; ' '; 's'; 'o'; 'm'; 'e'; ' '; 'u';
                                's'; 'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'h'; 's';
                                ','; ' '; 'r'; 'u'; 'l'; 'e'; ' '])
                            (showRule _A2 _B2 lr))
                          [' '; 's'; 'h'; 'o'; 'u'; 'l'; 'd'; ' '; 'b'; 'e';
                            ' '; 'u'; 's'; 'a'; 'b'; 'l'; 'e'; '.']))
                    r)
              (fun xa -> error (snd xa)));;

let rec checkUrPClosed_af (_A1, _A2) (_B1, _B2)
  r ur p pi =
    bind (catch
           (catch
             (checkAllMI
               (fun lr ->
                 checkUrClosedT_af (_A1, _A2) (_B1, _B2) r ur pi (snd lr))
               ur)
             (fun x -> error (snd x)))
           (fun x ->
             error [['e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h'; 'e'; 'n'; ' '; 'c';
                      'h'; 'e'; 'c'; 'k'; 'i'; 'n'; 'g'; ' '; 'c'; 'l'; 'o';
                      's'; 'u'; 'r'; 'e'; ' '; 'p'; 'r'; 'o'; 'p'; 'e'; 'r';
                      't'; 'i'; 'e'; 's'; ' '; 'o'; 'f'; ' '; 'r'; 'h'; 's';
                      ' '; 'o'; 'f'; ' '; 'u'; 's'; 'a'; 'b'; 'l'; 'e'; ' ';
                      'r'; 'u'; 'l'; 'e'; 's'];
                     x]))
      (fun x ->
        catch (catch
                (checkAllMI
                  (fun st ->
                    checkUrClosedT_af (_A1, _A2) (_B1, _B2) r ur pi (snd st))
                  p)
                (fun xa -> error (snd xa)))
          (fun xa ->
            error [['e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h'; 'e'; 'n'; ' '; 'c';
                     'h'; 'e'; 'c'; 'k'; 'i'; 'n'; 'g'; ' '; 'c'; 'l'; 'o'; 's';
                     'u'; 'r'; 'e'; ' '; 'p'; 'r'; 'o'; 'p'; 'e'; 'r'; 't'; 'i';
                     'e'; 's'; ' '; 'o'; 'f'; ' '; 'r'; 'h'; 's'; ' '; 'o'; 'f';
                     ' '; 'D'; 'P'; 's'];
                    xa]));;

let rec checkUrPClosedRM_af (_A1, _A2) (_B1, _B2)
  r rmo ur p pi =
    (match rmo with None -> checkUrPClosed_af (_A1, _A2) (_B1, _B2) r ur p pi
       | Some rm ->
         bind (catch
                (catch
                  (checkAllMI
                    (fun lr ->
                      checkUrClosedTRM_af (_A1, _A2) (_B1, _B2) rm ur pi
                        (snd lr))
                    ur)
                  (fun x -> error (snd x)))
                (fun x ->
                  error [['e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h'; 'e'; 'n'; ' ';
                           'c'; 'h'; 'e'; 'c'; 'k'; 'i'; 'n'; 'g'; ' '; 'c';
                           'l'; 'o'; 's'; 'u'; 'r'; 'e'; ' '; 'p'; 'r'; 'o';
                           'p'; 'e'; 'r'; 't'; 'i'; 'e'; 's'; ' '; 'o'; 'f';
                           ' '; 'r'; 'h'; 's'; ' '; 'o'; 'f'; ' '; 'u'; 's';
                           'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'u'; 'l'; 'e'; 's'];
                          x]))
           (fun x ->
             catch (catch
                     (checkAllMI
                       (fun st ->
                         checkUrClosedTRM_af (_A1, _A2) (_B1, _B2) rm ur pi
                           (snd st))
                       p)
                     (fun xa -> error (snd xa)))
               (fun xa ->
                 error [['e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h'; 'e'; 'n'; ' ';
                          'c'; 'h'; 'e'; 'c'; 'k'; 'i'; 'n'; 'g'; ' '; 'c'; 'l';
                          'o'; 's'; 'u'; 'r'; 'e'; ' '; 'p'; 'r'; 'o'; 'p'; 'e';
                          'r'; 't'; 'i'; 'e'; 's'; ' '; 'o'; 'f'; ' '; 'r'; 'h';
                          's'; ' '; 'o'; 'f'; ' '; 'D'; 'P'; 's'];
                         xa])));;

let rec checkRedPairUrAfProc (_A1, _A2) (_B1, _B2)
  checkSS checkNSS name representation p rP r rm ur pi =
    let a = partition (fun foo -> isValue (checkSS foo)) p in
    let (sp, nSp) = a in
    catch (bind (bind (bind (catch
                              (checkUrPClosedRM_af (_A1, _A2) (_B1, _B2) r rm ur
                                p pi)
                              (fun x ->
                                error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' ';
 'w'; 'i'; 't'; 'h'; ' '; 'c'; 'l'; 'o'; 's'; 'u'; 'r'; 'e'; ' '; 'p'; 'r'; 'o';
 'p'; 'e'; 'r'; 't'; 'i'; 'e'; 's'; ' '; 'o'; 'f'; ' '; 'u'; 's'; 'a'; 'b'; 'l';
 'e'; ' '; 'r'; 'u'; 'l'; 'e'; 's'; ' '; 'w'; '.'; 'r'; '.'; 't'; '.'; ' '; 'a';
 'r'; 'g'; 'u'; 'm'; 'e'; 'n'; 't'; ' '; 'f'; 'i'; 'l'; 't'; 'e'; 'r']
:: x)))
                        (fun x ->
                          catch (catch (checkAllMI checkNSS ur)
                                  (fun xa -> error (snd xa)))
                            (fun xa ->
                              error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' ';
                                       'w'; 'h'; 'e'; 'n'; ' '; 'o'; 'r'; 'i';
                                       'e'; 'n'; 't'; 'i'; 'n'; 'g'; ' '; 'u';
                                       's'; 'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'u';
                                       'l'; 'e'; 's']
                                      :: xa))))
                  (fun x ->
                    catch (catch (checkAllMI checkNSS nSp)
                            (fun xa -> error (snd xa)))
                      (fun xa ->
                        error (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' '; 'w';
                                 'h'; 'e'; 'n'; ' '; 'o'; 'r'; 'i'; 'e'; 'n';
                                 't'; 'i'; 'n'; 'g'; ' '; 'D'; 'P'; 's']
                                :: xa))))
            (fun x ->
              catch (checkSubset (eqb (eq_terma _A1 _B1) (eq_terma _A1 _B1)) nSp
                      rP)
                (fun xa ->
                  error ((match checkSS xa with Left err -> err
                           | Right ok ->
                             [['i'; 'n'; 't'; 'e'; 'r'; 'n'; 'a'; 'l'; ' '; 'e';
                                'r'; 'r'; 'o'; 'r'];
                               ['p'; 'l'; 'e'; 'a'; 's'; 'e'; ' '; 'c'; 'o';
                                 'n'; 't'; 'a'; 'c'; 't'; ' '; 'o'; 'n'; 'e';
                                 ' '; 'o'; 'f'; ' '; 't'; 'h'; 'e'; ' '; 'C';
                                 'e'; 'T'; 'A'; ' '; 'm'; 'e'; 'm'; 'b'; 'e';
                                 'r'; 's'];
                               ['h'; 't'; 't'; 'p'; ':'; '/'; '/'; 'c'; 'l';
                                 '-'; 'i'; 'n'; 'f'; 'o'; 'r'; 'm'; 'a'; 't';
                                 'i'; 'k'; '.'; 'u'; 'i'; 'b'; 'k'; '.'; 'a';
                                 'c'; '.'; 'a'; 't'; '/'; 's'; 'o'; 'f'; 't';
                                 'w'; 'a'; 'r'; 'e'; '/'; 'c'; 'e'; 't'; 'a'];
                               ['c'; 'e'; 't'; 'a'; '@'; 'i'; 'n'; 'f'; 'o';
                                 'r'; 'm'; 'a'; 't'; 'i'; 'k'; '.'; 'u'; 'i';
                                 'b'; 'k'; '.'; 'a'; 'c'; '.'; 'a'; 't']])))))
      (fun x ->
        error (['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'a'; 'p'; 'p';
                 'l'; 'y'; ' '; 't'; 'h'; 'e'; ' '; 'r'; 'e'; 'd'; 'u'; 'c';
                 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a'; 'i'; 'r'; ' '; 'p'; 'r';
                 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r'; ' '; 'w'; 'i'; 't'; 'h';
                 ' '; 't'; 'h'; 'e'; ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i';
                 'n'; 'g']
                :: append representation
                     ([] ::
                       ['a'; 'n'; 'd'; ' '; 't'; 'h'; 'e'; ' '; 'f'; 'o'; 'l';
                         'l'; 'o'; 'w'; 'i'; 'n'; 'g'; ' '; 'u'; 's'; 'a'; 'b';
                         'l'; 'e'; ' '; 'r'; 'u'; 'l'; 'e'; 's']
                         :: append (showRules _A2 _B2 ur) ([] :: x))));;

let rec create_redpair_ur_af_proc_comp (_A1, _A2) (_B1, _B2) (_C1, _C2)
  c i p rP r rm ur =
    let pi = to_lpoly_inter _B1 (get_comp_1 c) i in
    bind (check_lpoly_coeffs _A2 _B2 (get_comp_wf_elem c) (get_comp_apos c)
           (fun x -> get_comp_ge c x (get_comp_0 c)) i)
      (fun x ->
        checkRedPairUrAfProc (_B1, _B2) (_C1, _C2)
          (check_polo_sI (_A1, _A2) _B2 (_C1, _C2) (get_comp_mono c)
            (get_comp_0 c) (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) (get_comp_gt c) pi)
          (check_polo_nsI (_A1, _A2) _B2 (_C1, _C2) (get_comp_0 c)
            (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) pi)
          (append
            (append
              ['p'; 'o'; 'l'; 'y'; 'n'; 'o'; 'm'; 'i'; 'a'; 'l'; ' '; 'o'; 'r';
                'd'; 'e'; 'r'; ' '; '(']
              (get_comp_description c))
            [')'])
          (create_lpoly_repr (_A1, _A2) (_B1, _B2) c i) p rP r rm ur
          (create_afI _A1 _B1 (get_comp_0 c) i));;

let rec create_redpair_ur_af_proc (_A1, _A2) (_B1, _B2)
  = function
    Int_carrier ia ->
      create_redpair_ur_af_proc_comp (eq_int, sshow_inta) (_A1, _A2) (_B1, _B2)
        ((fun x -> true),
          ((Big_int.big_int_of_int 0),
            ((Big_int.big_int_of_int 1),
              (Big_int.add_big_int,
                (Big_int.mult_big_int,
                  (max ord_int (Big_int.big_int_of_int 0),
                    ((fun x y -> Big_int.le_big_int y x),
                      ((fun x y -> Big_int.lt_big_int y x),
                        (true, (int_name, (fun x -> true)))))))))))
        ia
    | Nat_carrier i ->
        create_redpair_ur_af_proc_comp (eq_nata, sshow_nata) (_A1, _A2)
          (_B1, _B2)
          ((fun x -> true),
            (Zero_nat,
              (one_nat,
                (plus_nat,
                  (times_nat,
                    (id, ((fun x y -> less_eq_nat y x),
                           ((fun x y -> less_nat y x),
                             (true, (nat_name, (fun x -> true)))))))))))
          i
    | Arctic_carrier ida ->
        create_redpair_ur_af_proc_comp (eq_arctica, sshow_arctica) (_A1, _A2)
          (_B1, _B2)
          ((fun x -> true),
            (MinInfty,
              (Num_arc (Big_int.big_int_of_int 0),
                (plus_arctic,
                  (mult_arctic,
                    (id, (geq_arctic,
                           (gt_arctic,
                             (false, (arctic_name, pos_arctic))))))))))
          ida
    | Int_mat_carrier (n, ib) ->
        create_redpair_ur_af_proc_comp
          (eq_lista (eq_lista eq_int), sshow_lista (sshow_lista sshow_inta))
          (_A1, _A2) (_B1, _B2)
          (wf_mat_rcI (Suc n) (Suc n),
            (mat0I (Big_int.big_int_of_int 0) (Suc n) (Suc n),
              (mat1I (Big_int.big_int_of_int 0) (Big_int.big_int_of_int 1)
                 (Suc n),
                (mat_plusI Big_int.add_big_int,
                  (mat_mulI (Big_int.big_int_of_int 0) Big_int.add_big_int
                     Big_int.mult_big_int (Suc n),
                    (mat_max0I (max ord_int (Big_int.big_int_of_int 0)),
                      (mat_geI (fun x y -> Big_int.le_big_int y x),
                        (mat_gtI (fun x y -> Big_int.le_big_int y x)
                           (fun x y -> Big_int.lt_big_int y x),
                          (true,
                            (int_mat_name,
                              mat_arc_posI (fun x -> true)))))))))))
          ib
    | Arc_mat_carrier (n, ic) ->
        create_redpair_ur_af_proc_comp
          (eq_lista (eq_lista eq_arctica),
            sshow_lista (sshow_lista sshow_arctica))
          (_A1, _A2) (_B1, _B2)
          (wf_mat_rcI (Suc n) (Suc n),
            (mat0I MinInfty (Suc n) (Suc n),
              (mat1I MinInfty (Num_arc (Big_int.big_int_of_int 0)) (Suc n),
                (mat_plusI plus_arctic,
                  (mat_mulI MinInfty plus_arctic mult_arctic (Suc n),
                    (mat_max0I id,
                      (mat_geI geq_arctic,
                        (mat_geI gt_arctic,
                          (false,
                            (arc_mat_name, mat_arc_posI pos_arctic))))))))))
          ic;;

let rec create_redpair_proc_comp (_A1, _A2) (_B1, _B2) (_C1, _C2)
  c i p rP r =
    let pi = to_lpoly_inter _B1 (get_comp_1 c) i in
    bind (check_lpoly_coeffs _A2 _B2 (get_comp_wf_elem c) (get_comp_apos c)
           (fun x -> get_comp_ge c x (get_comp_0 c)) i)
      (fun x ->
        checkRedPairProc _B1 _C1
          (check_polo_sI (_A1, _A2) _B2 (_C1, _C2) (get_comp_mono c)
            (get_comp_0 c) (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) (get_comp_gt c) pi)
          (check_polo_nsI (_A1, _A2) _B2 (_C1, _C2) (get_comp_0 c)
            (get_comp_1 c) (get_comp_plus c) (get_comp_times c)
            (get_comp_max0 c) (get_comp_ge c) pi)
          (append
            (append
              ['p'; 'o'; 'l'; 'y'; 'n'; 'o'; 'm'; 'i'; 'a'; 'l'; ' '; 'o'; 'r';
                'd'; 'e'; 'r'; ' '; '(']
              (get_comp_description c))
            [')'])
          (create_lpoly_repr (_A1, _A2) (_B1, _B2) c i) p rP r);;

let rec create_redpair_proc (_A1, _A2) (_B1, _B2)
  = function
    Int_carrier ia ->
      create_redpair_proc_comp (eq_int, sshow_inta) (_A1, _A2) (_B1, _B2)
        ((fun x -> true),
          ((Big_int.big_int_of_int 0),
            ((Big_int.big_int_of_int 1),
              (Big_int.add_big_int,
                (Big_int.mult_big_int,
                  (max ord_int (Big_int.big_int_of_int 0),
                    ((fun x y -> Big_int.le_big_int y x),
                      ((fun x y -> Big_int.lt_big_int y x),
                        (true, (int_name, (fun x -> true)))))))))))
        ia
    | Nat_carrier i ->
        create_redpair_proc_comp (eq_nata, sshow_nata) (_A1, _A2) (_B1, _B2)
          ((fun x -> true),
            (Zero_nat,
              (one_nat,
                (plus_nat,
                  (times_nat,
                    (id, ((fun x y -> less_eq_nat y x),
                           ((fun x y -> less_nat y x),
                             (true, (nat_name, (fun x -> true)))))))))))
          i
    | Arctic_carrier ida ->
        create_redpair_proc_comp (eq_arctica, sshow_arctica) (_A1, _A2)
          (_B1, _B2)
          ((fun x -> true),
            (MinInfty,
              (Num_arc (Big_int.big_int_of_int 0),
                (plus_arctic,
                  (mult_arctic,
                    (id, (geq_arctic,
                           (gt_arctic,
                             (false, (arctic_name, pos_arctic))))))))))
          ida
    | Int_mat_carrier (n, ib) ->
        create_redpair_proc_comp
          (eq_lista (eq_lista eq_int), sshow_lista (sshow_lista sshow_inta))
          (_A1, _A2) (_B1, _B2)
          (wf_mat_rcI (Suc n) (Suc n),
            (mat0I (Big_int.big_int_of_int 0) (Suc n) (Suc n),
              (mat1I (Big_int.big_int_of_int 0) (Big_int.big_int_of_int 1)
                 (Suc n),
                (mat_plusI Big_int.add_big_int,
                  (mat_mulI (Big_int.big_int_of_int 0) Big_int.add_big_int
                     Big_int.mult_big_int (Suc n),
                    (mat_max0I (max ord_int (Big_int.big_int_of_int 0)),
                      (mat_geI (fun x y -> Big_int.le_big_int y x),
                        (mat_gtI (fun x y -> Big_int.le_big_int y x)
                           (fun x y -> Big_int.lt_big_int y x),
                          (true,
                            (int_mat_name,
                              mat_arc_posI (fun x -> true)))))))))))
          ib
    | Arc_mat_carrier (n, ic) ->
        create_redpair_proc_comp
          (eq_lista (eq_lista eq_arctica),
            sshow_lista (sshow_lista sshow_arctica))
          (_A1, _A2) (_B1, _B2)
          (wf_mat_rcI (Suc n) (Suc n),
            (mat0I MinInfty (Suc n) (Suc n),
              (mat1I MinInfty (Num_arc (Big_int.big_int_of_int 0)) (Suc n),
                (mat_plusI plus_arctic,
                  (mat_mulI MinInfty plus_arctic mult_arctic (Suc n),
                    (mat_max0I id,
                      (mat_geI geq_arctic,
                        (mat_geI gt_arctic,
                          (false,
                            (arc_mat_name, mat_arc_posI pos_arctic))))))))))
          ic;;

let rec checkDPProof (_A1, _A2) (_B1, _B2)
  i p r m rm rrm x6 = match i, p, r, m, rm, rrm, x6 with
    i, p, r, m, rm, rrm, PisEmpty ->
      debug i ['P'; ' '; 'i'; 's'; ' '; 'e'; 'm'; 'p'; 't'; 'y']
        ((match p with [] -> return ()
           | x :: xa ->
             error (append i
                      [':'; ' '; 'P'; ' '; 'i'; 's'; ' '; 'n'; 'o'; 't'; ' ';
                        'e'; 'm'; 'p'; 't'; 'y'; ' '; 'i'; 'n'; ' '; 't'; 'h';
                        'e'; ' '; 'f'; 'o'; 'l'; 'l'; 'o'; 'w'; 'i'; 'n'; 'g';
                        ' '; 'D'; 'P'; '-'; 'p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm';
                        ' ']
                     :: showDPP (sshow_laba _A2 _B2) (sshow_lista sshow_chara) p
                          r)))
    | i, p, r, m, rm, rrm, RedPairProc (redp, rP, prf) ->
        debug i ['R'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'P'; 'r'; 'o'; 'c']
          (bind (catch
                  (create_redpair_proc (eq_laba _A1 _B1, sshow_laba _A2 _B2)
                    (eq_lista eq_char, sshow_lista sshow_chara) redp p rP r)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h';
                               'e'; 'n'; ' '; 'a'; 'p'; 'p'; 'l'; 'y'; 'i'; 'n';
                               'g'; ' '; 't'; 'h'; 'e'; ' '; 'r'; 'e'; 'd'; 'u';
                               'c'; 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a'; 'i'; 'r';
                               ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r';
                               ' '; 't'; 'o'; ' '; 's'; 'w'; 'i'; 't'; 'c'; 'h';
                               ' '; 'f'; 'r'; 'o'; 'm'; ' ']
                            :: append
                                 (showDPP (sshow_laba _A2 _B2)
                                   (sshow_lista sshow_chara) p r)
                                 (['t'; 'o'; ' '; 't'; 'h'; 'e'; ' '; 'D'; 'P';
                                    ' '; 'p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' ']
                                   :: append
(showDPP (sshow_laba _A2 _B2) (sshow_lista sshow_chara) rP r) x))))
            (fun x ->
              catch (checkDPProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1']) rP
                      r m rm rrm prf)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 't'; 'h'; 'e'; ' '; 'r'; 'e';
                             'd'; 'u'; 'c'; 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a';
                             'i'; 'r'; ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's';
                             'o'; 'r']
                          :: indent xa))))
    | i, p, r, m, rm, rrm, RedPairUrProc (redp, rP, ur, prf) ->
        debug i
          ['R'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'P'; 'r'; 'o'; 'c'; ' '; '('; 'u';
            's'; 'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'u'; 'l'; 'e'; 's'; ' '; '+';
            ' '; 'a'; 'f'; ')']
          (bind (catch
                  (create_redpair_ur_af_proc
                    (eq_laba _A1 _B1, sshow_laba _A2 _B2)
                    (eq_lista eq_char, sshow_lista sshow_chara) redp p rP r rm
                    ur)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h';
                               'e'; 'n'; ' '; 'a'; 'p'; 'p'; 'l'; 'y'; 'i'; 'n';
                               'g'; ' '; 't'; 'h'; 'e'; ' '; 'r'; 'e'; 'd'; 'u';
                               'c'; 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a'; 'i'; 'r';
                               ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r';
                               ' '; '('; 'w'; 'i'; 't'; 'h'; ' '; 'u'; 's'; 'a';
                               'b'; 'l'; 'e'; ' '; 'r'; 'u'; 'l'; 'e'; 's'; ' ';
                               'a'; 'n'; 'd'; ' '; 'a'; 'f'; ')'; ' '; 't'; 'o';
                               ' '; 's'; 'w'; 'i'; 't'; 'c'; 'h'; ' '; 'f'; 'r';
                               'o'; 'm'; ' ']
                            :: append
                                 (showDPP (sshow_laba _A2 _B2)
                                   (sshow_lista sshow_chara) p r)
                                 (['t'; 'o'; ' '; 't'; 'h'; 'e'; ' '; 'D'; 'P';
                                    ' '; 'p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' ']
                                   :: append
(showDPP (sshow_laba _A2 _B2) (sshow_lista sshow_chara) rP r) x))))
            (fun x ->
              catch (checkDPProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1']) rP
                      r m rm rrm prf)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 't'; 'h'; 'e'; ' '; 'r'; 'e';
                             'd'; 'u'; 'c'; 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a';
                             'i'; 'r'; ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's';
                             'o'; 'r']
                          :: indent xa))))
    | i, p, r, m, rm, rrm, DepGraphProc edpts ->
        debug i ['D'; 'e'; 'p'; 'G'; 'r'; 'a'; 'p'; 'h'; 'P'; 'r'; 'o'; 'c']
          (bind (catch
                  (checkDepGraphProc (eq_laba _A1 _B1, sshow_laba _A2 _B2)
                    (eq_lista eq_char, sshow_lista sshow_chara) p
                    (map (fun x ->
                           ((match fst x with None -> false | Some xa -> true),
                             snd x))
                      edpts)
                    rm rrm)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h';
                               'i'; 'l'; 'e'; ' '; 't'; 'r'; 'y'; 'i'; 'n'; 'g';
                               ' '; 't'; 'o'; ' '; 'p'; 'e'; 'r'; 'f'; 'o'; 'r';
                               'm'; ' '; 'S'; 'C'; 'C'; '-'; 'd'; 'e'; 'c'; 'o';
                               'm'; 'p'; 'o'; 's'; 'i'; 't'; 'i'; 'o'; 'n'; ' ';
                               ' '; 'o'; 'n'; ' ']
                            :: append
                                 (showDPP (sshow_laba _A2 _B2)
                                   (sshow_lista sshow_chara) p r)
                                 x)))
            (fun x ->
              catch (checkAllMIndex
                      (fun j rp ->
                        (match fst rp with None -> j | Some prf -> Suc j))
                      (fun j rp ->
                        (match fst rp with None -> return ()
                           | Some a ->
                             checkDPProof (_A1, _A2) (_B1, _B2)
                               (append (append i ['.']) (sshow_nat j)) (snd rp)
                               r m rm rrm a))
                      edpts)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 't'; 'h'; 'e'; ' '; 'd'; 'e';
                             'p'; 'e'; 'n'; 'd'; 'e'; 'n'; 'c'; 'y'; ' '; 'g';
                             'r'; 'a'; 'p'; 'h'; ' '; 'p'; 'r'; 'o'; 'c'; 'e';
                             's'; 's'; 'o'; 'r']
                          :: indent xa))))
    | i, p, r, m, rm, rrm, MonoRedPairProc (redp, rP, rR, prf) ->
        debug i
          ['M'; 'o'; 'n'; 'o'; 'R'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'P'; 'r'; 'o';
            'c']
          (bind (catch
                  (create_mono_redpair_proc
                    (eq_laba _A1 _B1, sshow_laba _A2 _B2)
                    (eq_lista eq_char, sshow_lista sshow_chara) redp p rP r rR)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h';
                               'e'; 'n'; ' '; 'a'; 'p'; 'p'; 'l'; 'y'; 'i'; 'n';
                               'g'; ' '; 't'; 'h'; 'e'; ' '; 'm'; 'o'; 'n'; 'o';
                               't'; 'o'; 'n'; 'i'; 'c'; ' '; 'r'; 'e'; 'd'; 'u';
                               'c'; 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a'; 'i'; 'r';
                               ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r';
                               ' '; 't'; 'o'; ' '; 's'; 'w'; 'i'; 't'; 'c'; 'h';
                               ' '; 'f'; 'r'; 'o'; 'm'; ' ']
                            :: append
                                 (showDPP (sshow_laba _A2 _B2)
                                   (sshow_lista sshow_chara) p r)
                                 (['t'; 'o'; ' '; 't'; 'h'; 'e'; ' '; 'D'; 'P';
                                    ' '; 'p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' ']
                                   :: append
(showDPP (sshow_laba _A2 _B2) (sshow_lista sshow_chara) rP rR) x))))
            (fun x ->
              catch (checkDPProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1']) rP
                      rR m (computeRuleMap (eq_laba _A1 _B1) rR)
                      (computeRuleMap (eq_laba _A1 _B1) (reverse_trs rR)) prf)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 't'; 'h'; 'e'; ' '; 'm'; 'o';
                             'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; ' '; 'r'; 'e';
                             'd'; 'u'; 'c'; 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a';
                             'i'; 'r'; ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's';
                             'o'; 'r']
                          :: indent xa))))
    | i, p, r, m, rm, rrm, MonoRedPairUrProc (redp, rP, rR, ur, prf) ->
        debug i
          ['M'; 'o'; 'n'; 'o'; 'R'; 'e'; 'd'; 'P'; 'a'; 'i'; 'r'; 'P'; 'r'; 'o';
            'c'; ' '; '('; 'u'; 's'; 'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'u'; 'l';
            'e'; 's'; ')']
          (bind (catch
                  (create_mono_redpair_ur_proc
                    (eq_laba _A1 _B1, sshow_laba _A2 _B2)
                    (eq_lista eq_char, sshow_lista sshow_chara) redp p rP r rm
                    rR ur)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h';
                               'e'; 'n'; ' '; 'a'; 'p'; 'p'; 'l'; 'y'; 'i'; 'n';
                               'g'; ' '; 't'; 'h'; 'e'; ' '; 'm'; 'o'; 'n'; 'o';
                               't'; 'o'; 'n'; 'i'; 'c'; ' '; 'r'; 'e'; 'd'; 'u';
                               'c'; 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a'; 'i'; 'r';
                               ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r';
                               ' '; '('; 'w'; 'i'; 't'; 'h'; ' '; 'u'; 's'; 'a';
                               'b'; 'l'; 'e'; ' '; 'r'; 'u'; 'l'; 'e'; 's'; ')';
                               ' '; 't'; 'o'; ' '; 's'; 'w'; 'i'; 't'; 'c'; 'h';
                               ' '; 'f'; 'r'; 'o'; 'm'; ' ']
                            :: append
                                 (showDPP (sshow_laba _A2 _B2)
                                   (sshow_lista sshow_chara) p r)
                                 (['t'; 'o'; ' '; 't'; 'h'; 'e'; ' '; 'D'; 'P';
                                    ' '; 'p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; ' ']
                                   :: append
(showDPP (sshow_laba _A2 _B2) (sshow_lista sshow_chara) rP rR) x))))
            (fun x ->
              catch (checkDPProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1']) rP
                      rR m (computeRuleMap (eq_laba _A1 _B1) rR)
                      (computeRuleMap (eq_laba _A1 _B1) (reverse_trs rR)) prf)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 't'; 'h'; 'e'; ' '; 'm'; 'o';
                             'n'; 'o'; 't'; 'o'; 'n'; 'i'; 'c'; ' '; 'r'; 'e';
                             'd'; 'u'; 'c'; 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'a';
                             'i'; 'r'; ' '; '('; 'w'; 'i'; 't'; 'h'; ' '; 'u';
                             's'; 'a'; 'b'; 'l'; 'e'; ' '; 'r'; 'u'; 'l'; 'e';
                             's'; ')'; ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's';
                             'o'; 'r']
                          :: indent xa))))
    | i, p, r, Suc m, rm, rrm, UnlabProc (ulP, ulR, prf) ->
        debug i ['U'; 'n'; 'l'; 'a'; 'b'; 'P'; 'r'; 'o'; 'c']
          (bind (check_unlab_proc (_A1, _A2) (_B1, _B2)
                  (eq_lista eq_char, sshow_lista sshow_chara) p r ulP ulR)
            (fun x ->
              catch (checkDPProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1'])
                      ulP ulR m (computeRuleMap (eq_laba _A1 _B1) ulR)
                      (computeRuleMap (eq_laba _A1 _B1) (reverse_trs ulR)) prf)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 't'; 'h'; 'e'; ' '; 'u'; 'n';
                             'l'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n'; 'g'; ' '; 'p';
                             'r'; 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r']
                          :: indent xa))))
    | i, p, r, Zero_nat, rm, rrm, UnlabProc (ulP, ulR, prf) ->
        error [['u'; 'n'; 'l'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n'; 'g'; ' '; 'n'; 'o';
                 't'; ' '; 'a'; 'l'; 'l'; 'o'; 'w'; 'e'; 'd'; ' '; 'd'; 'u';
                 'e'; ' '; 't'; 'o'; ' '; 'l'; 'a'; 'b'; 'e'; 'l'; '-'; 'd';
                 'e'; 'p'; 't'; 'h'; ' '; 'o'; 'f'; ' '; '0']]
    | i, p, r, m, rm, rrm, SemlabProc (sli, l, lP, lR, prf) ->
        debug i ['S'; 'e'; 'm'; 'l'; 'a'; 'b'; 'P'; 'r'; 'o'; 'c']
          (bind (catch
                  (create_sem_lab_proc (_A1, _A2) (_B1, _B2)
                    (eq_lista eq_char, sshow_lista sshow_chara) sli l p r lP lR)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h';
                               'e'; 'n'; ' '; 'a'; 'p'; 'p'; 'l'; 'y'; 'i'; 'n';
                               'g'; ' '; 't'; 'h'; 'e'; ' '; 's'; 'e'; 'm'; 'a';
                               'n'; 't'; 'i'; 'c'; ' '; 'l'; 'a'; 'b'; 'e'; 'l';
                               'i'; 'n'; 'g'; ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's';
                               's'; 'o'; 'r']
                            :: x)))
            (fun x ->
              catch (checkDPProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1']) lP
                      lR (Suc m) (computeRuleMap (eq_laba _A1 _B1) lR)
                      (computeRuleMap (eq_laba _A1 _B1) (reverse_trs lR)) prf)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 't'; 'h'; 'e'; ' '; 's'; 'e';
                             'm'; 'a'; 'n'; 't'; 'i'; 'c'; ' '; 'l'; 'a'; 'b';
                             'e'; 'l'; 'i'; 'n'; 'g'; ' '; 'p'; 'r'; 'o'; 'c';
                             'e'; 's'; 's'; 'o'; 'r']
                          :: indent xa))))
    | i, p, r, m, rm, rrm, RootlabProc (lP, lR, prf) ->
        debug i ['R'; 'o'; 'o'; 't'; 'l'; 'a'; 'b'; 'P'; 'r'; 'o'; 'c']
          (bind (catch (rl_proc (_A1, _A2) (_B1, _B2) p r lP lR)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h';
                               'e'; 'n'; ' '; 'a'; 'p'; 'p'; 'l'; 'y'; 'i'; 'n';
                               'g'; ' '; 't'; 'h'; 'e'; ' '; 'r'; 'o'; 'o'; 't';
                               ' '; 'l'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n'; 'g'; ' ';
                               'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r']
                            :: x)))
            (fun x ->
              catch (checkDPProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1']) lP
                      lR (Suc m) (computeRuleMap (eq_laba _A1 _B1) lR)
                      (computeRuleMap (eq_laba _A1 _B1) (reverse_trs lR)) prf)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 't'; 'h'; 'e'; ' '; 'r'; 'o';
                             'o'; 't'; '-'; 'l'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n';
                             'g'; ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's'; 'o';
                             'r']
                          :: indent xa))))
    | i, p, r, m, rm, rrm, SpscProc (sp, rP, prf) ->
        debug i ['S'; 'u'; 'b'; 't'; 'e'; 'r'; 'm'; 'P'; 'r'; 'o'; 'c']
          (bind (catch
                  (create_spsc_proc (eq_laba _A1 _B1, sshow_laba _A2 _B2)
                    (eq_lista eq_char, sshow_lista sshow_chara) sp p rP r)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h';
                               'e'; 'n'; ' '; 'a'; 'p'; 'p'; 'l'; 'y'; 'i'; 'n';
                               'g'; ' '; 't'; 'h'; 'e'; ' '; 's'; 'u'; 'b'; 't';
                               'e'; 'r'; 'm'; ' '; 'c'; 'r'; 'i'; 't'; 'e'; 'r';
                               'i'; 'o'; 'n'; ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's';
                               's'; 'o'; 'r']
                            :: x)))
            (fun x ->
              catch (checkDPProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1']) rP
                      r m rm rrm prf)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 't'; 'h'; 'e'; ' '; 's'; 'u';
                             'b'; 't'; 'e'; 'r'; 'm'; ' '; 'c'; 'r'; 'i'; 't';
                             'e'; 'r'; 'i'; 'o'; 'n'; ' '; 'p'; 'r'; 'o'; 'c';
                             'e'; 's'; 's'; 'o'; 'r']
                          :: indent xa))))
    | i, p, r, m, rm, rrm, SizeChangeSubTerm graphs ->
        debug i
          ['S'; 'i'; 'z'; 'e'; 'C'; 'h'; 'a'; 'n'; 'g'; 'e'; 'S'; 'u'; 'b'; 'T';
            'e'; 'r'; 'm']
          (check_sct_subterm_proc (eq_laba _A1 _B1, sshow_laba _A2 _B2)
            (eq_lista eq_char, sshow_lista sshow_chara) p graphs r rm rrm)
    | i, p, r, m, rm, rrm, SizeChangeRedPair (redp, ur, graphs) ->
        debug i
          ['S'; 'i'; 'z'; 'e'; 'C'; 'h'; 'a'; 'n'; 'g'; 'e'; 'R'; 'e'; 'd'; 'P';
            'a'; 'i'; 'r']
          (create_sct_ur_af_proc (eq_laba _A1 _B1, sshow_laba _A2 _B2)
            (eq_lista eq_char, sshow_lista sshow_chara) redp p r rm rrm ur
            graphs);;

let rec checkTRSProof (_A1, _A2) (_B1, _B2)
  i r m x3 = match i, r, m, x3 with
    i, r, m, DPTrans (p, prf) ->
      debug i ['D'; 'P'; ' '; 't'; 'r'; 'a'; 'n'; 's']
        (bind (catch
                (checkDPs (eq_laba _A1 _B1, sshow_laba _A2 _B2)
                  (eq_lista eq_char, sshow_lista sshow_chara) r p)
                (fun x ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h';
                             'e'; 'n'; ' '; 's'; 'w'; 'i'; 't'; 'c'; 'h'; 'i';
                             'n'; 'g'; ' '; 'f'; 'r'; 'o'; 'm'; ' '; 't'; 'h';
                             'e'; ' '; 'T'; 'R'; 'S']
                          :: append
                               (showTRS (sshow_laba _A2 _B2)
                                 (sshow_lista sshow_chara) r)
                               (['t'; 'o'; ' '; 't'; 'h'; 'e'; ' '; 'i'; 'n';
                                  'i'; 't'; 'i'; 'a'; 'l'; ' '; 'D'; 'P'; ' ';
                                  'p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm']
                                 :: append
                                      (showDPP (sshow_shpa (sshow_laba _A2 _B2))
(sshow_lista sshow_chara) p (computeID r))
                                      x))))
          (fun x ->
            catch (checkDPProof
                    (eq_shpa (eq_laba _A1 _B1), sshow_shpa (sshow_laba _A2 _B2))
                    (_B1, _B2) (append i ['.'; '1']) (map (map_rule init_lab) p)
                    (map (map_rule init_lab) (computeID r)) Zero_nat
                    (computeRuleMap (eq_laba (eq_shpa (eq_laba _A1 _B1)) _B1)
                      (map (map_rule init_lab) (computeID r)))
                    (computeRuleMap (eq_laba (eq_shpa (eq_laba _A1 _B1)) _B1)
                      (reverse_trs (map (map_rule init_lab) (computeID r))))
                    prf)
              (fun xa ->
                error (append i
                         [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e'; 'l';
                           'o'; 'w'; ' '; 's'; 'w'; 'i'; 't'; 'c'; 'h'; ' ';
                           't'; 'o'; ' '; 'd'; 'e'; 'p'; 'e'; 'n'; 'd'; 'e';
                           'n'; 'c'; 'y'; ' '; 'p'; 'a'; 'i'; 'r'; 's']
                        :: indent xa))))
    | i, r, m, RuleRemoval (rp, rR, prfa) ->
        debug i ['r'; 'u'; 'l'; 'e'; ' '; 'r'; 'e'; 'm'; 'o'; 'v'; 'a'; 'l']
          (bind (catch
                  (create_rule_removal_proc
                    (eq_laba _A1 _B1, sshow_laba _A2 _B2)
                    (eq_lista eq_char, sshow_lista sshow_chara) rp r rR)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'i'; 'n';
                               ' '; 'r'; 'u'; 'l'; 'e'; ' '; 'r'; 'e'; 'm'; 'o';
                               'v'; 'a'; 'l'; ' '; 'w'; 'h'; 'e'; 'n'; ' '; 's';
                               'w'; 'i'; 't'; 'c'; 'h'; 'i'; 'n'; 'g'; ' '; 'f';
                               'r'; 'o'; 'm'; ' '; 't'; 'h'; 'e'; ' '; 'T'; 'R';
                               'S']
                            :: append
                                 (showTRS (sshow_laba _A2 _B2)
                                   (sshow_lista sshow_chara) r)
                                 (['t'; 'o'; ' '; 't'; 'h'; 'e'; ' '; 'T'; 'R';
                                    'S']
                                   :: append
(showTRS (sshow_laba _A2 _B2) (sshow_lista sshow_chara) rR) x))))
            (fun x ->
              catch (checkTRSProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1'])
                      rR m prfa)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 't'; 'h'; 'e'; ' '; 'r'; 'u';
                             'l'; 'e'; ' '; 'r'; 'e'; 'm'; 'o'; 'v'; 'a'; 'l';
                             ' '; 'p'; 'r'; 'o'; 'c'; 'e'; 's'; 's'; 'o'; 'r']
                          :: indent xa))))
    | i, r, Suc m, Unlab (uR, prfa) ->
        debug i ['u'; 'n'; 'l'; 'a'; 'b'; 'e'; 'l']
          (bind (catch
                  (unlab_trs_tt (_A1, _A2) (_B1, _B2)
                    (eq_lista eq_char, sshow_lista sshow_chara) r uR)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'i'; 'n';
                               ' '; 'u'; 'n'; 'l'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n';
                               'g'; ' '; 'o'; 'f'; ' '; 'T'; 'R'; 'S'; ' '; 'w';
                               'h'; 'e'; 'n'; ' '; 's'; 'w'; 'i'; 't'; 'c'; 'h';
                               'i'; 'n'; 'g'; ' '; 'f'; 'r'; 'o'; 'm'; ' '; 't';
                               'h'; 'e'; ' '; 'T'; 'R'; 'S']
                            :: append
                                 (showTRS (sshow_laba _A2 _B2)
                                   (sshow_lista sshow_chara) r)
                                 (['t'; 'o'; ' '; 't'; 'h'; 'e'; ' '; 'T'; 'R';
                                    'S']
                                   :: append
(showTRS (sshow_laba _A2 _B2) (sshow_lista sshow_chara) uR) x))))
            (fun x ->
              catch (checkTRSProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1'])
                      uR m prfa)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 't'; 'h'; 'e'; ' '; 'u'; 'n';
                             'l'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n'; 'g']
                          :: indent xa))))
    | i, r, Zero_nat, Unlab (uR, prfa) ->
        error [['u'; 'n'; 'l'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n'; 'g'; ' '; 'n'; 'o';
                 't'; ' '; 'a'; 'l'; 'l'; 'o'; 'w'; 'e'; 'd'; ' '; 'f'; 'o';
                 'r'; ' '; 'l'; 'a'; 'b'; 'e'; 'l'; ' '; 'd'; 'e'; 'p'; 't';
                 'h'; ' '; '0']]
    | i, r, m, RisEmpty ->
        debug i ['R'; ' '; 'i'; 's'; ' '; 'e'; 'm'; 'p'; 't'; 'y']
          (checkB (null r)
            (append
              [append i
                 [':'; ' '; 'R'; ' '; 'i'; 's'; ' '; 'n'; 'o'; 't'; ' '; 'e';
                   'm'; 'p'; 't'; 'y'; ','; ' '; 'b'; 'u'; 't'; ' '; 'i'; 't';
                   ' '; 'i'; 's'; ' '; 't'; 'h'; 'e'; ' '; 'T'; 'R'; 'S']]
              (showTRS (sshow_laba _A2 _B2) (sshow_lista sshow_chara) r)))
    | i, r, m, Semlab (sli, l, lR, prfa) ->
        debug i ['S'; 'e'; 'm'; 'l'; 'a'; 'b']
          (bind (catch
                  (create_sem_lab_tt (_A1, _A2) (_B1, _B2)
                    (eq_lista eq_char, sshow_lista sshow_chara) sli l r lR)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h';
                               'e'; 'n'; ' '; 'a'; 'p'; 'p'; 'l'; 'y'; 'i'; 'n';
                               'g'; ' '; 's'; 'e'; 'm'; 'a'; 'n'; 't'; 'i'; 'c';
                               ' '; 'l'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n'; 'g']
                            :: x)))
            (fun x ->
              catch (checkTRSProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1'])
                      lR (Suc m) prfa)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 's'; 'e'; 'm'; 'a'; 'n'; 't';
                             'i'; 'c'; ' '; 'l'; 'a'; 'b'; 'e'; 'l'; 'i'; 'n';
                             'g']
                          :: indent xa))))
    | i, r, m, Rootlab (lR, prfa) ->
        debug i ['R'; 'o'; 'o'; 't'; 'l'; 'a'; 'b']
          (bind (catch (rl_tt (_A1, _A2) (_B1, _B2) r lR)
                  (fun x ->
                    error (append i
                             [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h';
                               'e'; 'n'; ' '; 'a'; 'p'; 'p'; 'l'; 'y'; 'i'; 'n';
                               'g'; ' '; 'r'; 'o'; 'o'; 't'; '-'; 'l'; 'a'; 'b';
                               'e'; 'l'; 'i'; 'n'; 'g']
                            :: x)))
            (fun x ->
              catch (checkTRSProof (_A1, _A2) (_B1, _B2) (append i ['.'; '1'])
                      lR (Suc m) prfa)
                (fun xa ->
                  error (append i
                           [':'; ' '; 'e'; 'r'; 'r'; 'o'; 'r'; ' '; 'b'; 'e';
                             'l'; 'o'; 'w'; ' '; 'r'; 'o'; 'o'; 't'; '-'; 'l';
                             'a'; 'b'; 'e'; 'l'; 'i'; 'n'; 'g']
                          :: indent xa))));;

let rec certifyCertProblem (_A1, _A2) (_B1, _B2)
  = function
    CertificationProblem (true, trs, Left prf) ->
      (match checkTRSProof (_A1, _A2) (_B1, _B2) ['1']
               (map (map_rule init_lab) trs) Zero_nat prf
         with Left err ->
           Error (showList
                   (['t'; 'e'; 'r'; 'm'; 'i'; 'n'; 'a'; 't'; 'i'; 'o'; 'n'; ' ';
                      'p'; 'r'; 'o'; 'o'; 'f'; ' '; 'n'; 'o'; 't'; ' '; 'a';
                      'c'; 'c'; 'e'; 'p'; 't'; 'e'; 'd']
                     :: err))
         | Right x -> Certified true)
    | CertificationProblem (false, trs, Right prfa) ->
        (match checkTRSDisproof (_A1, _A2)
                 (eq_lista eq_char, sshow_lista sshow_chara) ['1'] trs prfa
           with Left err ->
             Error (showList
                     (['n'; 'o'; 'n'; '-'; 't'; 'e'; 'r'; 'm'; 'i'; 'n'; 'a';
                        't'; 'i'; 'o'; 'n'; ' '; 'p'; 'r'; 'o'; 'o'; 'f'; ' ';
                        'n'; 'o'; 't'; ' '; 'a'; 'c'; 'c'; 'e'; 'p'; 't'; 'e';
                        'd']
                       :: err))
           | Right x -> Certified false)
    | CertificationProblem (false, va, Left v) ->
        Error ['N'; 'o'; 't'; ' '; 'a'; 'l'; 'l'; 'o'; 'w'; 'e'; 'd'; ' '; 't';
                'o'; ' '; 'g'; 'i'; 'v'; 'e'; ' '; 'a'; ' '; 't'; 'e'; 'r'; 'm';
                'i'; 'n'; 'a'; 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'r'; 'o'; 'o'; 'f';
                ' '; 'f'; 'o'; 'r'; ' '; 'a'; ' '; 'n'; 'o'; 'n'; '-'; 't'; 'e';
                'r'; 'm'; 'i'; 'n'; 'a'; 't'; 'i'; 'o'; 'n'; ' '; 'g'; 'o'; 'a';
                'l'; ' '; 'o'; 'r'; ' '; 'v'; 'i'; 'c'; 'e'; '-'; 'v'; 'e'; 'r';
                's'; 'a']
    | CertificationProblem (true, va, Right vc) ->
        Error ['N'; 'o'; 't'; ' '; 'a'; 'l'; 'l'; 'o'; 'w'; 'e'; 'd'; ' '; 't';
                'o'; ' '; 'g'; 'i'; 'v'; 'e'; ' '; 'a'; ' '; 't'; 'e'; 'r'; 'm';
                'i'; 'n'; 'a'; 't'; 'i'; 'o'; 'n'; ' '; 'p'; 'r'; 'o'; 'o'; 'f';
                ' '; 'f'; 'o'; 'r'; ' '; 'a'; ' '; 'n'; 'o'; 'n'; '-'; 't'; 'e';
                'r'; 'm'; 'i'; 'n'; 'a'; 't'; 'i'; 'o'; 'n'; ' '; 'g'; 'o'; 'a';
                'l'; ' '; 'o'; 'r'; ' '; 'v'; 'i'; 'c'; 'e'; '-'; 'v'; 'e'; 'r';
                's'; 'a'];;

let rec xml_certificationProblem _A _C
  (CertificationProblem (term, rls, prf)) =
    let inp =
      Xml (['i'; 'n'; 'p'; 'u'; 't'], [],
            [Xml (['t'; 'r'; 's'; 'I'; 'n'; 'p'; 'u'; 't'], [],
                   [Xml (['t'; 'r'; 's'], [], [xml_rules _A _C rls], None)],
                   None)],
            None)
      in
    let goal = (if isLeft prf then terminationGoal else nonTerminationGoal) in
    Xml (['p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'], [], [goal; inp], None);;

let rec eqWhiteSpace
  s1 s2 =
    eqop (eq_lista eq_char) (filter (fun c -> not (member eq_char c wspace)) s1)
      (filter (fun c -> not (member eq_char c wspace)) s2);;

let rec extract_goal_input
  s = (match search_len s ['<'; 'g'; 'o'; 'a'; 'l'; '>']
               (size_list ['<'; 'g'; 'o'; 'a'; 'l'; '>'])
         with None -> None
         | Some a ->
           let (x, mgoal) = a in
           (match search_len mgoal ['<'; '/'; 'g'; 'o'; 'a'; 'l'; '>']
                    (size_list ['<'; '/'; 'g'; 'o'; 'a'; 'l'; '>'])
              with None -> None
              | Some aa ->
                let (goal, s') = aa in
                (match search_len s' ['<'; 'i'; 'n'; 'p'; 'u'; 't'; '>']
                         (size_list ['<'; 'i'; 'n'; 'p'; 'u'; 't'; '>'])
                   with None -> None
                   | Some ab ->
                     let (xa, minput) = ab in
                     (match search_len minput
                              ['<'; '/'; 'i'; 'n'; 'p'; 'u'; 't'; '>']
                              (size_list
                                ['<'; '/'; 'i'; 'n'; 'p'; 'u'; 't'; '>'])
                        with None -> None
                        | Some ac ->
                          let (input, xb) = ac in
                          Some (concat
                                 [['<'; 'p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm'; '>'];
                                   ['<'; 'g'; 'o'; 'a'; 'l'; '>']; goal;
                                   ['<'; '/'; 'g'; 'o'; 'a'; 'l'; '>'];
                                   ['<'; 'i'; 'n'; 'p'; 'u'; 't'; '>']; input;
                                   ['<'; '/'; 'i'; 'n'; 'p'; 'u'; 't'; '>'];
                                   ['<'; '/'; 'p'; 'r'; 'o'; 'b'; 'l'; 'e'; 'm';
                                     '>']])))));;

let rec certifyProof
  problemString =
    (match parseCertProblem problemString
       with Left err ->
         Error (append
                 (append
                   ['e'; 'r'; 'r'; 'o'; 'r'; ' '; 'w'; 'h'; 'i'; 'l'; 'e'; ' ';
                     'p'; 'a'; 'r'; 's'; 'i'; 'n'; 'g']
                   newline)
                 err)
       | Right cp ->
         (match extract_goal_input problemString
            with None ->
              Error ['c'; 'o'; 'u'; 'l'; 'd'; ' '; 'n'; 'o'; 't'; ' '; 'f'; 'i';
                      'n'; 'd'; ' '; 'g'; 'o'; 'a'; 'l'; ' '; 'a'; 'n'; 'd';
                      ' '; 'i'; 'n'; 'p'; 'u'; 't'; ' '; 'i'; 'n'; ' '; 's';
                      't'; 'r'; 'i'; 'n'; 'g'; ' '; 'f'; 'o'; 'r'; ' '; 'c';
                      'o'; 'm'; 'p'; 'a'; 'r'; 'i'; 's'; 'o'; 'n'; ' '; 'w';
                      'i'; 't'; 'h'; ' '; 'p'; 'a'; 'r'; 's'; 'e'; 'd'; ' ';
                      'r'; 'e'; 's'; 'u'; 'l'; 't']
            | Some inpProblem ->
              (if eqWhiteSpace inpProblem
                    (sshow_xml
                      (xml_certificationProblem (sshow_lista sshow_chara)
                        (sshow_lista sshow_chara) cp))
                then certifyCertProblem
                       (eq_lista eq_char, sshow_lista sshow_chara)
                       (eq_lista eq_nata, sshow_lista sshow_nata) cp
                else Error (concat
                             [['p'; 'a'; 'r'; 's'; 'e'; 'd'; ' '; 'p'; 'r'; 'o';
                                'b'; 'l'; 'e'; 'm'; ' '; 'd'; 'o'; 'e'; 's';
                                ' '; 'n'; 'o'; 't'; ' '; 'c'; 'o'; 'r'; 'r';
                                'e'; 's'; 'p'; 'o'; 'n'; 'd'; ' '; 't'; 'o';
                                ' '; 'i'; 'n'; 'p'; 'u'; 't'];
                               newline; ['i'; 'n'; 'p'; 'u'; 't'; ':']; newline;
                               filter (fun c -> not (member eq_char c wspace))
                                 inpProblem;
                               newline; newline;
                               ['p'; 'a'; 'r'; 's'; 'e'; 'd'; ':']; newline;
                               filter (fun c -> not (member eq_char c wspace))
                                 (sshow_xml
                                   (xml_certificationProblem
                                     (sshow_lista sshow_chara)
                                     (sshow_lista sshow_chara) cp));
                               newline]))));;

end;; (*struct Ceta*)
